function [thhat,lamhat,Epsi]=adfilter(z,nnn,adg,adm,fflam,decfactor,P0,x0)

% ADFILTER filters data using model structure nnn
%
% function [thhat,lamhat,Epsi]=adfilter(z,nnn,adg,adm,fflam,decfactor,P0,x0)
%
% z=[y u]   output-input data
% nnn       model structure
% adg       Adaption gain, forgetting factor, step size or scaling of Q
%           in state space models.
% adm       Adaptation method: RLS, WLS, LMS, NLMS or KF
%           For state space model adm is always set to KF.
%           An intelligent defualt value between RLS, WLS and LMS
%           is based on the value of adg
% ff        forgetting factor used in RLS and/or to estimate lambda
% thhat     estimated parameter/state vector for each t, xhat(t+1|t) for KF
% lamhat    estimated noise variance for each t if fflam<1, otherwise 
%           for fflam=1 (default) a scalar batch estimate is computed.
% Epsi      the prediction errors for each t
% fflam     forgetting factor for estimating noise variance lambda
%           (default 1)
% decfactor Decimation factor for parametric modeling
% P0        Initial covariance for RLS and KF
% x0        Initial guess of x for KF
%
% The function will return the LS estimate when only two input arguments
% are provided

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 08-Aug-2003  $
  
%#
%#  
  
if nargin<6; decfactor=[]; end;
if nargin<5; fflam=[]; end;
if nargin<3; adg=1; end;

if isempty(decfactor); decfactor=1; end;
if isempty(fflam); fflam=1; end;

mtype=modeltype(nnn);
[N,nz]=size(z);

if mtype>4;   % State space model
  [n,x00,P00]=feval(nnn,0,0,0,0);
  d=n(1);
  nu=n(2);
  nv=n(3);
  nd=n(4);
  nth2=n(5);
  if nargin<7; 
    P=P00;
  else
    P=P0;  % user specified
    if size(P)==[1,1]
      P=P*eye(d);
    end
  end
  if nargin<8; 
    xhat=x00;
  else
    xhat=x0;  % user specified
    if size(xhat)==[1,1]
      xhat=zeros(d,1);
    end
  end
  ny=n(6);
  y=z(:,1:ny);
  u=z(:,ny+1:ny+nu);
  tstart=1;
  y=z(:,1:ny);
  u=z(:,ny+1:size(z,2));
  P=adg*P;
  Xhat=zeros(d,N); 
  lamhat=zeros(N,1);
  n=0;
  
  for t=1:N;
    [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,xhat,0,5);
    [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
    Q=Q*adg;
    L=P*C';
    S=R+C*L;
    den=inv(S);
    epsi=y(t,:).'-C*xhat;
    if ~isempty(u) & ~isempty(Du);
       epsi=epsi-Du*u(t,:)';
    end
    Epsi(t,:)=epsi.';
    K=L*den;
    xhat=xhat+K*epsi;
    xhat=A*xhat;
    Xhat(:,t)=xhat;
    if ~isempty(u) & ~isempty(Bu);
       xhat=xhat + Bu*u(t,:)';
    end
    P=A*(P-K*L')*A'+Bv*Q*Bv';
    P=.5*(P+P');
    if fflam<1;
      if t>1;
        lamhat(t)=fflam*lamhat(t-1)+(1-fflam)*epsi'*epsi;
      else
	lamhat(t)=epsi'*epsi;      
      end
    else
      n=n+1;
      if t>1;
	lamhat(t)=(n-1)/n*lamhat(t-1)+1/n*epsi'*epsi;      
      else
	lamhat(t)=epsi'*epsi;      
      end
    end
  end
  thhat=Xhat;

else        % Regression model
  [tstart,d]=z2phi(z,0,nnn);
  if nargin<7; 
    P=1000000*eye(d);
  else
    P=P0;
  end

  if nargin<4; 
    if adg<=1 & adg>0.1;
      adm='RLS'; 
    elseif adg<0.1;
      adm='LMS'; 
    else
      adm='WLS'; 
    end;
  end
  
  ff=1; R1=0; L=0;
  if strcmp(adm,'RLS')
    ff=adg; 
    if ff>1|ff<0; ff=0.98; disp('ff changed to 0.98'); end
  elseif strcmp(adm,'LMS') | strcmp(adm,'NLMS')
    mu=adg;
  elseif strcmp(adm,'WLS')
    L=adg;
    if L<1; L=10; disp('L changed to 10'); end
  elseif strcmp(adm,'KF')
    R1=adg;
    if size(R1,1)==1;
      R1=R1*eye(d);
    end;
  else
    disp('Not a valid method adm')
    disp('adm set to RLS with adg=0.98!')
    ff=0.98;
    adm='RLS';
  end;

  THhat=zeros(d,N/decfactor); 
  thhat=zeros(d,1);
  lamhat=zeros(N/decfactor,1);
  n=0;

  for t=tstart:N/decfactor;
    [phi,y]=z2phi(z,t*decfactor,nnn);
    epsi=y-phi'*thhat;
%    t,thhat,epsi,pause
    THhat(:,t)=thhat;
    Epsi(t,:)=epsi.';
    if strcmp(adm,'RLS') |strcmp(adm,'KF') |strcmp(adm,'WLS')
      M=P*phi;
      S=ff+phi'*M;
      den=inv(S);
      K=M*den;
      P=P-K*M'+R1;
      P=.5/ff*(P+P');
    elseif strcmp(adm,'LMS')
      K=mu*phi;
    elseif strcmp(adm,'NLMS')
      K=mu/(mu/1000+phi'*phi)*phi;
    end;
      
    thhat=thhat+K*epsi;

    if ff<1; 
      lamhat(t)=ff*lamhat(t-1)+(1-ff)*epsi'*epsi;
    else
      n=n+1;
      lamhat(t)=(n-1)/n*lamhat(t-1)+1/n*epsi'*epsi;      
    end
    if strcmp(adm,'WLS');         % Sliding window
      if t>=L+tstart;             % Remove influence of y(t-L)
        [phi,y]=z2phi(z,(t-L)*decfactor,nnn);
	M=P*phi;
        S=-ff+phi'*M;            % Note the minus sign
        den=inv(S);
        epsi=y-phi'*thhat;
        K=M*den;
        thhat=thhat+K*epsi;
        P=P-K*M'+R1;
        P=.5/ff*(P+P');
        lamhat(t)=lamhat(t)-1/n*epsi'*epsi;
	n=n-1;
      end
    end
  end;	
  thhat=THhat;
%y,K,phi,den,th,t,pause
end;


