function [xhat,yhat,P,Epsi,Epsin]=adkalman(z,nnn,adgain,kf,P0,x0)

% ADKALMAN implements various variants of the Kalman filter
%
% [xhat,yhat,P,Epsi,Epsin]=adkalman(z,nnn,adgain,kf,P0,x0)
%
% Model:   x(t+1) = A*x(t) + Bu*u + Bv*v(t)
%            y(t) = C*x(t) + Du*u + e(t)
%
% [y u]  Measurements
% nnn    State space model (see nnn)
% adgain Adaptation gain
% kf=[kfalg kftype] Algorithm and type (filter or predictor)
% kfalg= 1, stationary KF
%        2, time-varying KF (default)
%        3, square root filter
%        4, fixed interval KF smoother Rauch-Tung-Striebel
% kftype = 0 for filter (default) or 1 for predictor 
%
% P0     Initial covariance matrix (scalar accepted, gives scaled identity)
% x0     Initial guess of x for KF
% xhat   State estimate xhat(t|t) or xhat(t+1|t) (stacked to a thich matrix)
% P      State estimate covariance (stacked to a thin matrix)
% yhat   Output estimate
% Epsi   Innovation
% Epsin  Normalized innovation
%

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 08-Aug-2003  $

% kftype(2)<0 smoothing, >0 prediction  xhat(t+kftype(2)|t)
%        or k for k-step predictor

%#
%#  
  

if nargin<4; kf=[]; end
if nargin<3; adgain=1; end

if isempty(kf)==1; kf=[2 0]; end 
if length(kf)==1; kf=[kf 0]; end 
kfalg=kf(1); kftype=kf(2);

[N,nz]=size(z);
extmod=isstr(nnn);
if extmod==0; % Compute all state space matrices
  [A,B,C,D,R1,R2,P0,x0]=nnn2ss(nnn);
end;
if extmod==1; % Just initialize P0 and x0
  [n,x00,P00]=feval(nnn,0,0,0,0);
  nx=n(1);
  nu=n(2);
  nv=n(3);
  nd=n(4);
  nth2=n(5);
  ny=n(6);
  y=z(:,1:ny);
  u=z(:,ny+1:ny+nu);
  if nargin<5 | isempty(P0); 
    P0=P00;
  else  % user specified
    if size(P0)==[1,1]
      P0=P0*eye(nx);
    end
  end
  if nargin<6 | isempty(x0); 
    x0=x00;
  else  % user specified
    if size(x0)==[1,1]
      x0=zeros(nx,1);
    end
  end
  xhat=x0;
  P=P0;
end

if kfalg==1;
  [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,0,xhat,0,5);
  [C,Du,Dth,R]=feval(nnn,0,xhat,0,6);
  %[K,P,Qdum]=dlqe(A,Bv,C,adgain*Q,R);
  %[yhat,xhat]=dlsim(A-A*K*C,[K Bu],C,[zeros(nu,ny) Du],z);
  Pold=zeros(nx);
  P=eye(nx);
  n=0;
  while norm(P-Pold)>1e-10 % Stationary Riccati, nothing fancy
    n=n+1;
    Pold=P;
    P=A*(P-P*C'*inv(C*P*C'+R)*C*P)*A'+Bv*Q*Bv';
    P=0.5*(P+P');
  end
  Kbar=A*P*C'*inv(C*P*C'+R);
  xhat=zeros(nx,N+1);
  for t=1:N;
    xhat(:,t+1)=A*xhat(:,t);
    xhat(:,t+1)=xhat(:,t+1)+Kbar*(y(t,:).'-C*xhat(:,t+1));
  end 
  xhat=xhat(:,2:N+1);
  yhat=(C*xhat)';
  if kftype==1
    xhat=A*xhat;
    P=A*P*A'+adgain*Bv*Q*Bv';
    yhat=(C*xhat)';
  end
  if kftype>1
    xhat=A^kftype*xhat;
    P=A*P*A'+adgain*Bv*Q*Bv';  % Correct this!
    yhat=(C*xhat)';
  end
  Epsi=y-yhat;
end

if kfalg==2 
  xhatf=zeros(nx,N);
  xhatp=zeros(nx,N);
  Pf=zeros(N*nx,nx);
  Pp=zeros(N*nx,nx);
  yhatf=zeros(ny,N);
  yhatp=zeros(ny,N);
  xt=xhat;
  P=adgain*P;
  for t=1:N,
    if extmod==1;
       [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,xhat,0,5);
       [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
    end
    Q=Q*adgain;
    L=P*C';
    S=R+C*L;
    den=inv(S);
    epsi=y(t,:).'-C*xhat;
    if ~isempty(u) & ~isempty(Du);
       epsi=epsi-Du*u(t,:)';
    end
    Epsi(:,t)=epsi;
    Epsin(:,t)=sqrtm(den)*epsi;
    K=L*den;

    % Measurement update
    xhat=xhat+K*epsi;
    P=P-K*L';
    xhatf(:,t)=xhat;
    Pf(t*nx-nx+1:t*nx,:)=P;
    yhatf(:,t)=C*xhat;

    % Time update
    xhat=A*xhat;
    if ~isempty(u) & ~isempty(Bu);
       xhat=xhat + Bu*u(t,:)';
    end
    P=A*P*A'+Bv*Q*Bv';
    P=.5*(P+P');
    xhatp(:,t)=xhat;
    Pp(t*nx-nx+1:t*nx,:)=P;
    yhatp(:,t)=C*xhat;

  end
  if kftype==0
    xhat=xhatf;
    P=Pf;
    yhat=yhatf'; 
  elseif kftype==1
    xhat=xhatp;
    P=Pp;
    yhat=yhatp';
  end
end

if kfalg==3
  if length(P)==1; P=P*eye(nx); end;
  xhatmat=zeros(nx,N);
  Pmat=zeros(N*nx,nx);
  yhat=zeros(ny,N);
  Epsi=zeros(ny,N);
  Epsin=zeros(ny,N);
  P=adgain*P;
  Ph=sqrtm(P);

  if kftype==1
    for t=1:N,
      if extmod==1;
        [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,xhat,0,5);
        [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
      end
      Q=Q*adgain;

      qrmatrix = [sqrtm(R), C*Ph, zeros(ny,nv);...
                  zeros(nx,ny), A*Ph, Bv*sqrtm(Q)];
      [Qfact,Rfact]=qr(qrmatrix');
      Rfact=Rfact';
      Ph=Rfact(ny+1:ny+nx,ny+1:ny+nx);
      Sh=Rfact(1:ny,1:ny);
      Kh=Rfact(ny+1:ny+nx,1:ny);
%qrmatrix,Rfact,Ph,Sh,Kh,pause
      epsi=y(t,:).'-C*xhat;
      if ~isempty(u) & ~isempty(Du);
        epsi=epsi-Du*u(t,:).';
      end

      epsin=inv(Sh)*epsi;
      xhat=A*xhat+Kh*epsin;
      if ~isempty(u) & ~isempty(Bu);
        xhat=xhat + Bu*u(t,:).';
      end
      P=Ph*Ph';
      xhatmat(:,t)=xhat;
      Pmat(t*nx-nx+1:t*nx,:)=P;
      yhat(:,t)=C*xhat;
      Epsi(:,t)=epsi;
      Epsin(:,t)=epsin;
  end
  elseif kftype==0
    A=eye(nx);
    Bu=zeros(nx,nu);
    Bv=zeros(nx,nv);
    Q=eye(nv);

    for t=1:N,

      [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);  

      Q=Q*adgain;
      Qh=sqrtm(Q);
      qrmatrix = [sqrtm(R), C*A*Ph, C*Bv*Qh;...
                  zeros(nx,ny), A*Ph, Bv*Qh];
      [Qfact,Rfact]=qr(qrmatrix');
      Rfact=Rfact';
      Ph=Rfact(ny+1:ny+nx,ny+1:ny+nx);
      Sh=Rfact(1:ny,1:ny);
      Kh=Rfact(ny+1:ny+nx,1:ny);

      epsi=y(t,:).'-C*xhat;  % Include time update, xhat filtered estimate
      if ~isempty(u) & ~isempty(Du);
        epsi=epsi-Du*u(t,:)';
      end

      epsin=inv(Sh)*epsi;
      xhat=xhat+Kh*epsin; % xhat(t|t)=xhat(t|t-1)+K*epsi
      P=Ph*Ph'; 
      xhatmat(:,t)=xhat;
      Pmat(t*nx-nx+1:t*nx,:)=P;
      yhat(:,t)=C*xhat;
      Epsi(:,t)=epsi;
      Epsin(:,t)=epsin;
      [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,xhat,0,5); % State matrices for next iteration
      % Time update
      xhat=A*xhat;     %  xhat(t+1|t)
      if ~isempty(u) & ~isempty(Bu);
        xhat=xhat + Bu*u(t,:)';
      end
    end

  end
  xhat=xhatmat;
  P=Pmat;

end

if kfalg==4  % Smoothing
  % Forward recursion
  [xhatf,yhatf,Pf]=adkalman(z,nnn,adgain,[2 0],P0,x0);
  [xhatp,yhatp,Pp]=adkalman(z,nnn,adgain,[2 1],P0,x0);
  xhat=xhatf;
  P=Pf;
  xstmp=xhatf(:,N);
  Pstmp=Pf(N*nx-nx+1:N*nx,:);
  % Backward recursion
  for t=N-1:-1:1;
    xftmp=xhatf(:,t);
    xptmp=xhatp(:,t);
    Pptmp=Pp(t*nx-nx+1:t*nx,:);
    Pftmp=Pf(t*nx-nx+1:t*nx,:);

    A=feval(nnn,t,xstmp,0,5);
    C=feval(nnn,t,xstmp,0,6);
    % x(t|N)=x(t|t)+P(t|t)A'(t)P(t+1|t) (x(t+1|N)-x(t+1|t))  RTS formulas
    xstmp=xftmp + Pftmp*A'*inv(Pptmp)*(xstmp-xptmp);
    Pstmp=Pftmp + Pftmp*A'*inv(Pptmp)*(Pstmp-Pptmp)*inv(Pptmp)*A*Pftmp;

    xhat(:,t)=xstmp;
    yhat(:,t)=C*xstmp;
    Pp(t*nx-nx+1:t*nx,:)=Pstmp;
  end
  P=Pp;
end


