function [threc,uhat,epsi]=blindeq(y,nnn,adg,adm,th0);
 
% Blind equalization where only the output is measured
%
% [threc,uhat]=blindeq(y,nnn,adg,adm);
%
% Assumes that the adaptive equalizer is tuned such that
% its output z is approximately equal to the channel input u
%
% y       Output from channel
% uhat    Estimated input
% adg     adaptation gain in a stochastic gradient (LMS like) method
% adm=1   Loss function E[ (1-z^2)^2 ] (modulus restoral, Godard) (default)
% adm=2   Loss function E[ (sign(z)-z)^2 ] (decision feedback, Sato)
% th0     Initial estimate. Quite critical for convergence.
%
% Limitations: 
%  1. Only FIR equalizers
%  2. Only input alphabets [-1,+1]
%  3. Only scalar channels
%
% References: 
% D.N. Godard, Self-Recovering Equalization and Carrier Tracking 
% in Two-Dimensional Data Communication Systems, IEEE Tr. Com.
% 1980, pages 1867-1875
% R.W. Lucky, Techniques for Adaptive Equalization of Digital Communiqation Systems,
% Bell System Technical Journal, volume=45, 1966, pages 255-286
%

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 08-Aug-2003  $

if size(nnn)==[1 1]
  n=nnn;
elseif size(nnn)==[1 3]
  n=nnn(2);
  if nnn(3)~=0; disp('Warning: no time delays are allowed!'), end
else
  error('Only FIR models are allowed: let nnn be a scalar!')
end

if nargin<5; 
  if n/2~=round(n/2);  % odd n
    th=[zeros(floor(n/2),1);1;zeros(floor(n/2),1)];
  else
    th=[zeros(n/2-1,1);1;zeros(n/2,1)];
  end
else, 
  th=th0; 
end
if nargin<4; adm=1; end

N=length(y);
threc=zeros(n,N);
uhat=zeros(N,1);
z=zeros(N,1);


for t=n:N;
  phi=y(t-n+1:t);
  z(t)=phi'*th;
  if adm==1;
    epsi(t)=sign(z(t))-z(t);     % Sato
  else
    epsi(t)=z(t)*(1-z(t)^2);    % Godard
  end
  th=th+adg*phi*epsi(t);
  uhat(t)=sign(phi'*th);
  threc(:,t)=th;
end;

