function [theta,uhat]=blindeqM(z,nnn,alphabet,R,M,Q,p0,dsp)

% Maximum likelihood sequence detection
%
%   function [thhat,uhat]=blindeqM(y,nnn,alphabet,R,M,Q,p0,dsp)
%   
% Estimates the unknown input u(t)= +/- 1 and the unknown channel
% described by a MIMO FIR process.
%
% Inputs  
%   y   received signal, u (optional) digital input
%   nn  ny times nu matrix, where ny is the number of outputs and nu 
%       the number of input. The elements denote the corresponding FIR orders.
%   This implementation only work for nnn=nb*ones(ny,nu)
%   alphabet  Finite input alphabet (default [-1 1])
%   q  Q=eye(N)*q in state space model	default q=0
%   r  R=eye(ny)*r  	      		default r=0.1	
%   p0  P=eye(N)*p0   			default p0=10
%   M   Number of models (even!) 	default M=64
%   dsp  Display some information if dsp ~=0 default dsp=0
% Outputs
%   theta Estimated parameter vector as a function of time
%   uhat  Estimated input sequencyu
%
% Limitation
% 1. Works only for FIR with nnn=nb*I, i.e. all FIR transfer functions have the
%   same degree. Easy to modify to general case!
%
% Reference:
%   F. Gustafsson and B. Wahlberg, Blind equalization by direct examination 
%   of the input sequences, IEEE Tr. Com., 2213-2222, 1995

% MIMO version by Bo Wahlberg
% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 08-Aug-2003  $

[ny,nu]=size(nnn);
[N,m]=size(z);
L=ny;   % Number of channels, old variable
y=z(:,1:ny);
if m-ny>0; u=z(:,ny+1:m); end
n=sum(sum(nnn));   % the total number of FIR parameters to be estimated.
if nargin<8 dsp=0;end;
if nargin<7 p0=10;end;
if nargin<6 Q=zeros(n,n); else Q=eye(n)*Q; end;
if nargin<5 M=64; end;
if nargin<4 R=0.1 ;end;  R=eye(L)*R;
if nargin<3 alphabet=[] ;end; 
if isempty(alphabet); alphabet=[-1 1]; end
if length(Q)==1;Q=Q*eye(n);end

nb=max(max(nnn));
nstart=nb;
ustart=1;
Mm=1;
% Construct the 2^nu possible split input sequences
i=sqrt(-1);
alphabet=[-1 1 -i i];
alphabet=[-1 1];
Usplit=alphabet;
nsymb=length(alphabet);
nusplit=1;
for i=1:nu-1;
  Usplit=[kron(Usplit,[1 1]);kron(ones(1,size(Usplit,2)),[1 -1])];
%  nusplit=nusplit*nsymb;
%  Usplit=kron(ones(1,nsymb),Usplit);
%  Usplit(i,:)=kron(alphabet,ones(1,nusplit));
end
%Usplit
% Initiate the inputs, assuming that all input sequences start with 1
Utest= ones(nu,1);

for i=ustart+1:nb;
  Utest_old=Utest;
  t=size(Utest,1)/nu;
  Utest= [kron(Utest_old(1:t,:),ones(1,size(Usplit,2)));
  kron(ones(1,size(Utest_old,2)),Usplit(1,:))];
  for j=2:nu;
    Utest= [Utest;kron(Utest_old((j-1)*t+1:j*t,:),ones(1,size(Usplit,2)));
    kron(ones(1,size(Utest_old,2)),Usplit(j,:))];
  end
end
% Mm equals number of start sequences
Mm=size(Utest,2);
Utest=[Utest zeros(nu*nb,M-Mm)];
if Mm>M;
  error(['M is too small. It must be at least ',num2str(Mm)]);
end
lifelength=0; % Garanterad livslangd hos enskilda modeller
alfa=[1/Mm*ones(1,Mm) zeros(1,M-Mm)];

theta=zeros(n,N);
th=zeros(n,M);
% Multi-channel modification
P=eye(n)*p0;
for i=1:M-1,P=[P eye(n)*p0];end;

if dsp;clc,home,disp('    = i ');end

for i=nstart:N; 
  for j=1:M,
%Utest(i:-1:i-nb+1,:)
   if alfa(j)>0,
     phi=Utest(i:-1:i-nb+1,j);
     for k=2:nu
      phi=[phi; Utest(i*k:-1:i*k-nb+1,j)];
     end
     phi=kron(eye(L),phi); % This has to be modified for the general FIR case
     phi=conj(phi);
%phi,nb,nu,L
     Pj=P(:,n*(j-1)+1:n*(j-1)+n);
     den=(R+phi'*Pj*phi);
     invden=inv(den); 
     epsi=y(i,:)'-phi'*th(:,j);  
     alfa(j)=alfa(j)/sqrt(det(den))*exp(-0.5*epsi'*invden*epsi);
     Kj=Pj*phi*invden; 
     th(:,j)=th(:,j)+Kj*epsi;
     Pj=Pj-Kj*phi'*Pj'+Q; 
     P(:,n*(j-1)+1:n*(j-1)+n)=Pj; 
   end;
  end
  [dum ind]=sort(alfa);
  ind1=ind(M:-1:M-M/2^nu+1);
  % Keep M/2^nu branches
  Pp=[];
  for k=1:M/2^nu;
    Pp=[Pp P(:,n*(ind1(k)-1)+1:n*(ind1(k)-1)+n)];
  end;
  % Expand to M input sequences at time t+1
  Utest_old=Utest(:,ind1);
  t=size(Utest,1)/nu;
  Utest= [kron(Utest_old(1:t,:),ones(1,size(Usplit,2)));
              kron(ones(1,size(Utest_old,2)),Usplit(1,:))];
  for j=2:nu;
    Utest= [Utest;kron(Utest_old((j-1)*t+1:j*t,:),ones(1,size(Usplit,2)));
    kron(ones(1,size(Utest_old,2)),Usplit(j,:))];
  end
  P=[];
  for k=1:M/2^nu,
    P=[P kron(ones(1,size(Usplit,2)),Pp(:,n*(k-1)+1:n*(k-1)+n))];
  end
  th=kron(th(:,ind1),ones(1,size(Usplit,2)));
%  alfa_old=alfa(ind1)+[1e-3*max(alfa) zeros(1,length(ind1)-1)];
  alfa_old=alfa(ind1);
  alfa=kron(alfa(ind1),ones(1,size(Usplit,2)));
  alfa=1/sum(alfa)*alfa;
  if dsp;
    home
    disp(num2str(i));
    disp(cdnum2str(epsi)),
    disp(cdnum2str((ind1))),
    disp(cdnum2str(alfa(ind1))),
    disp(cdnum2str(th(:,i)'));
    pause
  end
  theta(:,i)=th(:,1); % the most probably instead of weighted below
  %theta(:,i)=th*alfa';
end
%if exist('u');  % use u2ber to correct modulus
%  [dum jmax]=max(alfa);
%  for i=1:nu,
%    udum(:,i)=Utest_old(N*(i-1)+1:N*(i-1)+N,jmax);
%    for j=1:nu,
%      ber_sym(j)=sum(abs(udum(ustart:N,i)-u(ustart:N,j)))/2/(N-ustart+1);
%    end
%    [ber(i),ind]=min([ber_sym 1-ber_sym]);
%    if ind>nu;  % reflect u
%      sign=-1;
%      ind=ind-nu;
%    else 
%      sign=1;
%    end
%    uhat(:,ind)=sign*udum(:,i);
%    thhatout(ind,:)=sign*theta(i,:);
%  end;
%  theta=thhatout;
%%   if ber>0.5; ber=1-ber; end;
%else;
  [dum jmax]=max(alfa);
  for i=1:nu,
    uhat(:,i)=Utest_old(N*(i-1)+1:N*(i-1)+N,jmax);
  end
%  ber=-1; % use u2ber
%end;





