/*
 *========================================================================
 *
 *detect1
 *
 *MEX-file implementing:
 *detect1 - detects changes using a one-filter approach
 *
 *Jonas Elbornsson November 1998
 *========================================================================
 */

#define S_FUNCTION_NAME detect1 /*defines the name of the S-function*/
#define S_FUNCTION_LEVEL 2     /*defines the level of the S-function*/

#include "simstruc.h"
#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif
#include <math.h>
#include "stop.h"
#include "matrixop.h"
#include "model.h"
#include "ztophi.h"
#include "inv.h"
#include "kalmanfilt.h"

#define NNN                ssGetSFcnParam(S,0)    /*model structure*/   
#define DM                 ssGetSFcnParam(S,1)    /*distance measure*/
#define SR                 ssGetSFcnParam(S,2)    /*stopping rule*/
#define LAMBDA             ssGetSFcnParam(S,3)    /*noise variance*/
#define TS                 ssGetSFcnParam(S,4)    /*sample time*/
#define IN                 ssGetSFcnParam(S,5)    /*number of input signals*/
#define OUT                ssGetSFcnParam(S,6)    /*number of output signals*/
const char *nnnfile="nnnfile";
  
/*
 *====================================================================
 *mdlInitializeSizes - initialize the sizes array
 *
 *The sizes array is used by SIMULINK to determine the S-function
 *block's characteristics (number of inputs, outputs, states, etc.)
 *
 *====================================================================
 */

static void mdlInitializeSizes(SimStruct *S)
{
  int_T   nrow   = mxGetM(NNN);           /*number of columns in NNN*/
  int_T   ncol   = mxGetN(NNN);           /*number of rows in NNN*/
  real_T  *model = mxGetPr(NNN);          /*pointer to NNN*/
  int_T   mtype;
  int_T   nstates;                        /*number of states*/
  real_T   *nu     = mxGetPr(IN);
  real_T   *ny     = mxGetPr(OUT);
  
  /*determine the type of the specified model*/

  mtype = modeltype(model,nrow,ncol);
  nstates = numberofstates(model,mtype,nnnfile);

  ssSetNumContStates(             S,0);           /*Number of continuous states*/
  ssSetNumDiscStates(S,nstates+nstates*nstates+3); /*Number of discrete states*/

  if (!ssSetNumInputPorts(        S,2)) return;   /*Number of inputports*/
  ssSetInputPortWidth(           S,0,*nu);         /*Width of inputport 0*/
  ssSetInputPortWidth(           S,1,*ny);         /*Width of inputport 1*/ 
  ssSetInputPortDirectFeedThrough(S,0,1);         /*Direct feedthrough flag*/
  ssSetInputPortDirectFeedThrough(S,1,1);

  if (!ssSetNumOutputPorts(       S,3)) return;   /*Number of outputports*/ 
  ssSetOutputPortWidth(           S,0,1);         /*Width of outputport 0*/
  ssSetOutputPortWidth(           S,1,2);         /*Width of outputport 1*/
  ssSetOutputPortWidth(     S,2,nstates);         /*Width of outputport 2*/


  ssSetNumSampleTimes(            S,1);           /*Number of sample times*/

  ssSetNumSFcnParams(             S,7);           /*Number of input arguments*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) 
    {
        /* Return if number of expected != number of actual parameters */
        return;
    }
  if (mtype == 5)
    ssSetNumRWork(S,8*nstates*nstates+(*nu+*ny)*nstates+(*nu)*(*ny)+nstates+*ny);/*Number of Rwork elements*/
  else if (mtype == 4)
    ssSetNumRWork( S,5+model[2]+5*nstates*nstates+3*nstates*(*ny+*nu)+2*(*ny)*(*ny)+*ny+(*nu)*(*ny));
  else
    ssSetNumRWork(S,5+5*nstates*nstates+3*nstates*(*ny+*nu)+2*(*ny)*(*ny)+*ny+(*nu)*(*ny)); 
  ssSetNumIWork(                  S,6);           /*Number of integer workvector elements*/
  ssSetNumPWork(                  S,0);           /*Number of pointer workvector elements*/

}

 
static void mdlInitializeSampleTimes(SimStruct *S)
{
  ssSetSampleTime(S,0,mxGetPr(TS)[0]);
  ssSetOffsetTime(S,0,0);
}

#define MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#if defined(MDL_INITIALIZE_CONDITIONS)
static void mdlInitializeConditions(SimStruct *S)
{
  int_T   nrow   = mxGetM(NNN);           /*number of columns in NNN*/
  int_T   ncol   = mxGetN(NNN);           /*number of rows in NNN*/
  real_T  *model = mxGetPr(NNN);          /*pointer to NNN*/
  real_T  *dstates = ssGetRealDiscStates(S); /*pointer to discrete states*/
  int_T   mtype,tstart,noiseest,nstates;
  real_T lambda0 = mxGetPr(LAMBDA)[0];
  real_T lambda;
  int_T *iwork = ssGetIWork(S);

  if (lambda0 == -1)
    {
      lambda = 1;
      noiseest = 1;
    }
  else
    {
      noiseest =0;
      lambda = lambda0;
    }

  /*determine the type of the specified model*/

  mtype = modeltype(model,nrow,ncol);
  nstates = numberofstates(model,mtype,nnnfile);
  tstart = initstates(1,model,nstates,mtype,S,nnnfile);


  dstates[nstates+nstates*nstates]=0;              /*initialize the decision function gt*/
  dstates[nstates+nstates*nstates+1]=0;
  dstates[nstates+nstates*nstates+2]=lambda;              /*initialize lambda*/

  iwork[0] = mtype;                                /*modeltype*/
  iwork[1] = nstates;                              /*number of states*/
  iwork[2] = noiseest;                             /*estimate the noise variance?*/   
  iwork[3] = 0;                                    /*time since last detected change*/ 
  iwork[4] = 0;                                    /*time*/
  iwork[5] = tstart;                               /*startingtime*/

}
#endif

static void mdlOutputs(SimStruct *S,int_T tid)
{ 
  real_T *model = mxGetPr(NNN);
  InputRealPtrsType u = ssGetInputPortRealSignalPtrs(S,0);
  InputRealPtrsType y = ssGetInputPortRealSignalPtrs(S,1);
  real_T *rwork = ssGetRWork(S);
  int_T ny = ssGetInputPortWidth(S,1);
  int_T nu = ssGetInputPortWidth(S,0);
  int_T nstates = ssGetIWork(S)[1];
  int_T mtype = ssGetIWork(S)[0];
  int_T noiseest = ssGetIWork(S)[2];
  int_T *n = ssGetIWork(S)+3;
  int_T dm = mxGetPr(DM)[0];
  real_T *sr = mxGetPr(SR);
  real_T *A,*B,*C,*D,*R1,*R2,*Bu,*Bv,*Bth,*Bd,*Du,*Dth;
  real_T *den,*K,*eps,*pc,*xhat,*phi,*GT,*threc,sigma,alfa,beta;
  real_T *temp3 = mxCalloc(nstates*nstates,sizeof(real_T));
  real_T *temp2 = mxCalloc(nstates*nstates,sizeof(real_T));
  real_T *temp1 = mxCalloc(nstates*nstates,sizeof(real_T));
  int_T i,j,k;
  int_T tstart = ssGetIWork(S)[5];
  real_T *th = ssGetRealDiscStates(S);
  real_T *P = th+nstates;
  real_T *gt = P+nstates*nstates;
  real_T *lambda = gt+2;
  real_T alarm;
  real_T *temp;
  mxArray *rhs[4],*lhs[6];
  real_T *n1=mxCalloc(6,sizeof(real_T));
  int_T tempn,tempn2,tempn3;


  threc = ssGetOutputPortRealSignal(S,2); 

  n[0]++;
  n[1]++;

  if (mtype == 5)  /*state space*/
    {


      rhs[0]=mxCreateDoubleMatrix(1,1,mxREAL);
      rhs[1]=mxCreateDoubleMatrix(1,1,mxREAL);
      rhs[2]=mxCreateDoubleMatrix(1,1,mxREAL);
      rhs[3]=mxCreateDoubleMatrix(1,1,mxREAL);
      lhs[0]=mxCreateDoubleMatrix(1,6,mxREAL);
      mexCallMATLAB(1,lhs,4,rhs,nnnfile);
      for (i=0;i<6;i++)
	n1[i]=mxGetPr(lhs[0])[i];
      /*for easy access of the matrices*/
 

      rhs[0]=mxCreateDoubleMatrix(1,1,mxREAL);
      rhs[1]=mxCreateDoubleMatrix(nstates,1,mxREAL);
      rhs[2]=mxCreateDoubleMatrix(n1[1]+n1[3]+n1[4]+1,1,mxREAL);
      rhs[3]=mxCreateDoubleMatrix(1,1,mxREAL);
      lhs[0]=mxCreateDoubleMatrix(nstates,nstates,mxREAL);
      lhs[1]=mxCreateDoubleMatrix(nstates,n1[1],mxREAL);
      lhs[2]=mxCreateDoubleMatrix(nstates,n1[2],mxREAL);
      lhs[3]=mxCreateDoubleMatrix(nstates,n1[3],mxREAL);
      lhs[4]=mxCreateDoubleMatrix(nstates,n1[4],mxREAL);
      lhs[5]=mxCreateDoubleMatrix(nstates,nstates,mxREAL);
      mxGetPr(rhs[0])[0]=n[0]; /*obs! n=tid*/
      for (i=0;i<nstates;i++)
	mxGetPr(rhs[1])[i]=th[i];
      for (i=0;i<n1[1]+n1[2]+n1[4]+1;i++)
	mxGetPr(rhs[2])[i]=*u[i];
      mxGetPr(rhs[3])[0]=5;
      mexCallMATLAB(6,lhs,4,rhs,nnnfile);

      temp=mxGetPr(lhs[0]);                    /*A*/
      A=mxCalloc(nstates*nstates,sizeof(real_T));
      for (i = 0; i < nstates*nstates; i++)                                                    
	A[i] = temp[i];

      temp=mxGetPr(lhs[1]);                   /*Bu*/
      Bu=mxCalloc(nstates*n1[1],sizeof(real_T));
      for (i = 0; i < n1[1]*nstates; i++)                                                             
	Bu[i]=temp[i];

      temp=mxGetPr(lhs[2]);                   /*Bv*/
      Bv=mxCalloc(nstates*n1[2],sizeof(real_T));
      for (i = 0; i < nstates*n1[2]; i++)                                                    
	Bv[i] = temp[i];

      temp=mxGetPr(lhs[3]);                   /*Bd*/
      Bd=mxCalloc(nstates*n1[3],sizeof(real_T));
      for (i = 0; i < nstates*n1[3]; i++)                                                    
	Bd[i] = temp[i];

      temp=mxGetPr(lhs[4]);                   /*Bth*/
      Bth=mxCalloc(nstates*n1[4],sizeof(real_T));
      for (i = 0; i < nstates*n1[4]; i++)                                                    
	Bth[i] = temp[i];

      temp=mxGetPr(lhs[5]);                   /*Q*/
      R1=mxCalloc(nstates*nstates,sizeof(real_T));
      for (i = 0; i < nstates*nstates; i++)                                              
	R1[i] = temp[i];

      for (i=0;i<6;i++)
	mxDestroyArray(lhs[i]);

      lhs[0]=mxCreateDoubleMatrix(n1[5],nstates,mxREAL);
      lhs[1]=mxCreateDoubleMatrix(n1[5],n1[1],mxREAL);
      lhs[2]=mxCreateDoubleMatrix(n1[5],n1[4],mxREAL);
      lhs[3]=mxCreateDoubleMatrix(n1[5],n1[5],mxREAL);
      mxGetPr(rhs[3])[0]=6;
      mexCallMATLAB(4,lhs,4,rhs,nnnfile);

      temp=mxGetPr(lhs[0]);                   /*C*/
      C=mxCalloc(n1[5]*nstates,sizeof(real_T));
      for (i = 0; i < n1[5]*nstates; i++)                                                    
	C[i] = temp[i];

      temp=mxGetPr(lhs[1]);                   /*Du*/
      Du=mxCalloc(n1[5]*n1[1],sizeof(real_T));
      for (i = 0; i < n1[5]*n1[1]; i++)                                                    
	Du[i] = temp[i];

      temp=mxGetPr(lhs[2]);                   /*Dth*/
      Dth=mxCalloc(n1[5]*n1[4],sizeof(real_T));
      for (i = 0; i < n1[5]*n1[4]; i++)                                                    
	Dth[i] = temp[i];

      temp=mxGetPr(lhs[3]);                   /*R*/
      R2=mxCalloc(n1[5]*n1[5],sizeof(real_T));
      for (i = 0; i < n1[5]*n1[5]; i++)                                                    
	R2[i] = temp[i];


      tempn=nu;
      tempn2=n1[1];
      B=mxCalloc(nstates*(n1[1]+n1[2]+n1[3]+n1[4]),sizeof(real_T));
      for (i=0;i<n1[1];i++)
	for (j=0;j<nstates;j++)
	  B[i*tempn+j]=Bu[i*tempn2+j];

      tempn=nu;
      tempn3=n1[2];
      for (i=0;i<n1[2];i++)
	for (j=0;j<nstates;j++)
	  B[i*tempn+tempn2+j]=Bv[i*tempn3+j];

      tempn2=n1[1]+n1[2];
      tempn3=n1[3];
      for (i=0;i<n1[3];i++)
	for (j=0;j<nstates;j++)
	  B[i*tempn+tempn2+j]=Bd[i*tempn3+j];

      tempn2=n1[1]+n1[2]+n1[3];
      tempn3=n1[4];
      for (i=0;i<n1[4];i++)
	for (j=0;j<nstates;j++)
	  B[i*tempn+tempn2+j]=Bth[i*tempn3+j];

      D=mxCalloc(ny*(n1[1]+n1[4]),sizeof(real_T));
      for (i=0;i<n1[1]*ny;i++)
	D[i]=Du[i];

      tempn2=n1[1]*ny;
      for (i=0;i<n1[4]*ny;i++)
	D[tempn2+i]=Dth[i];

      den = rwork;
      K = den + nstates*nstates;
      eps = K+nstates*nstates;
      pc = eps+ny;
      xhat=pc+nstates*nstates;

      kalman(den,A,B,C,D,R1,R2,P,th,eps,lambda[0],S,nstates);
      for (i=0; i<nstates; i++)
	threc[i] = th[i];
    }

  else
    {
      phi = ztophi(S,nstates,mtype,model);

      den = phi + nstates*ny;
      K = den+ny*ny;
      eps = K +nstates*ny;
      R1 = eps+nstates*nstates;
      R2 = R1+nstates*nstates;
      A = R2+ny*ny;
      B = A+nstates*nstates;
      C = B+nstates*nu;
      D = C+nstates*ny;
      for (i=0; i<nstates*nstates; i++)
	R1[i]=0;
      for (i=0; i<ny*ny; i++)
	{
	  R2[i]=0;
	  for (i=0; i<ny*ny; i=i+ny+1)
	    R2[i]=1;
	}
      for (i=0; i<nstates*nstates; i++)
	{
	  A[i]=0;
	  for (i=0; i<nstates*nstates; i=i+nstates+1)
	    A[i]=1;
	}
      for (i=0; i<nstates*nu; i++)
	B[i]=0;
      for (i=0; i<ny*nu; i++)
	D[i]=0;
      C = transpose(phi,nstates,ny,C);
      
      if (n[1]>tstart)
	{
	  kalman(den,A,B,C,D,R1,R2,P,th,eps,lambda[0],S,nstates);

	  for (i=0; i<nstates; i++)
	    threc[i] = th[i];
	}
    }
  
  if (noiseest==1)
    lambda[0] =(lambda[0])*(n[0]-1)/(n[0])+(matrixmult(eps,1,ny,eps,1,temp1)[0])/(n[0]);
  
  if (dm==1)
    {
      
    }
  else if (dm==2)
    {
      temp3 = matrixmult(den,ny,ny,eps,1,temp3);
      temp1 = matrixmult(eps,1,ny,temp3,1,temp1);
    }
  
  stop(gt,temp1[0],sr);

  if ((gt[0]>sr[1]) || (gt[1]>sr[1]))
    {
      alarm=1;
      gt[0]=0;
      gt[1]=0;
      n[0]=0;
      for (i=0; i<nstates; i++)
	th[i]=0;
      for (i=0; i<nstates*nstates; i++)
	P[i]=0;
      for (i=0; i<nstates*nstates; i=i+nstates+1)
	P[i]=100000*lambda[0];
    }
  else
    alarm=0;
  
  ssGetOutputPortRealSignal(S,0)[0] = alarm;
  GT = ssGetOutputPortRealSignal(S,1);
  GT[0] = gt[0];
  GT[1] = gt[1];
    
}


static void mdlTerminate(SimStruct *S)
{
  return;
}

#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif
