/*
 *========================================================================
 *
 *detect2
 *
 *MEX-file implementing:
 *detect2 - detects changes using a two-filter approach
 *
 *Jonas Elbornsson July 1998
 *========================================================================
 */

#define S_FUNCTION_NAME detect2 /*defines the name of the S-function*/
#define S_FUNCTION_LEVEL 2     /*defines the level of the S-function*/

#include "simstruc.h"
#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif
#include <math.h>
#include "stop.h"
#include "matrixop.h"
#include "model.h"
#include "ztophi.h"
#include "rls.h"
#include "inv.h"
#include "kalmanfilt.h"

#define NNN                ssGetSFcnParam(S,0)    /*model structure*/   
#define DM                 ssGetSFcnParam(S,1)    /*distance measure*/
#define SR                 ssGetSFcnParam(S,2)    /*stopping rule*/
#define L                  ssGetSFcnParam(S,3)    /*length of sliding window*/
#define LAMBDA             ssGetSFcnParam(S,4)    /*noise variance*/
#define TS                 ssGetSFcnParam(S,5)    /*sample time*/
#define IN                 ssGetSFcnParam(S,6)    
#define OUT                ssGetSFcnParam(S,7)

const char *nnnfile="nnnfile";

/*
 *====================================================================
 *mdlInitializeSizes - initialize the sizes array
 *
 *The sizes array is used by SIMULINK to determine the S-function
 *block's characteristics (number of inputs, outputs, states, etc.)
 *
 *====================================================================
 */

static void mdlInitializeSizes(SimStruct *S)
{
  int_T   nrow   = mxGetM(NNN);           /*number of columns in NNN*/
  int_T   ncol   = mxGetN(NNN);           /*number of rows in NNN*/
  real_T  *model = mxGetPr(NNN);          /*pointer to NNN*/
  int_T   mtype;                          /*the model type*/
  int_T   nstates;                        /*number of states*/
  int_T   winlength=mxGetPr(L)[0];
  real_T   *nu     = mxGetPr(IN);
  real_T   *ny     = mxGetPr(OUT);
  
  /*determine the type of the specified model*/

  mtype = modeltype(model,nrow,ncol);
  nstates = numberofstates(model,mtype,nnnfile);

  ssSetNumContStates(             S,0);           /*Number of continuous states*/
  /*ssSetNumDiscStates(S,nstates+nstates*nstates+3);*/  /*Number of discrete states*/

  if (!ssSetNumInputPorts(        S,2)) return;   /*Number of inputports*/
  ssSetInputPortWidth(            S,0,*nu);         /*Width of inputport 0*/
  ssSetInputPortWidth(            S,1,*ny);         /*Width of inputport 1*/ 
  ssSetInputPortDirectFeedThrough(S,0,1);         /*Direct feedthrough flag*/
  ssSetInputPortDirectFeedThrough(S,1,1);

  if (!ssSetNumOutputPorts(       S,3)) return;   /*Number of outputports*/ 
  ssSetOutputPortWidth(           S,0,1);         /*Width of outputport 0*/
  ssSetOutputPortWidth(           S,1,2);         /*Width of outputport 1*/
  ssSetOutputPortWidth(     S,2,nstates);         /*Width of outputport 2*/


  ssSetNumSampleTimes(            S,1);           /*Number of sample times*/

  ssSetNumSFcnParams(             S,8);           /*Number of input arguments*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) 
    {
        /* Return if number of expected != number of actual parameters */
        return;
    }
  if (mtype == 5)
    ssSetNumRWork(S,nrow*ncol+8*nstates*nstates+3*nstates);/*Number of Rworkvector elements*/
  else if (mtype == 4)
    ssSetNumRWork( S,model[2]+(winlength+3)*nstates+2*winlength+2*nstates*nstates+11);
  else
    ssSetNumRWork(S,(winlength+3)*nstates+2*winlength+2*nstates*nstates+11); 
  ssSetNumIWork(                  S,7);           /*Number of integer workvector elements*/
  ssSetNumPWork(                  S,0);           /*Number of pointer workvector elements*/

  if (mtype == 2)
    {
      nstates = ssGetInputPortWidth(S,0);
    }    
  ssSetNumDiscStates(S,2*nstates+2*nstates*nstates+3);


}



static void mdlInitializeSampleTimes(SimStruct *S)
{
  ssSetSampleTime(S,0,mxGetPr(TS)[0]);
  ssSetOffsetTime(S,0,0);
}


#define MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#if defined(MDL_INITIALIZE_CONDITIONS)
static void mdlInitializeConditions(SimStruct *S)
{
  int_T   nrow   = mxGetM(NNN);           /*number of columns in NNN*/
  int_T   ncol   = mxGetN(NNN);           /*number of rows in NNN*/
  real_T  *model = mxGetPr(NNN);          /*pointer to NNN*/
  real_T  *dstates = ssGetRealDiscStates(S); /*pointer to discrete states*/
  real_T  *rwork = ssGetRWork(S);  
  int_T   nrdstate = ssGetNumDiscStates(S); /*number of DiscState elements*/
  int_T   mtype;
  int_T   tstart = 2;
  int_T   nstates;                        /*number of states*/
  int_T   *iwork = ssGetIWork(S);
  int_T   i,j,nx,ny,nu,offset,offset2;
  real_T lambda = mxGetPr(LAMBDA)[0];


  if (lambda == -1)
    {
      lambda = 1;
      iwork[2] = 1;
    }
  else
    iwork[2] =0;
  

  /*determine the type of the specified model*/

  mtype = modeltype(model,nrow,ncol);
  nstates = numberofstates(model,mtype,nnnfile);
  tstart = initstates(2,model,nstates,mtype,S,nnnfile);


  dstates[nrdstate-3]=0;              /*initialize the decision fuction gt*/
  dstates[nrdstate-2]=0;
  dstates[nrdstate-1]=0;              /*initialize the alarm*/

  iwork[0] = mtype;
  iwork[1] = nstates;
  iwork[3] = 0;
  iwork[4] = 0;
  iwork[5] = 0;
  iwork[6] = tstart;

  if (mtype < 5)
    for (i=0; i<ssGetNumRWork(S); i++)
      rwork[i] = 0;
}
#endif



static void mdlOutputs(SimStruct *S,int_T tid)
{ 
  InputRealPtrsType u = ssGetInputPortRealSignalPtrs(S,0);
  InputRealPtrsType y = ssGetInputPortRealSignalPtrs(S,1);
  int_T  mtype = ssGetIWork(S)[0];
  int_T  nstates = ssGetIWork(S)[1];
  int_T  noiseest = ssGetIWork(S)[2];
  int_T  *n = ssGetIWork(S) + 3;
  int_T  *tstart = ssGetIWork(S) + 6;
  int_T  nu = ssGetInputPortWidth(S,0);
  int_T  ny = ssGetInputPortWidth(S,1);
  int_T  dm = mxGetPr(DM)[0];
  int_T  winlength = mxGetPr(L)[0];
  int_T  i,j,k;
  real_T *th0 = ssGetRealDiscStates(S);
  real_T *th1 = th0 + nstates;
  real_T *P0 = th1 + nstates;
  real_T *P1 = P0 + nstates*nstates;
  real_T *gt = P1 + nstates*nstates;
  real_T *model = mxGetPr(NNN);
  real_T *sr = mxGetPr(SR);
  real_T *rwork = ssGetRWork(S);
  real_T lambda = mxGetPr(LAMBDA)[0];
  real_T *A,*B,*C,*D,*R1,*R2,*K,*eps0,*eps1,*lambda1,*lambda0,*Lambda1,*epsbar1;
  real_T *Bu,*Bv,*Bth,*Bd,*Du,*Dth;
  real_T *threc,*GT,*phi,*phiold,*phiL,*den0,*den1,*den1old,*eps1old;
  real_T *temp1 = mxCalloc(nstates*nstates,8);
  real_T *temp2 = mxCalloc(nstates*nstates,8);
  real_T *temp3 = mxCalloc(nstates*nstates,8);
  real_T *temp4 = mxCalloc(nstates*nstates,8);
  real_T *yold = mxCalloc(ny,8);
  real_T st=0;
  int_T  alarm = 0;
  mxArray *rhs[4],*lhs[6];
  real_T *n1=mxCalloc(6,sizeof(real_T));
  int_T tempn,tempn2,tempn3;
  real_T *temp;


  GT = ssGetOutputPortRealSignal(S,1);
  threc = ssGetOutputPortRealSignal(S,2); 

  if (mtype == 5)   /*state space*/
    {
     
     rhs[0]=mxCreateDoubleMatrix(1,1,mxREAL);
      rhs[1]=mxCreateDoubleMatrix(1,1,mxREAL);
      rhs[2]=mxCreateDoubleMatrix(1,1,mxREAL);
      rhs[3]=mxCreateDoubleMatrix(1,1,mxREAL);
      lhs[0]=mxCreateDoubleMatrix(1,6,mxREAL);
      mexCallMATLAB(1,lhs,4,rhs,nnnfile);
      for (i=0;i<6;i++)
	n1[i]=mxGetPr(lhs[0])[i];
      /*for easy access of the matrices*/
 

      rhs[0]=mxCreateDoubleMatrix(1,1,mxREAL);
      rhs[1]=mxCreateDoubleMatrix(nstates,1,mxREAL);
      rhs[2]=mxCreateDoubleMatrix(n1[1]+n1[3]+n1[4]+1,1,mxREAL);
      rhs[3]=mxCreateDoubleMatrix(1,1,mxREAL);
      lhs[0]=mxCreateDoubleMatrix(nstates,nstates,mxREAL);
      lhs[1]=mxCreateDoubleMatrix(nstates,n1[1],mxREAL);
      lhs[2]=mxCreateDoubleMatrix(nstates,n1[2],mxREAL);
      lhs[3]=mxCreateDoubleMatrix(nstates,n1[3],mxREAL);
      lhs[4]=mxCreateDoubleMatrix(nstates,n1[4],mxREAL);
      lhs[5]=mxCreateDoubleMatrix(nstates,nstates,mxREAL);
      mxGetPr(rhs[0])[0]=n[0]; /*obs! n=tid*/
      for (i=0;i<nstates;i++)
	mxGetPr(rhs[1])[i]=th0[i];
      for (i=0;i<n1[1]+n1[2]+n1[4]+1;i++)
	mxGetPr(rhs[2])[i]=*u[i];
      mxGetPr(rhs[3])[0]=5;
      mexCallMATLAB(6,lhs,4,rhs,nnnfile);

      temp=mxGetPr(lhs[0]);                    /*A*/
      A=mxCalloc(nstates*nstates,sizeof(real_T));
      for (i = 0; i < nstates*nstates; i++)                                                    
	A[i] = temp[i];

      temp=mxGetPr(lhs[1]);                   /*Bu*/
      Bu=mxCalloc(nstates*n1[1],sizeof(real_T));
      for (i = 0; i < n1[1]*nstates; i++)                                                             
	Bu[i]=temp[i];

      temp=mxGetPr(lhs[2]);                   /*Bv*/
      Bv=mxCalloc(nstates*n1[2],sizeof(real_T));
      for (i = 0; i < nstates*n1[2]; i++)                                                    
	Bv[i] = temp[i];

      temp=mxGetPr(lhs[3]);                   /*Bd*/
      Bd=mxCalloc(nstates*n1[3],sizeof(real_T));
      for (i = 0; i < nstates*n1[3]; i++)                                                    
	Bd[i] = temp[i];

      temp=mxGetPr(lhs[4]);                   /*Bth*/
      Bth=mxCalloc(nstates*n1[4],sizeof(real_T));
      for (i = 0; i < nstates*n1[4]; i++)                                                    
	Bth[i] = temp[i];

      temp=mxGetPr(lhs[5]);                   /*Q*/
      R1=mxCalloc(nstates*nstates,sizeof(real_T));
      for (i = 0; i < nstates*nstates; i++)                                              
	R1[i] = temp[i];

      for (i=0;i<6;i++)
	mxDestroyArray(lhs[i]);

      lhs[0]=mxCreateDoubleMatrix(n1[5],nstates,mxREAL);
      lhs[1]=mxCreateDoubleMatrix(n1[5],n1[1],mxREAL);
      lhs[2]=mxCreateDoubleMatrix(n1[5],n1[4],mxREAL);
      lhs[3]=mxCreateDoubleMatrix(n1[5],n1[5],mxREAL);
      mxGetPr(rhs[3])[0]=6;
      mexCallMATLAB(4,lhs,4,rhs,nnnfile);

      temp=mxGetPr(lhs[0]);                   /*C*/
      C=mxCalloc(n1[5]*nstates,sizeof(real_T));
      for (i = 0; i < n1[5]*nstates; i++)                                                    
	C[i] = temp[i];

      temp=mxGetPr(lhs[1]);                   /*Du*/
      Du=mxCalloc(n1[5]*n1[1],sizeof(real_T));
      for (i = 0; i < n1[5]*n1[1]; i++)                                                    
	Du[i] = temp[i];

      temp=mxGetPr(lhs[2]);                   /*Dth*/
      Dth=mxCalloc(n1[5]*n1[4],sizeof(real_T));
      for (i = 0; i < n1[5]*n1[4]; i++)                                                    
	Dth[i] = temp[i];

      temp=mxGetPr(lhs[3]);                   /*R*/
      R2=mxCalloc(n1[5]*n1[5],sizeof(real_T));
      for (i = 0; i < n1[5]*n1[5]; i++)                                                    
	R2[i] = temp[i];


      tempn=nu;
      tempn2=n1[1];
      B=mxCalloc(nstates*(n1[1]+n1[2]+n1[3]+n1[4]),sizeof(real_T));
      for (i=0;i<n1[1];i++)
	for (j=0;j<nstates;j++)
	  B[i*tempn+j]=Bu[i*tempn2+j];

      tempn=nu;
      tempn3=n1[2];
      for (i=0;i<n1[2];i++)
	for (j=0;j<nstates;j++)
	  B[i*tempn+tempn2+j]=Bv[i*tempn3+j];

      tempn2=n1[1]+n1[2];
      tempn3=n1[3];
      for (i=0;i<n1[3];i++)
	for (j=0;j<nstates;j++)
	  B[i*tempn+tempn2+j]=Bd[i*tempn3+j];

      tempn2=n1[1]+n1[2]+n1[3];
      tempn3=n1[4];
      for (i=0;i<n1[4];i++)
	for (j=0;j<nstates;j++)
	  B[i*tempn+tempn2+j]=Bth[i*tempn3+j];

      D=mxCalloc(ny*(n1[1]+n1[4]),sizeof(real_T));
      for (i=0;i<n1[1]*ny;i++)
	D[i]=Du[i];

      tempn2=n1[1]*ny;
      for (i=0;i<n1[4]*ny;i++)
	D[tempn2+i]=Dth[i];


      K = rwork;
      eps0 = K + ny*nstates;
      eps1 = eps0 + ny;
      lambda1 = eps1 + ny;
      epsbar1 = lambda1 + winlength;
      phiL = epsbar1 + winlength*ny;

      if (n[2]==0)
	{
	  lambda0[0] = lambda;
	  lambda1[0] = lambda;
	  if (lambda <0)
	    {
	      lambda0[0] = 1;
	      lambda1[0] = 1;
	    }
	  for (i=0; i<winlength; i++)
	    {
	      epsbar1[i] = 0;
	      Lambda1[i] = 0;
	    }
	}

      for (i=0; i<winlength; i++)
	for (j=ny; j>0; j--)
	  phiL[i*(ny+1)+j] = phiL[i*(ny+1)+j-1];
      for (i=0; i<winlength; i++)
	{
	  phiL[i*(ny+1)]=*y[i];
	  yold[i]=phiL[i*(ny+1)+ny];
	}

      if (n[2]>=tstart[0])
	{
	  /*update model M1 by removing the inference about y(t-L)*/
	  if (n[2]>=tstart[0]+winlength)
	    {
	      temp2 = matrixmult(P1,nstates,nstates,transpose(C,nstates,ny,temp3),ny,temp2);
	      temp1 = matrixmult(C,ny,nstates,temp2,ny,temp1);
	      for (i=0; i<ny*ny; i++)
		den1old[i] = -Lambda1[0]*R2[i] + temp1[i];
	      inv(ny,den1old);
	      temp1 =  matrixmult(C,ny,nstates,th1,1,temp1);
	      for (i=0; i<nu; i++)
		temp4[i]=*u[i];
	      temp3 = matrixmult(D,ny,nu,temp4,1,temp3);
	      for (i=0; i<ny; i++)
		eps1old[i] = -yold[i] - temp1[i] - temp3[i];
	      K = matrixmult(temp2,nstates,ny,den1old,ny,K);
	      
	      temp1 = matrixmult(K,nstates,ny,eps1old,1,temp1);
	      for (i=0; i<nstates; i++)
		temp1[i]=th1[i]+temp1[i];
	      temp2 = matrixmult(A,nstates,nstates,temp1,1,temp2);
	      for (i=0; i<nu; i++)
		temp4[i] = *u[i];
	      temp3 = matrixmult(B,nstates,nu,temp4,1,temp3);
	      for (i=0; i<nstates; i++)
		th1[i] = temp2[i] + temp3[i];
	      temp2 = matrixmult(C,ny,nstates,P1,nstates,temp2);
	      temp1 = matrixmult(K,nstates,ny,temp2,nstates,temp1);
	      for (i=0; i<nstates*nstates; i++)
		temp1[i] = P1[i] - temp1[i];
	      temp2 = matrixmult(A,nstates,nstates,temp1,nstates,temp2);
	      temp1 = matrixmult(temp2,nstates,nstates,transpose(A,nstates,nstates,temp3),nstates,temp1);
	      for (i=0; i<nstates*nstates; i++)
		P1[i]=temp1[i]+Lambda1[0]*R1[i];

	      temp1 = transpose(P1,nstates,nstates,temp1);
	      for (i=0; i<nstates*nstates; i++)
		P1[i] = 0.5*(P1[i]+temp1[i]);
	      n[1]--;
	    }
	  
	  /*update model M0 with y(t)*/
	  kalman(den0,A,B,C,D,R1,R2,P0,th0,eps0,lambda0[0],S,nstates);
	  n[0]++;
	  for (i=0; i<nstates; i++)
	    threc[i] = th0[i];
	  
	  /*update model M1 with y(t)*/
	  kalman(den1,A,B,C,D,R1,R2,P1,th1,eps1,lambda1[0],S,nstates);

	  n[1]++;
	  for (i=0; i<winlength-1; i++)
	    epsbar1[i] = epsbar1[i+1];
	  epsbar1[winlength-1] = eps1[0]*eps1[0]*den1[0];
	  
	  if (noiseest == 1)
	    {
	      lambda0[0] = lambda0[0]*(n[0]-1)/n[0]+eps0[0]*eps0[0]*den0[0]/n[0];
	      if (n[1] == winlength)
		lambda1[0] = lambda1[0]+(epsbar1[winlength-1]-epsbar1[0])/n[1];
	      else
		lambda1[0] = lambda1[0]*(n[1]-1)/n[1]+eps1[0]*eps1[0]*den1[0]/n[1];
	    }
	  for (i=0; i<winlength-2; i++)
	    Lambda1[i] = Lambda1[i+1];
	  Lambda1[winlength-2] = lambda1[0];
	}
       
    }
  else
    {
      phi = ztophi(S,nstates,mtype,model);

      phiold = phi + nstates+1;
      for (i=winlength; i>0; i--)
	for (j=0; j<nstates; j++)
	  phiold[i*nstates+j] = phiold[(i-1)*nstates+j];
      for (j=0; j<nstates; j++)
	phiold[j] = phi[j];

      den0 = phiold + (winlength+1)*nstates;
      den1 = den0 + 1;
      den1old = den1 + 1;
      R1 = den1old + 1;
      R2 = R1 + 1;
      K = R2 + 1;
      eps0 = K + nstates;
      eps1 = eps0 + 1;
      eps1old = eps1 + 1;
      lambda0 = eps1old + 1;
      lambda1 = lambda0 + 1;
      Lambda1 = lambda1 + 1;
      epsbar1 = Lambda1 + winlength;

      *R1 = 0;
      *R2 = 1;

      if (n[2]==0)
	{
	  lambda0[0] = lambda;
	  lambda1[0] = lambda;
	  if (lambda <0)
	    {
	      lambda0[0] = 1;
	      lambda1[0] = 1;
	    }
	  for (i=0; i<winlength; i++)
	    {
	      epsbar1[i] = 0;
	      Lambda1[i] = 0;
	    }
	}

      if (n[2]>=tstart[0])
	{
	  /*update model M1 by removing the inference about y(t-L)*/
	  if (n[2]>=tstart[0]+winlength)
	    {
	      phiL = phiold + winlength*nstates;
	      temp2 = matrixmult(P1,nstates,nstates,phiL,1,temp2);
	      temp1 = matrixmult(phiL,1,nstates,temp2,1,temp1);
	      den1old[0] = 1/(-Lambda1[0]*R2[0] + temp1[0]);
	      eps1old[0] = -(phiL-nstates)[0] - matrixmult(phiL,1,nstates,th1,1,temp1)[0];
	      for (i=0; i<nstates; i++)
		{
		  K[i] = temp2[i]*den1old[0];
		  th1[i] = th1[i] + K[i]*eps1old[0];
		}
	      temp2 = matrixmult(phiL,1,nstates,P1,nstates,temp2);
	      temp1 = matrixmult(K,nstates,1,temp2,nstates,temp1);
	      for (i=0; i<nstates*nstates; i++)
		P1[i] = P1[i] - temp1[i];
	      temp1 = transpose(P1,nstates,nstates,temp1);
	      for (i=0; i<nstates*nstates; i++)
		P1[i] = 0.5*(P1[i]+temp1[i]);
	      n[1]--;
	    }

	  /*update model M0 with y(t)*/
	  temp2 = matrixmult(P0,nstates,nstates,phi,1,temp2);
	  temp1 = matrixmult(phi,1,nstates,temp2,1,temp1);
	  den0[0] = 1/(lambda0[0]*R2[0] + temp1[0]);
	  eps0[0] = *y[0] - matrixmult(phi,1,nstates,th0,1,temp1)[0];
	  for (i=0; i<nstates; i++)
	    {
	      K[i] = temp2[i]*den0[0];
	      th0[i] = th0[i] + K[i]*eps0[0];
	    }
	  temp2 = matrixmult(phi,1,nstates,P0,nstates,temp2);
	  temp1 = matrixmult(K,nstates,1,temp2,nstates,temp1);
	  for (i=0; i<nstates*nstates; i++)
	    P0[i] = P0[i] - temp1[i];
	  temp1 = transpose(P0,nstates,nstates,temp1);
	  for (i=0; i<nstates*nstates; i++)
	    P0[i] = 0.5*(P0[i]+temp1[i]);
	  n[0]++;
	  for (i=0; i<nstates; i++)
	    threc[i] = th0[i];

	  /*update model M1 with y(t)*/
	  temp2 = matrixmult(P1,nstates,nstates,phi,1,temp2);
	  temp1 = matrixmult(phi,1,nstates,temp2,1,temp1);
	  den1[0] = 1/(lambda1[0]*R2[0] + temp1[0]);
	  eps1[0] = *y[0] - matrixmult(phi,1,nstates,th1,1,temp1)[0];
	  for (i=0; i<nstates; i++)
	    {
	      K[i] = temp2[i]*den1[0];
	      th1[i] = th1[i] + K[i]*eps1[0];
	    }
	  temp2 = matrixmult(phi,1,nstates,P1,nstates,temp2);
	  temp1 = matrixmult(K,nstates,1,temp2,nstates,temp1);
	  for (i=0; i<nstates*nstates; i++)
	    P1[i] = P1[i] - temp1[i];
	  temp1 = transpose(P1,nstates,nstates,temp1);
	  for (i=0; i<nstates*nstates; i++)
	    P1[i] = 0.5*(P1[i]+temp1[i]);
	  n[1]++;
	  for (i=0; i<winlength-1; i++)
	    epsbar1[i] = epsbar1[i+1];
	  epsbar1[winlength-1] = eps1[0]*eps1[0]*den1[0];

	  if (noiseest == 1)
	    {
	      lambda0[0] = lambda0[0]*(n[0]-1)/n[0]+eps0[0]*eps0[0]*den0[0]/n[0];
	      if (n[1] == winlength)
		lambda1[0] = lambda1[0]+(epsbar1[winlength-1]-epsbar1[0])/n[1];
	      else
		lambda1[0] = lambda1[0]*(n[1]-1)/n[1]+eps1[0]*eps1[0]*den1[0]/n[1];
	    }
	  for (i=0; i<winlength-2; i++)
	    Lambda1[i] = Lambda1[i+1];
	  Lambda1[winlength-2] = lambda1[0];
	}
    }

  if (n[0]>=winlength && n[1]==winlength)
    {
      /*compute the distance measure*/
    if (dm==1)
        st=0.5*(log(lambda0[0]/lambda1[0])+eps0[0]*eps0[0]*den0[0]/lambda0[0]-eps1[0]*eps1[0]*den1[0]/lambda1[0]);
    else if (dm==2)
        st=0.5*(lambda0[0]/lambda1[0]-1 + (1+lambda0[0]/lambda1[0])*eps0[0]*eps0[0]*den0[0]/lambda0[0]- 2*sqrt(den0[0]*den1[0])*eps0[0]*eps1[0]/lambda1[0]);
    else if (dm==3)
        st=eps1[0]-eps0[0];
    
    /*apply the stopping rule*/
    stop(gt,st,sr);

    GT[0] = gt[0];
    GT[1] = gt[1];
    /*check alarm*/
    if (gt[0]>sr[1] || gt[1]>sr[1])
      {
	alarm = 1;
	gt[0]=0;
	gt[1]=0;
	for (i=0; i<nstates; i++)
	  {
	    th0[i]=0;
	    th1[i]=0;
	  }
	for (i=0; i<nstates*nstates; i++)
	  {
	    P0[i]=0;
	    P1[i]=0;
	  }
	for (i=0; i<nstates*nstates; i=i+nstates+1)
	  {
	    P0[i]=100000/*lambda*/;
	    P1[i]=P0[i];
	  }
	n[0] = 0;
	n[1] = 0;
	lambda1[0]=lambda;
	lambda0[0]=lambda;
	tstart[0] = n[2]+1;
      }
    }
  ssGetOutputPortRealSignal(S,0)[0] = alarm;

  /*GT[0] = lambda0[0];
    GT[1] = lambda1[0];*/
    /*for (i=0; i<nstates; i++)
  threc[i] = th0[i];*/

  n[2]++;
}


static void mdlTerminate(SimStruct *S)
{
  return;
}


#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif
