/*
 *========================================================================
 *
 *detect1
 *
 *MEX-file implementing:
 *detect1 - detects changes using a one-filter approach
 *
 *Jonas Elbornsson July 1998
 *========================================================================
 */

#define S_FUNCTION_NAME lms    /*defines the name of the S-function*/
#define S_FUNCTION_LEVEL 2     /*defines the level of the S-function*/

#include "simstruc.h"
#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif
#include <math.h>
#include "stop.h"
#include "matrixop.h"
#include "model.h"
#include "ztophi.h"
#include "lms2.h"

#define NNN                ssGetSFcnParam(S,0)    /*model structure*/   
#define MU                 ssGetSFcnParam(S,1)    /*noise variance*/
#define TS                 ssGetSFcnParam(S,2)    /*sample time*/
#define IN                 ssGetSFcnParam(S,3)
#define OUT                ssGetSFcnParam(S,4)
/*
 *====================================================================
 *mdlInitializeSizes - initialize the sizes array
 *
 *The sizes array is used by SIMULINK to determine the S-function
 *block's characteristics (number of inputs, outputs, states, etc.)
 *
 *====================================================================
 */

static void mdlInitializeSizes(SimStruct *S)
{
  int_T   nrow   = mxGetM(NNN);           /*number of columns in NNN*/
  int_T   ncol   = mxGetN(NNN);           /*number of rows in NNN*/
  real_T  *model = mxGetPr(NNN);          /*pointer to NNN*/
  int_T   mtype;
  int_T   nstates;                        /*number of states*/
  real_T   *nu     = mxGetPr(IN);
  real_T   *ny     = mxGetPr(OUT);
  
  /*determine the type of the specified model*/

  mtype = modeltype(model,nrow,ncol);
  nstates = numberofstates(model,mtype,"nofile");

  ssSetNumContStates(             S,0);           /*Number of continuous states*/
  ssSetNumDiscStates(S,     nstates+2);           /*Number of discrete states*/

  if (!ssSetNumInputPorts(        S,2)) return;   /*Number of inputports*/
  ssSetInputPortWidth(           S,0,*nu);         /*Width of inputport 0*/
  ssSetInputPortWidth(           S,1,*ny);         /*Width of inputport 1*/ 
  ssSetInputPortDirectFeedThrough(S,0,1);         /*Direct feedthrough flag*/
  ssSetInputPortDirectFeedThrough(S,1,1);

  if (!ssSetNumOutputPorts(       S,2)) return;   /*Number of outputports*/ 
  ssSetOutputPortWidth(           S,0,1);         /*Width of outputport 0*/
  ssSetOutputPortWidth(     S,1,nstates);         /*Width of outputport 2*/


  ssSetNumSampleTimes(            S,1);           /*Number of sample times*/

  ssSetNumSFcnParams(             S,5);           /*Number of input arguments*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) 
    {
        /* Return if number of expected != number of actual parameters */
        return;
    }
  if (mtype == 5)
    ssSetNumRWork(S,0);/*Number of Rwork elements*/
  else if (mtype == 4)
    ssSetNumRWork( S,3+model[2]+nstates);
  else
    ssSetNumRWork(S,3+nstates); 
  ssSetNumIWork(                  S,4);           /*Number of integer workvector elements*/
  ssSetNumPWork(                  S,0);           /*Number of pointer workvector elements*/

}

 
static void mdlInitializeSampleTimes(SimStruct *S)
{
  ssSetSampleTime(S,0,mxGetPr(TS)[0]);
  ssSetOffsetTime(S,0,0);
}

#define MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#if defined(MDL_INITIALIZE_CONDITIONS)
static void mdlInitializeConditions(SimStruct *S)
{
  int_T   nrow   = mxGetM(NNN);           /*number of columns in NNN*/
  int_T   ncol   = mxGetN(NNN);           /*number of rows in NNN*/
  real_T  *model = mxGetPr(NNN);          /*pointer to NNN*/
  real_T  *dstates = ssGetRealDiscStates(S); /*pointer to discrete states*/
  int_T   mtype,tstart,nstates,i;
  int_T *iwork = ssGetIWork(S);


  /*determine the type of the specified model*/

  mtype = modeltype(model,nrow,ncol);
  nstates = numberofstates(model,mtype,"nofile");
  
  for (i=0; i<nstates+2; i++)
    dstates[i]=0;

  tstart=2;
  if (mtype == 2)
    tstart = nstates + 1;
  
  else if (mtype == 3)
    tstart = nstates + 1;

  else if (mtype == 4)
    if (model[0]+1 > model[1]+model[2])
      tstart = model[0]+1;
    else
      tstart = model[1]+model[2];

  iwork[0] = mtype;                                /*modeltype*/
  iwork[1] = nstates;                              /*number of states*/
  iwork[2] = 0;                                    /*time*/
  iwork[3] = tstart;                               /*startingtime*/

}
#endif

static void mdlOutputs(SimStruct *S,int_T tid)
{
  real_T *model = mxGetPr(NNN);
  real_T *rwork = ssGetRWork(S);
  int_T ny = ssGetInputPortWidth(S,1);
  int_T nu = ssGetInputPortWidth(S,0);
  int_T nstates = ssGetIWork(S)[1];
  int_T mtype = ssGetIWork(S)[0];
  int_T *n = ssGetIWork(S)+2;
  real_T *eps,*phi,*threc;
  int_T i;
  int_T tstart = ssGetIWork(S)[3];
  real_T *th = ssGetRealDiscStates(S);
  real_T mu = mxGetPr(MU)[0];
  real_T *norm=mxCalloc(1,8);
 

  threc = ssGetOutputPortRealSignal(S,2);

  n[0]++;

  if (mtype == 5)  /*state space*/
    {
      ssSetErrorStatus(S,"Not valid modeltype for LMS");
      return;
    }
  else
    {
      phi = ztophi(S,nstates,mtype,model);

      eps = phi + nstates;

      if (n[0]>tstart)
	{
	  lms2(phi,th,eps,norm,mu,nstates,S);
	  for (i=0; i<nstates; i++)
	    threc[i] = th[i];
	}

    }



}


static void mdlTerminate(SimStruct *S)
{
  return;
}

#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif
