/*
 *========================================================================
 *
 *rdetect1
 *
 *MEX-file implementing:
 *rdetect1 - detects changes using a one-filter approach
 *
 *Jonas Elbornsson August 1998
 *========================================================================
 */

#define S_FUNCTION_NAME rdetect1 /*defines the name of the S-function*/
#define S_FUNCTION_LEVEL 2     /*defines the level of the S-function*/

#include "simstruc.h"
#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif
#include <math.h>
#include "stop.h"
#include "matrixop.h"


#define DM                 ssGetSFcnParam(S,0)    /*distance measure*/
#define SR                 ssGetSFcnParam(S,1)    /*stopping rule*/
#define LAMBDA             ssGetSFcnParam(S,2)    /*noise variance*/
#define TS                 ssGetSFcnParam(S,3)    /*sample time*/
#define N                  ssGetSFcnParam(S,4)    /*number of eps-signals*/



static void mdlInitializeSizes(SimStruct *S)
{
  int_T   neps = mxGetPr(N)[0];                        /*number of states*/

  ssSetNumContStates(             S,0);           /*Number of continuous states*/
  ssSetNumDiscStates(             S,3);           /*Number of discrete states*/

  if (!ssSetNumInputPorts(        S,1)) return;   /*Number of inputports*/
  ssSetInputPortWidth(           S,0,neps);         /*Width of inputport 0*/
  ssSetInputPortDirectFeedThrough(S,0,1);         /*Direct feedthrough flag*/


  if (!ssSetNumOutputPorts(       S,2)) return;   /*Number of outputports*/ 
  ssSetOutputPortWidth(           S,0,1);         /*Width of outputport 0*/
  ssSetOutputPortWidth(           S,1,2);         /*Width of outputport 1*/


  ssSetNumSampleTimes(            S,1);           /*Number of sample times*/

  ssSetNumSFcnParams(             S,5);           /*Number of input arguments*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) 
    {
        /* Return if number of expected != number of actual parameters */
        return;
    }

    ssSetNumRWork(                  S,1+neps); 
    ssSetNumIWork(                  S,2);           /*Number of integer workvector elements*/
    ssSetNumPWork(                  S,0);           /*Number of pointer workvector elements*/

}

 
static void mdlInitializeSampleTimes(SimStruct *S)
{
  ssSetSampleTime(S,0,mxGetPr(TS)[0]);
  ssSetOffsetTime(S,0,0);
}

#define MDL_INITIALIZE_CONDITIONS  
#if defined(MDL_INITIALIZE_CONDITIONS)
static void mdlInitializeConditions(SimStruct *S)
{
  real_T  *dstates = ssGetRealDiscStates(S); /*pointer to discrete states*/
  int_T  noiseest;
  real_T lambda0 = mxGetPr(LAMBDA)[0];
  real_T lambda;
  int_T *iwork = ssGetIWork(S);

  if (lambda0 == -1)
    {
      lambda = 1;
      noiseest = 1;
    }
  else
    {
      noiseest = 0;
      lambda = lambda0;
    }



  dstates[0]=0;                   /*initialize the decision function gt*/
  dstates[1]=0;
  dstates[2]=lambda;              /*initialize lambda*/

  iwork[0] = noiseest;
  iwork[1] = 0;

}
#endif


static void mdlOutputs(SimStruct *S,int_T tid)
{
  InputRealPtrsType eps = ssGetInputPortRealSignalPtrs(S,0);
  int_T *n = ssGetIWork(S)+1;
  int_T dm = mxGetPr(DM)[0];
  real_T *sr = mxGetPr(SR);
  real_T *gt = ssGetRealDiscStates(S);
  real_T *lambda = gt+2;
  int_T alarm,i;
  int_T noiseest=ssGetIWork(S)[0];
  real_T st=0;
  real_T *GT;
  int_T neps = ssGetInputPortWidth(S,0);
  real_T *rwork = ssGetRWork(S);
  real_T epssq;

  for (i=0; i<neps; i++)
    rwork[i+1] = *eps[i];

  epssq = matrixmult(rwork+1,neps,1,rwork+1,1,rwork)[0];

  n[0]++;

  if (noiseest == 1)
    lambda[0] = lambda[0]*(n[0]-1)/n[0]+epssq/n[0];
  
  if (dm==1)
    {
      for (i=0; i<neps; i++)
	st = st + *eps[i];
      st = st/neps;
    }
  else if (dm==2)
    st = epssq/neps;


  stop(gt,st,sr);

  if ((gt[0]>sr[1]) || (gt[1]>sr[1]))
    {
      alarm=1;
      gt[0]=0;
      gt[1]=0;
      n[0] =0;
    }
  else
    {
      alarm=0;
    }
  
  ssGetOutputPortRealSignal(S,0)[0] = alarm;
  GT = ssGetOutputPortRealSignal(S,1);
  GT[0] = gt[0];
  GT[1] = gt[1];
}


static void mdlTerminate(SimStruct *S)
{
  return;
}


#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif



