/*
 *========================================================================
 *
 *rls
 *
 *MEX-file implementing:
 *rls - the recursive least squares filter algorithm
 *
 *Jonas Elbornsson July 1998
 *========================================================================
 */

#define S_FUNCTION_NAME rls /*defines the name of the S-function*/
#define S_FUNCTION_LEVEL 2     /*defines the level of the S-function*/

#include "simstruc.h"
#ifdef MATLAB_MEX_FILE
#include "mex.h"
#endif
#include <math.h>
#include "matrixop.h"
#include "model.h"
#include "ztophi.h"
#include "rls.h"
#include "inv.h"


#define NNN                ssGetSFcnParam(S,0)    /*model structure*/   
#define LAMBDA             ssGetSFcnParam(S,1)    /*forgetting factor*/
#define TS                 ssGetSFcnParam(S,2)    /*sample time*/
#define IN                 ssGetSFcnParam(S,3)    /*number of input signals*/
#define OUT                ssGetSFcnParam(S,4)    /*number of output signals*/


/*
 *====================================================================
 *mdlInitializeSizes - initialize the sizes array
 *
 *The sizes array is used by SIMULINK to determine the S-function
 *block's characteristics (number of inputs, outputs, states, etc.)
 *
 *====================================================================
 */

static void mdlInitializeSizes(SimStruct *S)
{
  int_T   nrow   = mxGetM(NNN);           /*number of columns in NNN*/
  int_T   ncol   = mxGetN(NNN);           /*number of rows in NNN*/
  real_T  *model = mxGetPr(NNN);          /*pointer to NNN*/
  int_T   mtype;
  int_T   nstates;                        /*number of states*/
  real_T   nu     = mxGetPr(IN)[0];
  real_T   ny     = mxGetPr(OUT)[0];
  
  /*determine the type of the specified model*/

  mtype = modeltype(model,nrow,ncol);
  if (mtype>4)
    {
      ssSetErrorStatus(S,"Not a valid modeltype for RLS");
      return;
    }
  nstates = numberofstates(model,mtype,"nofile");

  ssSetNumContStates(             S,0);           /*Number of continuous states*/
  ssSetNumDiscStates(S,nstates+nstates*nstates); /*Number of discrete states*/

  if (!ssSetNumInputPorts(        S,3)) return;   /*Number of inputports*/
  ssSetInputPortWidth(           S,0,nu);         /*Width of inputport 0*/
  ssSetInputPortWidth(           S,1,ny);         /*Width of inputport 1*/ 
  ssSetInputPortWidth(           S,2,1);         /*Width of inputport 2*/ 
  ssSetInputPortDirectFeedThrough(S,0,1);         /*Direct feedthrough flag*/
  ssSetInputPortDirectFeedThrough(S,1,1);

  if (!ssSetNumOutputPorts(       S,2)) return;   /*Number of outputports*/ 
  ssSetOutputPortWidth(           S,0,1);         /*Width of outputport 0*/
  ssSetOutputPortWidth(     S,1,nstates);         /*Width of outputport 1*/


  ssSetNumSampleTimes(            S,1);           /*Number of sample times*/

  ssSetNumSFcnParams(             S,5);           /*Number of input arguments*/
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) 
    {
        /* Return if number of expected != number of actual parameters */
        return;
    }
  if (mtype == 4)
    ssSetNumRWork( S,5+model[2]+5*nstates*nstates+3*nstates*(ny+nu)+2*ny*ny+ny+nu*ny);
  else
    ssSetNumRWork(S,5+5*nstates*nstates+3*nstates*(ny+nu)+2*ny*ny+ny+nu*ny); 
  ssSetNumIWork(                  S,4);           /*Number of integer workvector elements*/
  ssSetNumPWork(                  S,0);           /*Number of pointer workvector elements*/

}

 
static void mdlInitializeSampleTimes(SimStruct *S)
{
  ssSetSampleTime(S,0,mxGetPr(TS)[0]);
  ssSetOffsetTime(S,0,0);
}

#define MDL_INITIALIZE_CONDITIONS   /* Change to #undef to remove function */
#if defined(MDL_INITIALIZE_CONDITIONS)
static void mdlInitializeConditions(SimStruct *S)
{
  int_T   nrow   = mxGetM(NNN);           /*number of columns in NNN*/
  int_T   ncol   = mxGetN(NNN);           /*number of rows in NNN*/
  real_T  *model = mxGetPr(NNN);          /*pointer to NNN*/
  real_T  *dstates = ssGetRealDiscStates(S); /*pointer to discrete states*/
  int_T   mtype,tstart,nstates;
  int_T *iwork = ssGetIWork(S);


  /*determine the type of the specified model*/

  mtype = modeltype(model,nrow,ncol);
  nstates = numberofstates(model,mtype,"nofile");
  tstart = initstates(1,model,nstates,mtype,S,"nofile");



  iwork[0] = mtype;                                /*modeltype*/
  iwork[1] = nstates;                              /*number of states*/
  iwork[2] = 0;                                    /*time*/
  iwork[3] = tstart;                               /*startingtime*/

}
#endif

static void mdlOutputs(SimStruct *S,int_T tid)
{ 
  real_T *model = mxGetPr(NNN);
  InputRealPtrsType u = ssGetInputPortRealSignalPtrs(S,0);
  InputRealPtrsType y = ssGetInputPortRealSignalPtrs(S,1);
  InputRealPtrsType reset= ssGetInputPortRealSignalPtrs(S,2);
  int_T ny = ssGetInputPortWidth(S,1);
  int_T nu = ssGetInputPortWidth(S,0);
  int_T nstates = ssGetIWork(S)[1];
  int_T mtype = ssGetIWork(S)[0];
  int_T *n = ssGetIWork(S)+2;
  int_T i;
  int_T tstart = ssGetIWork(S)[3];
  real_T *th = ssGetRealDiscStates(S);
  real_T *P = th+nstates;
  real_T *epsout = ssGetOutputPortRealSignal(S,0); 
  real_T *threc = ssGetOutputPortRealSignal(S,1); 
  real_T lambda = mxGetPr(LAMBDA)[0];
  real_T *eps = mxCalloc(ny,sizeof(real_T));
  real_T *phi;

  n[0]++;


    
  phi = ztophi(S,nstates,mtype,model);
 

  if (n[0]>tstart)
    {
      rls(phi,th,P,eps,lambda,nstates,S);
      
      for (i=0; i<nstates; i++)
	threc[i] = th[i];
      for (i=0; i<ny; i++)
	epsout[i]=eps[i];
	
    }
  
  
  
  if (*reset[0]==1)
    {
      n[0]=0;
      for (i=0; i<nstates; i++)
	th[i]=0;
      for (i=0; i<nstates*nstates; i++)
	P[i]=0;
      for (i=0; i<nstates*nstates; i=i+nstates+1)
	P[i]=100000;
    }
  
}


static void mdlTerminate(SimStruct *S)
{
  return;
}

#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif
