function [jumphat,thseg,GT,U]=cpe(z,nnn,dm,h,lambda,dsp);

% CPE Change point estimation 
%
% function [jumphat,thseg,GT]=cpe(z,nnn,dm,h,lambda,dsp);
%
% Implementation of some statistical off-line methods for
% estimating the change point for mean in white noise
%
% z=[y u]   output-input data
% nnn       model structure is set to [0 1 0] !
% dm        distance measure = [approach problem]: 
% approach  1 = Bayesian (Gaussian noise)
%           2 = ML       (Gaussian noise)
%           3 = Non-parametric I (default)
%           4 = Non-parametric II
% problem   1 = Detect increase in mean, th0 unknown
%           2 = Detect increase in mean, th0 = 0 known
%           3 = Detect change in mean, th0 unknown  (default)
%           4 = Detect change in mean, th0 = 0 known
% h         Threshold
% lambda    Known noise variance
% jumphat   estimated change points
% thseg     estimated parameters
% GT        The decision function
%
% Reference:  These methods are surveyed and proposed in 
% Sen & Srivastava, The annals of statistics, 1975, vol 3, p 98-108.

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if nargin<5 lambda=-1; end;    if lambda<0, lambda=1;  end
if nargin<4 h=-1; end;    if h<0,   h=3;  end
if nargin<3 dm=-1; end;   if dm<0,  dm=1; end
if length(dm) == 1; dm=[dm 3]; end;


nnn=[0 1 0];
[N,nz]=size(z);
y=z(:,1);
ybar=mean(y);
GT=zeros(N-1,1);

if dm(1)==1;  
%-------- Bayes ----------------
  U=0;
  for k=1:N-1;
    GT(k)=(sum(y(k+1:N))-(N-k)*ybar)^2;
  end;
  U=sum(GT)/N/N;
  GT=GT./(N-1:-1:1)';         % Ad-hoc
  [dum,jumphat]=max(GT);

elseif dm(1)==2;
%-------- ML ----------------
  for k=1:N-1;
    GT(k)=(mean(y(k+1:N))-mean(y(1:k)))/(1/k+1/(N-k));
  end;
  [U,jumphat]=max(GT);

elseif dm(1)==3;
%-------- Non-parametric I --------
  ym=median(y);
  for k=1:N-1;
    ind=find(y(k+1:N) > ym);
    GTp(k)=length(ind);
    ind=find(y(k+1:N) < ym);
    GTm(k)=length(ind);
  end;
  GTp=(GTp-(N-1:-1:1)/2);   %./sqrt(1:N-1);
  GTm=(GTm-(N-1:-1:1)/2);   %./sqrt(1:N-1);
  GT=max([GTp;GTm]);    % Pair-wise maximization
  GT=GT';
  [U,jumphat]=max(GT);

elseif dm(1)==4;
%-------- Non-parametric II --------
  for k=1:N-1;
    GTp(k)=0; GTm(k)=0;
    for n=k+1:N;
      ind=find(y(n) <= y);
      GTp(k)=GTp(k)+length(ind);
    end;
    for n=k+1:N;
      ind=find(y(n) >= y);
      GTm(k)=GTm(k)+length(ind);
    end;
  end;
  GTp=(GTp/N-(N-1:-1:1)/2);   %./sqrt(1:N-1);
  GTm=(GTm/N-(N-1:-1:1)/2);   %./sqrt(1:N-1);
  GT=max([GTp;GTm]);    % Pair-wise maximization
  GT=GT';
  [U,jumphat]=max(GT);
end;

if U<lambda; 
  jumphat=0; 
  thseg=ybar*ones(1,N);
else;
  thseg=[mean(y(1:jumphat))*ones(1,jumphat) mean(y(jumphat+1:N))*ones(1,N-jumphat)];
end;


