function [L0,N0]=cusumarl(h,mu,acc,Nstep,sigma,dsp);

% CUSUMARL computes the arl function
%
% L0=cusumarl(h,mu,acc,Nstep,sigma,dsp);
%
% The ARL function is defined as E[ta|h,mu] where
% ta is the stopping time from the CUSUM algorithm.
% The signal is assumed to be white with mean th0 
% and variance sigma^2. Zero initial
% condition in the CUSUM test is assumed.
% A Fredholm integral equation is solved numerically
% but Siegmund's approximation can be obtained as well.
%
% L0    mean time between alarms
% N0    mean number of samples between reset of CUSUM
% h     threshold in cusum algorithm
% mu    mean E(s(k)) = true mean of signal th0 - drift parameter nu
% sigma standard deviation of s(k) (Default 1). NOTE: sigma=1
%       assumed in the numerical solution, but not in the approximations.
% acc   relative error in L0. Default is 0.5.
% Nstep Number of grid points in the numerical function approximation
%       Default is 200.
%       Nstep=-1 gives Siegmund's approximation.
%       Nstep=-2 gives Wald's approximation.
% dsp   Display some info in each rekursion
%

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if nargin<3; acc=0.5; end;
if nargin<4; Nstep=200; end;
if nargin<5; sigma=1; end;
if nargin<6; dsp=0; end;

if Nstep>0;
% Solve the Fredholm equation numerically
  xmax=max([h mu+4]);
  xstep=2*xmax/Nstep;
  x=(-xmax:xstep:xmax)';
  n=length(x);
  f=1/sqrt(2*pi)*exp(-0.5*(-x-mu).^2);  % f(-x)
  fdum=1/sqrt(2*pi)*exp(-0.5*(x-mu).^2);% f(x)
  Fdum=cumsum(fdum)/sum(fdum);          % F(x)
  F=Fdum(n:-1:1);                       % F(-x)
  L=zeros(n,1);
  Lnew=.3*ones(n,1);

  ind=find(x>=0);
  ind0=ind(1); 
  indz=find(x<0 | x>h);
  Lnew(indz)=zeros(size(indz));

  N=L;
  Nnew=Lnew;
  Lnew(ind0)=0;
  while abs(Lnew(ind0)-L(ind0)) > acc | norm(Lnew-L)>0.1; 
      L=Lnew;
      y=xstep*conv(L,f);  y=y(:);
      y=y(Nstep/2:Nstep+Nstep/2);
      Lnew=1+L(ind0)*F+y; 
      Lnew(indz)=zeros(size(indz));
      if dsp==1;
        figure(1)
        plot(x,[Lnew L]),
        title(['Norm(Lnew-L) = ',num2str(norm(Lnew-L)),'   Sensitivity of L(0) = ',num2str(abs(Lnew(ind0)-L(ind0))/(1-Lnew(ind0)))]), 
        drawnow
      end;
  end;
  L0=acc*round(L(ind0)/acc);

  if nargout>1;
    while Nnew(ind0)*abs(Nnew(ind0)-N(ind0)) > acc | norm(Nnew-N) > 0.1; 
        N=Nnew;
        y=xstep*conv(N,f); y=y(:);
        y=y(Nstep/2:Nstep+Nstep/2);
        Nnew=1+y;            
        Nnew(indz)=zeros(size(indz));
        if dsp==1;
          plot(x,[Nnew N]),
          title(['Norm(Nnew-N) = ',num2str(norm(Nnew-N)),' Sensitivity of N(0) = ',num2str(Nnew(ind0)*abs(Nnew(ind0)-N(ind0)))]), 
          drawnow
        end;
    end;
  N0=acc*round(N(ind0)/acc);
  end;

elseif Nstep==-1;
% Siegmund's approximation
  x=(h/sigma+1.166);
  if mu==0; 
    L0=x^2;
  else;
    L0=(exp(-2*x*mu/sigma)-1+2*x*mu/sigma)/(2*mu^2/sigma^2);
  end;
  L0=acc*round(L0/acc);
  N0=[];

elseif Nstep==-2;
% Wald's approximation
  x=(h/sigma);
  if mu==0; 
    L0=x^2;
  else;
    L0=(exp(-2*x*mu/sigma)-1+2*x*mu/sigma)/(2*mu^2/sigma^2);
  end;
  L0=acc*round(L0/acc);
  N0=[];
end;
