echo on
clc
%    Demostration of design of CUSUM detectors
%   ===========================================
%
% Assume we observe a signal y(t) which is N(0,sigma^2) before the change and
% N(theta,sigma) after the change. We apply the CUSUM algorithm with threshold h
% and drift nu. A succesful design requires of course that theta > nu/2.
% Assume in this example that theta = nu.
%
% The average run length (ARL) function for a stopping rule for 
% detecting a change theta in the mean of a signal is defined as
% 
%        E(ta|theta)
% 
% where ta is the stopping time.
% There are two design parameter in the CUSUM test, the threshold h
% and the drift parameter nu. If sigma is the standard deviation of the noise, then
%
%             /   h      theta-nu/2 \
%     ta = f |  ----- , ------------ |
%             \ sigma     sigma     /
%
% That is, it is a function of two arguments. We can assume that sigma=1
% and, for simplicity, nu = theta (a standard choice of drift if theta is known).
% The mean time between false alarms is f(h,-theta/2) and 
% the mean time for detection is        f(h,+theta/2).
% To check these for a given h, use
%
h=3;  % 5
nu=1;
th=1;
mu=th-nu/2;  % 0.4
[ta0,N0] = cusumarl(h,-mu)
[ta1,N0] = cusumarl(h,+mu)
pause
%
% That is, the mean time between false alarms is 4 times larger.
% N0 denotes the mean time beween resets in the CUSUM algorithm.
% A reset is caused by either an alarm or a negative test statistic.
%
% The approximation suggested by Siegmund is obtained by

[ta0siegmund] = cusumarl(h,-mu,0.5,-1)
[ta1siegmund] = cusumarl(h,+mu,0.5,-1)
pause

% and the approximation suggested by Wald is obtained by

[ta0wald] = cusumarl(h,-mu,0.5,-2)
[ta1wald] = cusumarl(h,+mu,0.5,-2)
pause


% The mean times do not say anything about the distribution of the run length,
% which can be quite unsymmetric. Monte Carlo simulations can be used for further
% analysis of the run length function:
%
Nmc=25;
L0=cusumMC(h,-mu,Nmc);
h1=cdfigure(1);
hist(L0,30);
title('Distribution of false alarm times')
L1=cusumMC(h,mu,Nmc);
h2=cdfigure(2,1);
hist(L1,30);
title('Distribution of correct alarm times')
% Compare theoretical with achieved mean times;

disp([ta0 ta0wald ta0siegmund mean(L0)])
disp([ta1 ta1wald ta1siegmund mean(L1)])

pause
close([h1 h2])


% The distribution for false alarms is basically binominal where 
% p=P(reset caused by alarm | reset)  since N0 is small.
% A rough estimate of the delay for detection is L0=h/(theta-nu);
% 
% Finally, what is really wanted is to determine h from a specified 
% mean time for detection L0. This is computed by a numerical search in
%
hmax=10;
L0=10;
h=cusumdesign(L0,mu,hmax)
pause

% Evaluate the design by Monte Carlo simulations
L1=cusumMC(h,mu,Nmc);
mean(L1)


% Finally, we can generate a table:

MU=[-2 -1.5 -1 -0.5 0 0.5 1 1.5 2.0];
h=3;
for i=1:length(MU);
  [ta0,N0] = cusumarl(h,MU(i),0.1);
  [tas,N0] = cusumarl(h,MU(i),0.1,-1);
  [taw,N0] = cusumarl(h,MU(i),0.1,-2);
  L1=cusumMC(h,MU(i),Nmc);
  tab(i,:)=[MU(i),ta0,taw,tas,mean(L1)];
end
echo off
format short e
disp(' ')
disp('       mu       Exact   Wald     Siegmund      MC')
disp(tab(1:3,:))
format short
disp(tab(4:9,:))
disp(' ')
disp('Table 5.1 in Basseville and Nikiforov.')
