% demo_digcom

randn('seed',2)
echo on
clc
%        Demonstration of multiple jump hypothesis
%       ===========================================
%
% Some real world signals are characterized by abrupt changes in
% the parameters or in the noise variance, and it might be
% desirable to discriminate between these changes.
%
% One example is a 2-wire to 4-wire telephone line with an 
% echo return path.
% It is important to adapt to a change in echo path quickly, to
% be able to equalize the echo. On the other hand, so called double
% talk  must not cause fast adaptation.
pause

clc
% Consider a second order FIR model with 4 change times.
% After the first and fourth jump the parameters change,
% corresponding to an echo path change, while
% after the second and third jump the noise variance changes
% corresponding to double talk.
% Let's simulate the signal:

d=2;
TH=[1.6 .64; -1.6 .64; -1.6 .64; -1.6 .64; 1.6 .64;]';
Lam=[1 1 10 1 1];
jumptimes=[50;100;150;200];
N=250;
nnn=[0 2 1];  % Model structure

e=randn(N,1);
u=randn(N,1);
y=simchange([e u],nnn,jumptimes,TH,Lam);
pause

clc
% Now, we estimate the change times and the type of change
%
L=20;        % Length of sliding window

t0=clock;
[jhat,jtype,thseg,lamseg]=multihyp([y u],nnn,L);
t1=etime(clock,t0);

h1=cdfigure(1);
subplot(311)
thplot(thseg,TH,jumptimes)
title('Parameter estimate')
subplot(312)
plot(lamseg)
title('Noise variance estimate')
subplot(313)
segplot(y,jhat)
title('Signal and jump times')
[jhat',jtype']
pause

clc
% As a last experiment, let's do a Monte Carlo simulation.
% Each simulation takes
t1
% seconds. Enter the number of simulations
echo off
nrep=input('nrep = ');
THseg=zeros(d,N);
LAMseg=zeros(1,N);
jumps1=[];
jumps2=[];
jumps3=[];
for i=1:nrep;
  e=randn(N,1);
  u=randn(N,1);
  y=simchange([e u],nnn,jumptimes,TH,Lam);
  [jhat,jtype,thseg,lamseg]=multihyp([y u],nnn,L);
  THseg=THseg+thseg;
  LAMseg=LAMseg+lamseg;
  jumps1=[jumps1 jhat(find(jtype==1))];
  jumps2=[jumps2 jhat(find(jtype==2))];
  jumps3=[jumps3 jhat(find(jtype==3))];
  clc
  disp([' Iteration number ',num2str(i)])
  disp('Estimate of jhat and jtype')
  [jhat',jtype']
end;
thseg=THseg/nrep;
lamseg=LAMseg/nrep;
h2=cdfigure(2,1);
subplot(211),plot(thseg'),title('MC parameter estimate')
subplot(212),plot(lamseg),title('MC noise variance estimate')
h3=cdfigure(3,2);
X=(5:5:N);
subplot(211),hist(jumps1,X),title('Jump times for hypothesis 1')
subplot(212),hist(jumps2,X),title('Jump times for hypothesis 2')
pause
close([h1 h2 h3])



