% script demo_rec3

clc
echo on
%         RECURSIVE TWO-MODEL DETECTION
%        ===============================
%
% There are a number of recursive implementations of the functions
% in DetectTB. These have the prefix 'r', e.g. radfilter.
% The syntax of these functions are almost the same as in the original,
% except that a state has to be included in the input and output arguments.
% The state is initialized by calling the function itself with no data.
% In this way, the user can use his own filters, e.g. EKF, and make
% appropriate actions after a detected change.
%
% In this demo, a first order AR model with one jump is simulated.

randn('seed',1)
N=100;
u=randn(N,1);
e=randn(N,1);
th0=[0.5 1.5;0.8 1.8]';
d=2;
lam=0.1;
nnn=[1 1 1];
y=simchange([lam*e u],nnn,N/2,th0);
pause

% Then, we initialize the recursive versions of filt, one using all data
% and one using a sliding window and detect2

L=10;
[X0]=radfilter([NaN NaN],nnn,1,'RLS');
[X1]=radfilter([NaN NaN],nnn,L,'WLS');
dm=2;
h=10;
nu=lam;
sr=[1 h nu];
[Xdet]=rdetect2([],[],[],[],[],dm,sr);
pause

% Let's also initialize some parameters we want to save for plots

TH0=[];
TH1=[];
LAM=[];
GT1=[];
alarmtimes=[];
pause

% The following loop does what 
%
%     detect2([y u],nnn,dm,sr,-1)
%
% would do.
%
% Note that the task of residual generation and the detector is split into 
% separate functions in contrast to detect2.

% Start recursion
pause
 
for t=1:N;
  [thhat0,lam0,epsi0,S0,X0]=radfilter([y(t) u(t)],nnn,1,'RLS',X0);
  [thhat1,lam1,epsi1,S1,X1]=radfilter([y(t) u(t)],nnn,L,'WLS',X1);
  [alarm,Xdet]=rdetect2(epsi0/sqrt(S0),epsi1/sqrt(S1),lam0,lam1,Xdet);
  GT1=[GT1;Xdet(1)]; 
  TH0=[TH0 thhat0];
  TH1=[TH1 thhat1];
  LAM=[LAM;lam0];
  if alarm~=0; 
    alarmtimes=[alarmtimes t]; 
  end;
end;

% Illustrate the result

h1=cdfigure(1);
plot([TH0' TH1'])
axis([0 N -1 3])
h2=cdfigure(2,1);
segplot([real(GT1)],alarmtimes)
title(['Test statistic and alarm times, Threshold = ',num2str(h)])
pause

% For comparison, we generate the smoothed parameter estimates from SEGM,
% the test statistic off-line from TWOMODEL and the estimates from FILT.

[jhat,thseg]=detectM([y u],nnn);
[jdum,thdum,GT2]=detect2([y u],nnn,dm,sr,-1);
[TH2,LAM2]=adfilter([y u],nnn,0.97);
figure(h1);
plot([thseg' TH0' TH1' TH2'])
axis([0 N -1 3])
figure(h2);
segplot([real(GT1)],alarmtimes)
hold on, plot(real(GT2)), hold off
title(['Test statistic and alarm times, Threshold = ',num2str(h)])

% From figure 1 we conclude that the same estimates are generated
% and from figure 2 that we get several alarms after the change.
pause

% The most appropriate thing to do after the alarm is to increase 
% the tracking ability of the filter. This is done below by increasing
% the covariance matrix of the parameters a factor 10.

[X0]=radfilter([NaN NaN],nnn,1,'RLS');
[X1]=radfilter([NaN NaN],nnn,L,'WLS');
[Xdet]=rdetect2([],[],[],[],[],dm,sr);
TH=[];
LAM=[];
GT1=[];
alarmtimes=[];

% Start recursion
pause

for t=1:N;
  [thhat0,lam0,epsi0,S0,X0]=radfilter([y(t) u(t)],nnn,1,'RLS',X0);
  [thhat1,lam1,epsi1,S1,X1]=radfilter([y(t) u(t)],nnn,L,'WLS',X1);
  [alarm,Xdet]=rdetect2(epsi0/sqrt(S0),epsi1/sqrt(S1),lam0,lam1,Xdet);
  GT1=[GT1;Xdet(1)]; 
  TH=[TH thhat0];
  LAM=[LAM;lam0];
  if alarm~=0; 
    alarmtimes=[alarmtimes t]; 
    X0(:,6:5+d)=10*X0(:,6:5+d);
  end;
end;
h3=cdfigure(3,2);
segplot([real(GT1)],alarmtimes)
title(['Test statistic and alarm times, Threshold = ',num2str(h)])

% This time there is only one alarm.
pause
close(h1)
close(h2)
close(h3)
