function [threc,jumphat,thseg,GT,ta]=detect1(z,nnn,dm,sr,lambda,dsp)

% DETECT1 detects abrupt changes from the one model approach.
%
% [threc,jumphat,thseg,GT,ta]=detect1(z,nnn,dm,sr,lambda,dsp)
%
% z=[y u]   output-input data
% nnn       model structure
% dm        distance measure: 1 = normalized sample mean
%                             2 = normalized squared sample mean
% sr        stopping rule
%           sr=[1 h nu] CUSUM detector with threshold h and drift parameter nu
%           sr=[2 h nu] two-sided CUSUM detector with threshold h and 
%           drift parameter nu
%           sr=[3 h gamma] GMA detector with threshold h and forgetting gamma 
%           sr=[4 h gamma] two-sided GMA detector with threshold h and 
%           forgetting gamma 
% lambda    For parametric models, this is the noise variance.
%           A negative value implies estimation of it.
%           For state space models, it is a scaling of the 
%           noise variance and thus the inverse adaptation gain for KF. 
%           Default: lambda=1
% threc     recursively estimated parameters or states. xhat(t+1|t) for KF.
% jumphat   estimated change points
% thseg     smoothed estimated parameters (not used for state space models)
% GT        The decision function
% dsp       Display info for each recursion (default 0, no info)

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 08-Aug-2003  $

%#
%#  
  

if nargin<6; dsp=0; end;
if nargin<5 lambda=1; end; 
if isempty(lambda) | lambda<0; lambda=1; noiseest=1; else noiseest=0; end;
if nargin<4 sr=-1; end;   if isempty(sr) | sr<0,  sr=[1 10 0]; end
if nargin<3 dm=-1; end;   if isempty(dm) | dm<0,  dm=1; end

if dm~=1 & dm ~=2, error('Incorrect distance measure dm'), end

mtype=modeltype(nnn);
if mtype>4;
  [n,x0,P0]=feval(nnn,0,0,0,0);
  d=n(1);
  nux=n(2);
  nv=n(3);
  nth2=n(5);
  ny=n(6);
  y=z(:,1:ny);
  u=z(:,ny+1:ny+nux);
  tstart=1;
  xhat=x0;
else
  [tstart,d]=z2phi(z,0,nnn);
  A=1; Q=0; B=0; 
  C=1; D=0; R=1;
  P0=lambda*1000*max(max(z))*eye(d);
  th=zeros(d,1);
end;
[N,nz]=size(z);

if mtype==1 & nz~=1; error('Incorrect number of inputs'), return, end
if mtype==2 & nz~=d+1; error('Incorrect number of inputs'), return, end
if mtype==3 & nz~=1; error('Incorrect number of inputs'), return, end
if mtype==4 & nz~=2; error('Incorrect number of inputs'), return, end


gt=0; gtm=0;
h=sr(2);
if sr(1)==1;
    hm=Inf; nu=sr(3);   % One-sided CUSUM
elseif sr(1)==2;
    hm=h;  nu=sr(3);   % Two-sided CUSUM
elseif sr(1)==3;
    hm=Inf; gamma=sr(3);  % One-sided GMA
elseif sr(1)==4;
    hm=h; gamma=sr(3);    % Two-sided GMA
elseif sr(1)==5;
    hm=Inf; q=sr(3);    % Bayes
end;

P=P0;
jumphat=[0];   ta=[];  thseg=[];
threc=zeros(d,N);
GT=zeros(N,2);
if dsp==1; clc; end; 
n=0;
R=1; %lambda; 
adg=1;

for t=tstart:N;
  if mtype>4;
    [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,xhat,0,5);
    [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
    Q=Q*adg;
    L=P*C';
    S=lambda*R+C*L;
    den=inv(S);
    epsi=y(t,:).'-C*xhat;
    if ~isempty(u) & ~isempty(Du);
       epsi=epsi-Du*u(t,:)';
    end
    Epsi(t,:)=epsi.';
    K=L*den;
    xhat=xhat+K*epsi;
    xhat=A*xhat;
    threc(:,t)=xhat;
    if ~isempty(u) & ~isempty(Bu);
       xhat=xhat + Bu*u(t,:)';
    end
    P=A*(P-K*L')*A'+Bv*Q*Bv';
    P=.5*(P+P');
    n=n+1;
    th=xhat;
  else
    [phi,y]=z2phi(z,t,nnn);
    den=inv(lambda*R+phi'*P*phi);
    epsi=y-phi'*th;
    K=P*phi*den;
    th=th+K*epsi;
    threc(:,t)=th;
    P=P-K*phi'*P+Q;
    P=.5*(P+P');
    n=n+1;
  end

    if noiseest==1; 
        lambda=(n-1)/n*lambda+1/n*epsi'*epsi+1e-6;
	% Regularization introduced for low noise data
    end;

    if dm==1;
        [stmp,dtmp,dum]=svd(den);
        st=sum(stmp*diag(sqrt(diag(dtmp)))*epsi)/sqrt(length(epsi));
    elseif dm==2;
        st=epsi'*den*epsi/length(epsi);
    end;
    if sr(1)==1 | sr(1)==2;      %CUSUM
%t,st,den,epsi,R,pause
        gt=max([0; gt+st-nu]);
        gtm=max([0; gtm-st-nu]);
    elseif sr(1)==3 | sr(1)==4;  %GMA
        gt=gamma*gt+(1-gamma)*st;
        gtm=gamma*gtm-(1-gamma)*st;
    elseif sr(1)==5;             %Bayes
        gt=log(q+exp(gt))-log(1-q)+st;
    end;
%    disp([st gt gtm]), pause
    GT(t,:)=[gt gtm];
    if dsp==1;
        home
        disp(['t = ',num2str(t),'      '])
        disp(['gt = ',num2str(gt),'      '])
        disp(['st = ',num2str(st),'      '])
        disp(['jumphat = ']),disp(jumphat)
        disp(th)
    end;
    if (gt>h | gtm> hm) 
        if gt>h; testind=1; else; testind=2; end;
        ta=[ta t];
        if sr(1)>2; 
             ind=find(GT(1:t,testind)<h/10);
        else
             ind=find(GT(1:t,testind)==0);
        end; 
	if isempty(ind), ind=t; end % bug fix
	thisjump=ind(length(ind));  % last reset, GT(:,testind)==0
        jumphat=[jumphat thisjump];
        thseg=[thseg threc(:,jumphat(length(jumphat)))*ones(1,jumphat(length(jumphat))-jumphat(length(jumphat)-1))];
        gt=0;
        gtm=0;
        th=zeros(d,1);
        if mtype>4;
          P=100*P;  % For state space models
        else
          P=P0;     % For parametric models
        end
        n=0; 
    end;
end;
if mtype<5
  thseg=[thseg th*ones(1,t-jumphat(length(jumphat)))];
else
  thseg=[];
end
jumphat=jumphat';
if length(jumphat)>1;   
  jumphat=jumphat(2:length(jumphat));   % remove padded zero
else
  jumphat=[];
end;

if sr(1)==1 | sr(1)==3
  GT=GT(:,1);
end

if nargout==1; jumphat=threc; end
