function [threc,jumphat,thseg,GT,ta]=detect2(z,nnn,dm,sr,L,lambda,dsp)

% DETECT2 detects abrupt changes in parametric models with the sliding window approach.
%
% function [threc,jumphat,thseg,GT,ta]=detect2(z,nnn,dm,sr,L,lambda,dsp)
%
% z=[y u]   output-input data
% nnn        model structure
% L         length of the sliding window
% dm        distance measure: 1 = Brandt's GLR algorithm 
%                             2 = divergence test 
%                             3 = mean
% sr        stopping rule
%           sr=[1 h nu] CUSUM detector with threshold h and drift parameter nu
%           sr=[2 h nu] two-sided CUSUM detector with threshold h and drift parameter nu
%           sr=[3 h gamma] GMA detector with threshold h and forgetting gamma 
%           sr=[4 h gamma] two-sided GMA detector with threshold h and forgetting gamma 
%           sr=[5 h q] Bayes detector with threshold h and jump probability q.
% lambda    noise variance/scaling. Default: estimated
% threc     recursively estimated parameters
% jumphat   estimated change points
% thseg     estimated parameters
% GT        the decision function

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

%#
%#  
  

mtype=modeltype(nnn);
if mtype>4;
  error('detect2 not defined for state space models')
  [n,x0,P0]=feval(nnn,0,0,0,0);
  d=n(1);
  nux=n(2);
  nv=n(3);
  nth2=n(5);
  ny=n(6);
  y=z(:,1:ny);
  u=z(:,ny+1:ny+nux);
  tstart=1;
  xhat=x0;
else
  [tstart,d]=z2phi(z,0,nnn);
  A=1; R1=0; B=0; 
  C=1; D=0; R2=1;
  P0=100;
end;
mtype=modeltype(nnn);
[N,nz]=size(z);

if nargin<7; dsp=0; end;
if nargin<6 lambda=1; end; 
if lambda<0; 
  lambda=1; noiseest=1; 
else 
   noiseest=0; 
end;
lambda0=lambda; lambda1=lambda;
if nargin<5 L=-1;  end;   if L <0,  L=20 ;end
if nargin<4 sr=-1; end;   if sr<0,  sr=[1 10 0]; end
if nargin<3 dm=-1; end;   if dm<0,  dm=1 ;end

gt=0; gtm=0;
h=sr(2);
hm=Inf;
if sr(1)==1;
    hm=Inf; nu=sr(3);   % One-sided CUSUM
elseif sr(1)==2;
    hm=h;  nu=sr(3);   % Two-sided CUSUM
elseif sr(1)==3;
     hm=Inf; gamma=sr(3);  % One-sided GMA
elseif sr(1)==4;
    g0=-Inf; gamma=sr(3);    % Two-sided GMA
elseif sr(1)==5;
    g0=-Inf; q=sr(3);    % Bayes
end;

jumphat=[0];   ta=[];  thseg=[];
th1=zeros(d,1); th0=zeros(d,1);
threc=zeros(d,N);
if mtype==5;
  Pinit=lambda0*P0;
else
  Pinit=lambda0*100*eye(d);
end;
P0=Pinit;
P1=Pinit;
n0=0;          n1=0;
Lambda1=zeros(L,1);  Epsibar1=zeros(L,1);
GT=zeros(N,2);
if dsp==1; clc; end; %figure(1),axis([0 N -1 5]); hold on, end;

for t=tstart:N;
%------ "Update" model M1 by removing the inference about y(t-L) -----
    if t >= tstart+L;
        [phi,y,u]=z2phi(z,t-L,nnn);
        den1old=inv(-Lambda1(1)*R2+phi'*P1*phi);
        epsi1old=y-phi'*th1-D*u;
        K=P1*phi*den1old;
        th1=A*(th1+K*epsi1old)+B*u;
        P1=A*(P1-K*phi'*P1)*A'+Lambda1(1)*R1;
        P1=.5*(P1+P1');
        n1=n1-1;
    end;

%--------- Inference from y(t) -------------
    [phi,y,u]=z2phi(z,t,nnn);

%--------- Update model M0 with y(t) --------------
    den0=inv(lambda0*R2+phi'*P0*phi);
    epsi0=y-phi'*th0-D*u; 
    K=P0*phi*den0;
    th0=A*(th0+K*epsi0)+B*u;
    P0=A*(P0-K*phi'*P0)*A'+lambda0*R1;
    P0=.5*(P0+P0');
    n0=n0+1;
    threc(:,t)=th0;

%--------- Update model M1 with y(t) --------------
    den1=inv(lambda1*R2+phi'*P1*phi);
    epsi1=y-phi'*th1-D*u;
    K=P1*phi*den1;
    th1=A*(th1+K*epsi1)+B*u;
    P1=A*(P1-K*phi'*P1)*A'+lambda1*R1;
    P1=.5*(P1+P1');
    n1=n1+1;
    Epsibar1=[Epsibar1(2:L);epsi1^2*den1];

%----------- Update the estimated noise variance ----
    if noiseest==1; 
        lambda0=(n0-1)/n0*lambda0+1/n0*epsi0^2*den0;
        if n1 == L; 
            lambda1=lambda1+1/n1*(Epsibar1(L)-Epsibar1(1));
        else;
            lambda1=(n1-1)/n1*lambda1+1/n1*epsi1^2*den1;
        end;
    end;
%    t,n1,epsi1old,den1old,epsi1,den1,lambda1,pause
    Lambda1=[Lambda1(2:L-1); lambda1];
%----------- Continue only if M0 and M1 are accurate ------------
if n0>=L & n1 == L;    %t-tstart > 2*L+d;    

%----------- Compute the distance measure ---------

    if dm==1;
        st=0.5*(log(lambda0/lambda1)+epsi0^2*den0/lambda0-epsi1^2*den1/lambda1);
    elseif dm==2;
        st=0.5*(lambda0/lambda1-1 + (1+lambda0/lambda1)*epsi0^2*den0/lambda0- 2*sqrt(den0*den1)*epsi0*epsi1/lambda1);
    elseif dm==3;
        st=epsi1-epsi0;
    end;
%----------- Apply the stopping rule --------------
    if sr(1)==1 | sr(1)==2;      %CUSUM
        gt=max([0; gt+st-nu]);
        gtm=max([0; gt-st-nu]);
    elseif sr(1)==3 | sr(1)==4;  %GMA
        gt=gamma*gt+(1-gamma)*st;
        gtm=gamma*gt-(1-gamma)*st;
    elseif sr(1)==5;             %Bayes
        gt=log(q+exp(gt))-log(1-q)+st;
    end;

    GT(t,:)=[gt gtm];

%------------ Check alarm --------------------
    if (gt>h | gtm> hm) 
        if gt>h; testind=1; else; testind=2; end;
        ta=[ta t];
        if sr(1)>2; 
             ind=find(GT(1:t,testind)<h/10);
        else
             ind=find(GT(1:t,testind)==0);
        end;             
	thisjump=ind(length(ind));
        jumphat=[jumphat thisjump];
        thseg=[thseg th0*ones(1,thisjump-jumphat(length(jumphat)-1))];
        tstart=t+1;
        gt=0;
        gtm=0;
        th1=zeros(d,1);       th0=th1;
        P1=100000*lambda1*eye(d);P0=P1;
        n1=0;                 n0=n1;
        lambda1=lambda;            lambda0=lambda1;
    end;
end;

%---------- Display some info --------------
    if dsp==1;
        home
        disp(['t = ',num2str(t),'      '])
        disp(['gt = ',num2str(gt),'      '])
        disp(['st = ',num2str(st),'      '])
%        disp(['jumphat = ']),disp(jumphat)
        disp(['th0 = ',num2str(th0),'      '])
        disp(['th1 = ',num2str(th1),'      '])
%        disp(['P0 = ',num2str(P0),'      '])
%        disp(['P1 = ',num2str(P1),'      '])
        disp(['lambda0 = ',num2str(lambda0),'      '])
        disp(['lambda1 = ',num2str(lambda1),'      '])
        disp(['n0 = ',num2str(n0),'      '])
        disp(['n1 = ',num2str(n1),'      '])
pause
    end;
end;
thseg=[thseg th0*ones(1,t-jumphat(length(jumphat)))];
jumphat=jumphat';
if length(jumphat)>1;   
  jumphat=jumphat(2:length(jumphat));   % remove padded zero
else
  jumphat=[];
end;

