function [threc,jumphat,thseg,lamseg,Alfa,Xn,V]=detectM(z,nnn,lf,q,ss,dsp)

% Change detection with multiple filters to approximate the MAP/ML estimator
%
% [threc,jumphat,thseg,lamseg,Alfa,Xn,V]=detectM(z,nnn,lf,q,ss,dsp)
%
% z         The output-input data vector z = [y u] or z = [y Phi].
% nnn       Model structure (see help nnn)
% lf        loss function, 
%           lf=[1 lambda]  MAP estimate with known noise scaling lambda
%           lf=[2]  MAP estimate with unknown noise scaling (default)
%           lf=[3]  BIC/MDL penalty term for complexity
%           lf=[4]  AIC penalty term for complexity
% q         The probability that the system jumps at each sample 
%           (default 0.5, which gives ML)
% ss        Search scheme = [M,ll,minseg] 
%           If M=0, a specified search is obtained with the syntax
%           ss=[M jumps], where jumps is a vector of change times.
% M         The number of filters (default 8+na+nb)
% ll        Guaranteed lifelength of a jump sequence (default M-4)
% minseg    The minimum allowed segment size (default 0)
% dsp (0/1) Display some information (default 0)
% threc     Recursive estimate of parameters/states. xhat(t+1|t) for KF.
% jumphat   The estimated jump times
% thseg     Estimated and smoothed piecewise constant parameters.
% lamseg    Estimated and smoothed piecewise constant noise variance.
% Alfa      Log of relative probability of each filter
% Xn        filter histories, used for display in hypplot
% V         Least squares loss function of prediction errors 
% 
% The ML estimate is achieved if q=1/2 (default). The exact ML/MAP estimate 
% of delta is obtained if M=N+1 (and ll=M-2) where N is 
% the number of data. If M is less an approximative local search is
% performed. thseg is the smoothed parameter estimate conditioned on
% deltahat. 
% Note that the syntax is the same as for the parameter estimation methods in SITB,
% e.g. arx and rarx and time series analysis is possible as well by omitting u, nb, nk.
%

% lamseg instead of LAM and default by [] insteaed of -1,  980302

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $
  
%#
%#  
  
mtype=modeltype(nnn);
if mtype>4;
  [n,x0,P0]=feval(nnn,0,0,0,0);
  d=n(1);
  nu=n(2);
  nv=n(3);
  nth2=n(5);
  ny=n(6);
  y=z(:,1:ny);
  u=z(:,ny+1:ny+nu);
  tstart=1;
  xhat=x0;
else
  [tstart,d]=z2phi(z,0,nnn);
  A=1; Q=0; B=0; 
  C=1; Dss=0; R=1;
  P0=1000000000*eye(d);
  x0=zeros(d,1);
  ny=1;
end;
[N,nz]=size(z);

   
fudgefactor=1000000;  % Increase in P for state space models
if nargin<6 dsp=0; end;
if nargin<5 ss=[]; end; 
if nargin<4 q =[]; end; 
if nargin<3 lf=[]; end; 
if isempty(lf) | lf<0; lf=2; end;
if isempty(q) | q<0, q =0.5; end
if isempty(ss) | ss(1)<0, ss=d+8; end
M=ss(1);
if M>1
  if length(ss)>1; ll=ss(2); else ll=[]; end;
  if length(ss)>2; minseg=ss(3); else minseg=[]; end;
  if isempty(ll); ll=M-4; end;
  if isempty(minseg); minseg=d+1; end;
  if ll<3; ll=3; disp('Warning: lifelength ll changed to 3'); end;
  % Create log(gamma(.)) function.
  for i=1:50;
    loggamma(i)=log(gamma(i/2));
  end;
  for i=51:N+ll;
    loggamma(i)=0.5*log(2*pi)-i/2+(i-1)/2*log(i/2);
  end;
elseif M<=1
  jumphat=ss(2:length(ss));
  if isempty(jumphat); jumphat=0; end
  jumphat=jumphat(:);
  M=1; 
  ll=1;
else
  error('The value of ss(1) does not make sense')
end

if lf(1)==1; 
  if length(lf)>1;
    lambda=lf(2); lf=lf(1); 
  else
    disp('Warning: a lambda should be specified for loss function 1 (lambda=1 now)')
    lambda=1; lf=lf(1);
  end
else
  lf=lf(1); lambda=1; 
end;
if lf>1; minseg=max([minseg,d+4]); end;

if dsp==1; clc; end;
alfa=[0 1E6*ones(1,M-1)];
alfabar=zeros(1,M);
th=x0*ones(1,M);
thf=x0*ones(1,M);
epsi=zeros(ny,M);
threc=zeros(d,N);
threcf=zeros(d,N);
hist=zeros(N,M);
numseg=ones(1,M);
age=[0 ll*ones(1,M-1)];
P=[]; for i=1:M,P=[P P0];end;
lamhat=zeros(N,M);
ths=zeros(d*N,M);                    
D=zeros(1,M); 
V=zeros(1,M)+eps;
Vls=zeros(1,M);
Alfa=zeros(N,M);
Xn=ones(N,M);

for t=tstart:N; 
  if mtype>4;
    for j=1:M,
      xhat=thf(:,j);
      Pj=P(:,d*(j-1)+1:d*(j-1)+d);
      % Time update
      [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,xhat,0,5);
      xhat=A*xhat;
      Pj=A*Pj*A'+Bv*Q*Bv';
      th(:,j)=xhat; % Prediction

      % Measurement update
      [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
      den=inv(lambda*R+C*Pj*C');
      epsij=y(t,:)'-C*xhat;
      if ~isempty(u) & ~isempty(Du);
        epsij=epsij-Du*u(t,:)';
      end
      K=Pj*C'*den;
      xhat=xhat+K*epsij;  % Filtered output
      if ~isempty(u) & ~isempty(Bu);
        xhat=xhat + Bu*u(t,:)';
      end
      Pj=Pj-K*C*Pj;
      Pj=.5*(Pj+Pj');
      D(j)=D(j)-log(det(den));
      V(j)=V(j)+epsij'*den*epsij;
      Vls(j)=Vls(j)+epsij'*epsij;
      epsi(:,j)=epsij;
      P(:,d*(j-1)+1:d*(j-1)+d)=Pj;
      thf(:,j)=xhat;  % Filter
    end

  elseif mtype<5;
    [phi,y,u]=z2phi(z,t,nnn);
    for j=1:M,
        Pj=P(:,d*(j-1)+1:d*(j-1)+d);
        denj=inv(lambda*R+phi'*Pj*phi);
        epsij=y-phi'*th(:,j);
        K=Pj*phi*denj;
        th(:,j)=th(:,j)+K*epsij;
        Pdum=Pj-K*phi'*Pj+lambda*Q;
        P(:,d*(j-1)+1:d*(j-1)+d)=.5*(Pdum+Pdum');
        D(j)=D(j)-log(det(denj));
        V(j)=V(j)+epsij'*denj*epsij;
        Vls(j)=Vls(j)+epsij'*epsij;
        epsi(j)=epsij;
    end;
    thf=th; % filter = predictor for linear regressions
  end
  if M>1
    aind=find((age(1:M)>minseg));  % changed from ll 980910 
    if length(aind)>0,
	if lf==1;
		alfabar(aind)=alfa(aind)+D(aind)+V(aind);
	elseif lf==2;
		dum=age(aind)-2; 
		alfabar(aind)=alfa(aind)+D(aind)+dum.*log(V(aind))-2*loggamma(dum);
        elseif lf==3;
                alfabar(aind)=alfa(aind)+age(aind).*log(V(aind)./age(aind)) + d*numseg(aind)*log(t);
        elseif lf==4;
                alfabar(aind)=alfa(aind)+age(aind).*log(V(aind)./age(aind)) + d*numseg(aind)*2;
        elseif lf==5;  
                % New feature: measurements from exponential distribution
                % z(t) = exp(th(t))
                % -log f(y|th)=log(th)+y/th;
                ybar=[];
                for jind=aind;
                  ybar=[ybar mean(z(t+4-age(jind):t))];
                end
%t,ybar,th(:,aind),age(aind)
                alfabar(aind)=alfa(aind)+age(aind).*(1+log(th(:,aind)));  % MGL estimator
%t,alfabar,age(aind)
	end;
	[dummy,jmin]=min(alfabar(aind));
        jmin=aind(jmin);
	threc(:,t)=th(:,jmin);
	threcf(:,t)=thf(:,jmin);
	aind2=find((age(1:M)>ll));
	[dummy,jmax]=max(alfabar(aind2));
        Xn(t+1,:)=abs(Xn(t,:));   % number of jumps in each branch
        Xn(t+1,jmin)=-abs(Xn(t+1,jmin));   % most probable branch
        jmax=aind2(jmax);
%t,jmin,jmax,Xn(t,:)
            if age(jmin)>minseg & age(jmax)>ll;
            % Condition for new branch
                Xn(t,jmax)=abs(Xn(t,jmin));  % deleted branch
                Xn(t+1,jmax)=abs(Xn(t+1,jmin))+1;  % deleted branch
                if mtype<5; %linear regression
                  P(:,d*(jmax-1)+1:d*(jmax-1)+d)=P0;
                  th(:,jmax)=zeros(d,1);            
                elseif mtype>4; % state space model
                  P(:,d*(jmax-1)+1:d*(jmax-1)+d)=fudgefactor*P(:,d*(jmin-1)+1:d*(jmin-1)+d);
                  th(:,jmax)=th(:,jmin);
                end
                if lf<3; 
                    alfa(jmax)=alfabar(jmin)-2*log(q)+2*log(1-q); 
                else;
                    alfa(jmax)=alfa(jmin);
                end;
                D(jmax)=0; V(jmax)=0; Vls(jmax)=Vls(jmin);
                ths(:,jmax)=ths(:,jmin);      
                age(jmax)=0;
                hist(:,jmax)=hist(:,jmin);
                hist(t,jmax)=1;
                numseg(jmax)=numseg(jmax)+1;
                ind=length(find(hist(:,jmax)>0));
                ths(ind*d-d+1:ind*d,jmax)=th(:,jmin);
                lamhat(:,jmax)=lamhat(:,jmin);
                if lf==2;
                    lamhat(ind,jmax)=V(jmin)/(age(jmin)-4); 
                else;
                    lamhat(ind,jmax)=V(jmin)/(age(jmin)+1); 
                end;
            end
        end
	age=age+ones(1,M);
        alfabar=alfabar-min(alfabar);
        Alfa(t,:)=alfabar;
        if dsp~=0;
            lam=V./age;
            clc,t,age,aind,alfabar,jmax,jmin,pause,%th,epsi,pause
        end
  elseif M==1   % Follow pre-defined jumphat path
    threc(:,t)=th;
    threcf(:,t)=thf;
    if ~isempty(find(t==jumphat))  % jump time
      if mtype<5; %linear regression
        P=P0;
        th=zeros(d,1);            
      elseif mtype>4; % state space model
        P=fudgefactor*P;
        th=th;
      end
    end
  end
end

if M>1
  ind=length(find(hist(:,jmin)==1));
  ths(ind*d+1:ind*d+d,jmin)=th(:,jmin);
  deltahat=hist(:,jmin);
  ind=find(deltahat==1);
  dum=length(ind);
  if dum>1; if ind(dum)>N-minseg; ind=ind(1:dum-1); end; end;  %No jumps in the end
  if dum==1; if ind(dum)>N-minseg; ind=[]; end; end; 
  jumphat=ind;
  Xn=Xn(2:N+1,:);
elseif M==1
  jmin=1;
  Xn=[];
end

jumphatx=[0; jumphat; N];

if lf==2;
  lamhat=[lamhat(1:length(jumphat),jmin);  V(jmin)/(age(jmin)-4)];
else;
  lamhat=[lamhat(1:length(jumphat),jmin);  V(jmin)/(age(jmin)+1)];
end;

V=Vls(jmin);
thseg=[];
lamseg=[];
for i=1:length(jumphatx)-1;
  thseg=[thseg ths(i*d-d+1:i*d,jmin)*ones(1,jumphatx(i+1)-jumphatx(i))];
  lamseg=[lamseg lamhat(i)*ones(1,jumphatx(i+1)-jumphatx(i))];
end;

 threc=threcf;  % Output filtered state for consistency
% Output predictions

