function [r,xhat,Ordo,Hu,Hd,Hf,Hv,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc,minorder,dsp);

% Faultdetect using parity spaces
%
% [r,xhat,Ordo,Hu,Hd,Hf,Hv,Null,T]=faultdetect(z,nnn,L,Rstruc,Fstruc,minorder);
%
% Model assumes time invariance and no stochastic terms:
%
% x(t+1) = A x(t) + Bu u(t) + Bd d(t) + Bf f(t)
%   y(t) = C x(t) + Du u(t) + Df f(t)
%
% L        Size of sliding window
% Rstruc   Residual structure
% Fstruc   Fault pattern
% minorder 0/1, Compute minimum order residual filters (default 0)
% dsp     Display information
%
% r       The residuals are zero during non-faulty mode.
%         The optional transformation using R and F makes the residual parallel to
%         column i of R when the fault is parallel to column
%         i of F, and the size of the residual is the fault size.
%

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $


[n,x0,P0]=feval(nnn,0,0,0,0);
[A,Bu,Bv,Bd,Bf,Q]=feval(nnn,0,x0,0,5);
[C,Du,Df,R]=feval(nnn,0,x0,0,6);
nx=n(1);
nu=n(2);
nv=n(3);
nd=n(4);
nf=n(5);
ny=n(6);

[N,nz]=size(z);
y=z(:,1:ny);
u=z(:,ny+1:ny+nu);
if nargin<7;
  dsp=0;
end

if nargin<3;
  L=nx*3;
end

Ordo=C;
Ordotmp=C;
for i=1:L-1;
  Ordotmp=Ordotmp*A;
  Ordo=[Ordo;Ordotmp];
end

Hutmp1=Du;
Hutmp2=Bu;
Hu=[zeros((L-1)*ny,nu);Du];

Dd=zeros(ny,nd);
Hdtmp1=Dd;
Hdtmp2=Bd;
Hd=[zeros((L-1)*ny,nd);Dd];

Hftmp1=Df;
Hftmp2=Bf;
Hf=[zeros((L-1)*ny,nf);Df];

Dv=zeros(ny,nv);
Hvtmp1=Dv;
Hvtmp2=Bv;
Hv=[zeros((L-1)*ny,nv);Dv];
for i=1:L-1;
  Hutmp1=[Hutmp1;C*Hutmp2];
  Hutmp2=A*Hutmp2;
  Hu=[ [zeros((L-1-i)*ny,nu);Hutmp1] Hu];

  Hftmp1=[Hftmp1;C*Hftmp2];
  Hftmp2=A*Hftmp2;
  Hf=[ [zeros((L-1-i)*ny,nf);Hftmp1] Hf];
end

if nd>0; % only for disturbance
  for i=1:L-1;
    Hdtmp1=[Hdtmp1;C*Hdtmp2];
    Hdtmp2=A*Hdtmp2;
    Hd=[ [zeros((L-1-i)*ny,nd);Hdtmp1] Hd];
  end
else
  Hd=[];
end

if nv>0; % only for state noise
  for i=1:L-1;
    Hvtmp1=[Hvtmp1;C*Hvtmp2];
    Hvtmp2=A*Hvtmp2;
    Hv=[ [zeros((L-1-i)*ny,nv);Hvtmp1] Hv];
  end
else
  Hv=[];
end

% ----- Null space ---------
[Usvd,Dsvd,Vsvd]=svd([Ordo Hd]);
n=rank(Dsvd);
Null=Usvd(:,n+1:size(Usvd,2))';
%Null=null([Ordo Hd]')';
%size(Null)
nw=size(Null,1);


% ---- Residual generation -------------

Y=[]; U=[];
for t=1:L;
  Y=[Y;y(t,:)'];
  U=[U;u(t,:)'];
end
for t=L+1:N
  Y=[Y(ny+1:L*ny); y(t,:)'];
  U=[U(nu+1:L*nu); u(t,:)'];
  xhat(:,t)=Ordo\(Y-Hu*U);
  r(:,t)=Null*(Y-Hu*U);
end

% ----- Residual transformation ----------

if nargin>3  % optional
  if nargin>5  & minorder==1 % minimum order
    [Q,Null]=qr(Null);
    dum=size(Null,1);
    Null=Null(dum-size(Fstruc,2)+1:dum,:);
    r=Q'*r;
    dum=size(r,1);
    r=r(dum-size(Fstruc,2)+1:dum,:);
  end
  
  T = Rstruc / ( Null*Hf*kron(ones(L,1),Fstruc));
  if any(  T*( Null*Hf*kron(ones(L,1),Fstruc)) - Rstruc > 1e-8)
     disp('Warning: structured residuals could not be achieved')
  end
  r=T*r;
else
  T=[];
end
r=r';  % Time signals are column vectors