function [threc,jumpsmc,jumphat]=gibbs(z,nnn,lf,q,par,opt,jumpinit)

% Segmentation using Gibbs-Metropolis MCMC resampling with syntax as detectM
%
% [threc,jumpsmc,jumphat]=gibbs(z,nnn,lf,q,par,opt,jumpinit)
%
% z         The output-input data vector z = [y u] or z = [y Phi].
% nnn       Model structure (see help nnn)
% lf        loss function, 
%           lf=[1 lambda]  MAP estimate with known noise scaling lambda
%           lf=[2]  MAP estimate with unknown noise scaling (default)
%           lf=[3]  BIC/MDL penalty term for complexity
%           lf=[4]  AIC penalty term for complexity
% q         The probability that the system jumps at each sample 
%           (default 0.5, which gives ML)
% jumpsmc   The jump times for each Gibbs iteration
% jumphat   The most likely jump sequence encountered in the search.
% par=[n M prior] Number of jumps, Gibbs iterations and prior
%           on the jump times (prior=0 uniform, >0 Gaussian with
%           standard deviation=prior)
% opt=[o1 o2 o3] on-line (o1) or off-line (o2) jump plot
% and histogram (o3) with assumed burnin-time 0.25*M.
% jumpinit  Initial estimate of jump sequence
% The ML estimate is achieved if q=1/2 (default). 

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if nargin>4
   n=par(1);
   if length(par)>1
      M=par(2);
   end
   if length(par)>2
      prior=par(3);
   end
end

o1=0; o2=1; o3=0;
if nargin>5
   o1=opt(1);
   if length(opt)>1, o2=opt(2); end
   if length(opt)>2, o3=opt(3); end
end
N=length(z);
if nargin<7
   kn=ceil(N*rand(n,1));
else
   kn=jumpinit;
end

KN=zeros(n,M);
[jumphat,thseg,lamseg,threc,Alfa,Xn,V]=detectM(z,nnn,lf,q,[1;kn(:)]);
Vmin=V; knmin=kn;
p= exp(-2*V);
if o1
   figure
   set(gca,'FontSize',14)
   plot(0,0)
   axis([0,M+1,0,N]),
%   set(gca,'erasemod','xor')
   xlabel('Iteration number')
   ylabel('Change times')
   hold on,
end

for m=1:M
   for i=1:n
      if prior==0
         ktest=ceil(N*rand(1,1));
      elseif prior>1
         ktest=kn(i)+round(prior*randn(1,1));
         if ktest<1; ktest=1; end
         if ktest>N; ktest=N; end
      end
      
      knbar=kn;
      knbar(i)=ktest;
      [jumphat,thseg,lamseg,threc,Alfa,Xn,Vbar]=detectM(z,nnn,lf,q,[1;knbar(:)]);
      pbar= exp(-2*Vbar);
      if Vbar<V | Vbar-V<-0.5*log(rand(1,1));
%      if pbar>p | pbar/p>(rand(1,1));
         kn=knbar;
         p=pbar;
         V=Vbar;
      end
      if o1
         if Vbar<Vmin-1e-12;
            h=plot(m+i/n*ones(size(kn)),kn,'o');
         else
            h=plot(m+i/n*ones(size(kn)),kn,'x');         
         end
%         set(h,'erasemod','xor')
         drawnow
      end
      if Vbar<Vmin-1e-12;
         Vmin=Vbar; knmin=knbar;
      end
   end
   KN(:,m)=kn(:);
end
hold off
if o3
   figure
   set(gca,'FontSize',14)
   KN
   dum=KN(:,M/4:M)
   hist(dum(:),N)
end
if o2
   figure
   set(gca,'FontSize',14)
   axis('off')
   ha=axes('position',[0.1 0.1 0.65 0.8]);
   plot(KN','x'),
   axis([0,M,0,N]),
   disp(['Estimated jump sequence is ',cdnum2str(knmin)])
   hold on
   plot(ones(M,1)*knmin(:)','--')
   hold off
   xlabel('Iteration number')
   ylabel('Change times')
   title('Iteration plot')
   ha=axes('position',[0.8 0.1 0.15 0.8]);

   dum=KN(:,round(M/4):M);
   [No,Xo]=hist(dum(:),1:N);
   intw=Xo(2)-Xo(1);
   No=[0 No 0];
   Xdum(1:2:2*length(Xo))=Xo-intw/2;
   Xdum(2:2:2*length(Xo))=Xo+intw/2;
%   Xdum=kron(Xo,[1 1]);
   Xdum=[0 Xdum(1) Xdum Xdum(length(Xdum)) N];
   plot(kron(No,[1 1]),Xdum);
   axis([0 max(No)+1 0 N])
   title('Histogram')
%   set(ha,'xdir','reverse')
end


[threc,jumphat,thseg,lamseg,Alfa,Xn,Vdum]=detectM(z,nnn,lf,q,[1;knmin(:)]);
jumpsmc=KN';
jumphat=knmin;
