function  [Xhat,jumphat,ta,LR,NUhat,mu]=glr(z,nnn,adg,M,h,pnu);

%  GLR estimates the jump time for a change in the state of a linear system
%
% [Xhat,jumphat,ta,LR,nuhat,mu]=glr(z,nnn,adg,M,h,pnu);
%
%  Computes the log likelihood ratios of a jump at time k versus no jump 
%  according to the formulas in Willsky and Jones (1976).
%
% jumphat  estimated jump time, no jump if jumphat=0
% LR       log likelihood ratio = log p(Y|k)/p(Y)
% nuhat    estimated jump
% mu       Influence of nuhat on xhat(end)
% ta       alarm times
% f        number of flops
% z        output-input data
% nnn      structure parameters
% adg      Adaptation gain in Kalman filter
% M        size of sliding window of considered change times. 
%          If M=0 or N, then all time instants are considered (default). 
%          If M=1, then a Gaussian prior on the jump is used.
%          If M<0 (e.g. M=-N), then the change point is estimated as a batch test
% h        threshold in the test (default 10)
% pnu      Covariance of the jump. Default is unknown jump (pnu=1000).
%

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

[N,nz]=size(z);

if nargin<3; adg=1; end
if nargin<4; M=0; end
if nargin<5; h=10; end;
if nargin<6; pnu=100; end;
if M<0; M=-M; recflag=0; else; recflag=1; end

mtype=modeltype(nnn);
if mtype>4;
  [n,x0,P0]=feval(nnn,0,0,0,0);
  d=n(1);
  nu=n(2);
  nv=n(3);
  nd=n(4);
  nth2=n(5);
  ny=n(6);
  y=z(:,1:ny);
  u=z(:,ny+1:ny+nu);
  tstart=1;
else
  [tstart,d]=z2phi(z,0,nnn);
  A=1; Bv=0; Bu=0; Q=0; 
  C=1; Du=0; R=1;
  P0=100000000;
  ny=1;
end;
y=z(:,1:ny);
u=z(:,ny+1:size(z,2));
P=adg*P0;
Xhat=zeros(d,N); 
lamhat=zeros(N,1);
n=0;

xhat=zeros(d,1);
lr=zeros(N,1);
LR=zeros(N,1);
P=P0;
RN=kron(ones(1,N),0.001*eye(d));
fN=zeros(d,N);
Xhat=zeros(d,N);
phi=zeros(N,d);
mu=zeros(d,N*d);
lr=zeros(N,1);
kstart=1;
ta=[];
jumphat=[];
NUhat=[];

for t=tstart:N; 
    if mtype>4;
      [A,Bu,Bv,Bd,Bth,Q]=feval(nnn,t,xhat,0,5);
      [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
    else
      [phi,ydum]=z2phi(z,t,nnn);
      C=phi';    
    end
    Q=Q*adg;
    L=P*C';
    S=R+C*L;
    den=inv(S);
    epsi=y(t,:).'-C*xhat;
    if ~isempty(u) & ~isempty(Du);
       epsi=epsi-Du*u(t,:).';
    end
    Epsi(t,:)=epsi.';
    K=L*den;
    xhat=xhat+K*epsi;
    Xhat(:,t)=xhat;
    xhat=A*xhat;
    if ~isempty(u) & ~isempty(Bu);
       xhat=xhat + Bu*u(t,:)';
    end
    P=A*(P-K*L')*A'+Bv*Q*Bv';
    P=.5*(P+P');
  if M>1; kstart=max(1,t-M); end
  % Matched filters
  for k=kstart:t-1; 
    % update \varphi_{t}(k) and \mu_{t}(k)
    % according to (35)-(36) with t<->t+1
    % Here A=A_{t-1}, K=K_t and C=C_t
    phi(k,:)=C*(A^(t-k)-A*mu(:,k*d-d+1:k*d));    
    mu(:,k*d-d+1:k*d)=K*phi(k,:)+A*mu(:,k*d-d+1:k*d);
    % update f_N(k) and R_N(k) according to LS
    RN(:,k*d-d+1:k*d)=phi(k,:)'*den*phi(k,:)+RN(:,k*d-d+1:k*d);
    fN(:,k)=phi(k,:)'*den*epsi+fN(:,k);
    if M==1;
      nuhat=inv(RN(:,k*d-d+1:k*d))*fN(:,k);
      lr(k)=fN(:,k)'*nuhat - log(det(RN(:,k*d-d+1:k*d))) -nuhat'*nuhat/pnu; 
    else
      lr(k)=(fN(:,k)'*inv(RN(:,k*d-d+1:k*d))*fN(:,k))/2;
    end
  end;
  % Apply the GLR test
  [lrmax,newjump]=max(lr(kstart:t-d-1));
  if ~isempty(lrmax); LR(t)=lrmax; end
  newjump=newjump+kstart-1;
  % recursive change detection test, otherwise batch test
  if recflag;  
    % If alarm, update jumphat, ta and the state vector
    if lrmax>h;
      Pnu=inv(RN(:,newjump*d-d+1:newjump*d));
      nuhat=Pnu*fN(:,newjump);
      ta=[ta t];
      jumphat=[jumphat newjump];
      NUhat=[NUhat nuhat];
      % Compute the transition matrix between newjump and t
      Phi=eye(d);
      for k=newjump:t-1;
        Phi=(A-mu(:,k*d-d+1:k*d))*Phi;
      end;
      Phi;
      % Compensate the state vector
      xhat=xhat+Phi*nuhat;
      P=1000*(P+Phi*Pnu*Phi');
      RN=kron(ones(1,N),0.001*eye(d));
      fN=zeros(d,N);
      phi=zeros(N,d);
      mu=zeros(d,N*d);
    end;
  end
  % Start a new hypothesis
  RN(:,t*d-d+1:t*d)=C'*den*C+inv(eye(d));
  fN(:,t)=C'*den*epsi;
  phi(t,:)=C*A;
  mu(:,t*d-d+1:t*d)=K*C*A;
end;

if ~recflag  % batch test
  [dum,jumphat]=max(lr);
  if lr(jumphat)>h;
    mu=mu(:,jumphat*d-d+1:jumphat*d);
    Pnu=inv(RN(:,jumphat*d-d+1:jumphat*d));
    NUhat=Pnu*fN(:,jumphat);
    jumphat=jumphat-1;
  else;
    jumphat=0;
  end;
end

lr(1:N-2)=lr(2:N-1);
lr(N-1)=0;
