function [uhat,fhat,z]=lineq(y,neq,alphabet,delay,utrain)
%
% LINEQ implements the Viterbi algorithm for equalization
%
% [uhat,thhat]=lineq(y,neq,alphabet,delay,utrain)
%   
% Assumes that there is a training sequence available.
%
% y        Output from channel
% utrain   If provided, utrain is the training sequence
%          and th the FIR order, and thhat is the FIR estimate
% uhat     Estimated input
% z        Equalizer output before decision
% fhat     Estimated equalizer FIR parameters in a column vector
% neq      Equalizer FIR order.
% alphabet Finite input alphabet
% delay    Delay in channel/equalizer   
%
% Limitations: 
%  1. Only FIR equalizers
%  2. Only scalar channels
%  3. Training sequence assumed to be initially in y
%
% See also viterbi, implementing a linear equalizer with a similar syntax

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

[N,m]=size(y);
Ntrain=length(utrain);

alphabet=alphabet(:)';
if nargin<4 | isempty(delay);
  delay=neq/2;
end

% Least squares with pre- and post-windowing
ustart=neq-delay+1;
ystart=neq+1;
if ustart<1;
  ystart=ystart+1-ustart;
  ustart=1;
end
Y=utrain(ustart:Ntrain-delay);
Phi=[];
for i=0:neq;
  Phi=[Phi y(ystart-i:Ntrain-i)];
end
fhat=Phi\Y;

z=filter(fhat,1,y);

[dum,ind]=min(abs(z*ones(1,length(alphabet))-ones(N,1)*alphabet)');

uhat=alphabet(ind)';

if nargin>4   % uhat for training sequence is set to "0"
  uhat=[zeros(Ntrain,1);uhat(Ntrain+1:N)];
end
