function [jumphat,lr,f]=mlr(z,nnn,lf,L)

%  MLR estimates the jump time for a change in the state of a linear system
%
%  [jumphat,lr,f]=mlr(z,nnn,lf,L)
%
%  Computes the log likelihood ratios of a jump at time k versus no jump 
%  according to formulas given in 
%  Willsky and Jones, The Generalized Likelihood Ratio Test for Change Detection, AC-41, no 1.
%
% jumphat  estimated jump time, no jump if jumphat=0
% lr       log likelihood ratio = log p(Y|k)/p(Y)
% f        number of flops
% z        output-input data
% nnn      structure parameters
% lf       loss function
% lf = [1 lambda];  known noise level
% lf = 2;  unknown constant noise level
% lf = 3;  unknown changing noise level  (Default)
% L        number of ignorance data at the end (Default length(H))
% P0 < 0   stationary covariance       
%

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if ~isstr(nnn), error('nnn must be a state space model'), end
[n,x0,P0]=feval(nnn,0,0,0,0);
d=n(1);
nu=n(2);
nv=n(3);
nth2=n(4);
ny=n(5);
y=z(:,1:ny);
u=z(:,ny+1:ny+nu);
tstart=1;
xhat=x0;
y=z(:,1:ny);
u=z(:,ny+1:size(z,2));
P=adg*P0;
Xhat=zeros(d,N); 
lamhat=zeros(N,1);
n=0;


if nargin<3; lf=3; end;
if nargin<4; L=d; end;

if lf(1)>4;
	R=R/R(1,1);
	if nargin<8; Elam=.01; Vlam=1; end;
	m=4+Elam^2/Vlam;
	sigma=(m-2)*Elam;
end;

m=-2; sigma=0;  %flat prior on lambda
N=length(y);
MAP=zeros(N,1);
Vf=zeros(N+1,1);Vb=zeros(N+1,1);Vn=zeros(N+1,1);
Dn=zeros(N+1,1);Df=zeros(N+1,1);Db=zeros(N+1,1);
P00=1000*eye(d);
mod=0;  %Assume nonsingular F

if P0<0;
	P0=zeros(d,d);
	for i=1:100;
		P0=F*P0*F'+Q;
	end;
	mod=1
else;
	mod=0;
end; 


flops(0);
% forward filter
map=0;
xhat=zeros(d,1);
P=P0;
Pi=P0;
for t=tstart:N;
    [A,Bu,Bv,Bth,Q]=feval(nnn,t,xhat,0,5);
    [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
    den=inv(C*P*C'+R);
    epsi=y(t,:)'-C*xhat;
    if ~isempty(u) & ~isempty(Du);
       epsi=epsi-Du*u(t,:)';
    end
    K=P*C'*den;
    xhat=A*(xhat+K*epsi);
    if ~isempty(u) & ~isempty(Bu);
       xhat=xhat + Bu*u(t,:)';
    end
    P=F*(P-K*C*P)*F'+Bv*Q*Bv';
    if t<N-L; Pi=A*Pi*A'+Bv*Q*Bv'; end;
    Vf(t+1)=Vf(t)+epsi'*den*epsi;
    Df(t+1)=Df(t)-log(det(den));
end;
Vf(1:N)=Vf(2:N+1);
Df(1:N)=Df(2:N+1);

% Normalization filter
xhat=zeros(d,1);
P=Pi;
for t=N-L+1:N;
    [A,Bu,Bv,Bth,Q]=feval(nnn,t,xhat,0,5);
    [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
    den=inv(C*P*C'+R);
    epsi=y(t,:)'-C*xhat;
    if ~isempty(u) & ~isempty(Du);
       epsi=epsi-Du*u(t,:)';
    end
    K=P*C'*den;
    xhat=A*(xhat+K*epsi);
    if ~isempty(u) & ~isempty(Bu);
       xhat=xhat + Bu*u(t,:)';
    end
    P=F*(P-K*C*P)*F'+Bv*Q*Bv';
    Vn(t)=Vn(t-1)+epsi'*den*epsi;
    Dn(t)=Dn(t-1)-log(det(den));
end;
%Vf(1)=Vf(N+1); Df(1)=Df(N+1);

% backward filter
% First information filter on backward model
map=0;
ahat=zeros(d,1);
Pinv=zeros(d,d);
GB=inv(F)*G;
FB=inv(F);
QB=FB*G*Q*G'*FB';
for t=N:-1:N-L+1;
    [A,Bu,Bv,Bth,Q]=feval(nnn,t,xhat,0,5);
    [C,Du,Dth,R]=feval(nnn,t,xhat,0,6);
    den=inv(C*P*C'+R);
    epsi=y(t,:)'-C*xhat;
    if ~isempty(u) & ~isempty(Du);
       epsi=epsi-Du*u(t,:)';
    end
    K=P*C'*den;
    xhat=A*(xhat+K*epsi);
    if ~isempty(u) & ~isempty(Bu);
       xhat=xhat + Bu*u(t,:)';
    end
    P=F*(P-K*C*P)*F'+Bv*Q*Bv';
	% Measurement update
	Pinv=Pinv+C'*inv(R)*C;      % hit kom jag
	ahat=ahat+C'*inv(R)*y(t,:)'; 
	% Time update
	if mtype==5;
  	  Adum=F'*Pinv*F;             % using F=inv(FB) etc
	  Bdum=Adum*GB*inv(GB'*Adum*GB+inv(Q));
	  Pinv=(eye(d)-Bdum*GB')*Adum;    
	  ahat=(eye(d)-Bdum*GB')*(F'*ahat-Adum*B*u) ;  
	end;
end;


% then Kalman filter on backward model
P=inv(Pinv);
xhat=P*ahat;
for t=N-L:-1:tstart;
        [phi,y,u]=z2phi(z,t,nnn);
	H=phi';
	den=inv(H*P*H'+R);
	epsi=y-H*xhat;
	K=P*H'*den;
	if t>2; 
            xhat=FB*(xhat+K*epsi-B*u);     % E[x(t)|y(t),...,y(N)]
            P=FB*(P-K*H*P)*FB'+QB;
        end;
	Vb(t)=Vb(t+1)+epsi'*den*epsi;
	Db(t)=Db(t+1)-log(det(den));
end;

% Compute a posteriori probabilities
jumptimes=[];
if lf(1)==1;
	jumptimes=tstart:N-L-1;
	for t=jumptimes;
		MAP(t)=.5*(Vf(t)+Vb(t+1)+Df(t)+Db(t+1));
	end;
	MAP(N)=.5*(Vf(N)-Vn(N)+Df(N)-Dn(N));
end;
if lf(1)==2;
	MAP(N)=-log(gamma((m+N-L)/2))+.5*(Df(N)-Dn(N)+(m+N-L)*log(Vf(N)-Vn(N)+sigma));
	jumptimes=5:N-L-5;
	for t=jumptimes;
		MAP(t)=-log(gamma((m+t-1)/2))-log(gamma((m+N-L-t-1)/2))+.5*(Df(t)+Db(t+1)+(m+t)*log(Vf(t)+sigma)+(m+N-L-t-1)*log(Vb(t+1)+sigma));
	end;
end;
if lf(1)==3;
	MAP(N)=.5*(Df(N)-Dn(N)+(m+N-L)*log(Vf(N)-Vn(N)+sigma));
	jumptimes=tstart:N-L-1;
	for t=jumptimes;
		MAP(t)=.5*(Df(t)+Db(t+1)+(m+N-L)*log(Vf(t)+Vb(t+1)+sigma));
	end;
end;
%Df(1:2),Db(1:2),Vf(1:2),Vb(1:2)
V=[Df,Db,Vf,Vb];

f=flops;
[dum,jumphat]=min(MAP(jumptimes));
jumphat=jumptimes(jumphat);
if MAP(jumphat)>MAP(N); jumphat=0; end;
lr=[MAP(N)-MAP(1:N-L-1); zeros(L+1,1)];
if lf==2; lr([1:4,N-L-5+1:N])=zeros(1,9+L); end;





