function [jumptime,jumptype,thseg,lambdaseg]=multihyp(z,nnn,L,hypmask,q,dsp)

% MULTIHYP detects abrupt changes in the parameters and/or noise variance
% with the sliding window approach.
%
% function [jtime,jtype,thseg,lamseg]=multihyp(z,nnn,L,hypmask,q,dsp)
%
% jumptime  detected jump times
% jumptype  1 for jump in parameters, 
%           2 for jump in noise variance and
%           3 for jump in both
% z=[y u]   output-input data
% nnn       model structure
% L         length of the sliding window. Default L=50.
% hypmask   possibility to mask, e.g. [0 1 0] means only variance
%           changes considered. Default: [1 1 0].
% q         Prior probability for a jump at time t (default q=0.5 (ML))
% dsp       display some information every dsp sample. Default dsp=0.
%
% Note: note yet prepared for state space models

if nargin<6; dsp=0; end;
if nargin<5; q=0.5; end;
if nargin<4; hypmask=[1 1 0]; end;
if nargin<3; L=50; end;


[tstart,d]=z2phi(z,0,nnn);
mtype=modeltype(nnn);
[N,nz]=size(z);

p0=1;
if mtype==5;
  Pinit=P0;
else
  Pinit=p0*eye(d);
end;
P0=Pinit; P1=Pinit;
Q=zeros(d);
R=1;
jumptime=[0];   ta=[];  
jumptype=[];
thseg=zeros(d,N); 
lambdaseg=zeros(1,N); 
th1=zeros(d,1); th0=zeros(d,1);
prior=log(det(P0));
qprior=log((1-q)/q);    % Prior against a jump
f0=zeros(d,1); f1=f0;
R0=1/p0*eye(d); R1=R0;
y2sum0=0;      y2sum1=0;
n0=0;          n1=0;
Rp=zeros(d,d); fp=zeros(d,1); Vp=0;
nth=0;   nlam=0;
tthp=0;  tlamp=0;
tthpp=0; tlampp=0;
ldiff=0; ncount=0;
Hhat=0;
epsi1old=0;

for t=tstart:N;
%--------- Update model M0 with y(t-L) --------------
    if t >= tstart + L;
        [phi,y,u]=z2phi(z,t-L,nnn);
        den0=inv(R+phi'*P0*phi);
        epsi0=y-phi'*th0; 
        K=P0*phi*den0;
        th0=th0+K*epsi0;
        P0=P0-K*phi'*P0+Q;
        P0=.5*(P0+P0');
        f0=f0+phi*y;
        R0=R0+phi*phi';
        y2sum0=y2sum0+y*y;
        n0=n0+1;
    end;

%--------- "Update" model M1 by removing the inference about y(t-L) ----------
    if t >= tstart + L;
        [phi,y,u]=z2phi(z,t-L,nnn);
        den=inv(-R+phi'*P1*phi);  % Note the minus sign on R!
        epsi1old=y-phi'*th1;
        K=P1*phi*den;
        th1=th1+K*epsi1old;
        P1=P1-K*phi'*P1+Q;
        P1=.5*(P1+P1');
        f1=f1-phi*y;
        R1=R1-phi*phi';
        y2sum1=y2sum1-y*y;
        n1=n1-1;
    end;

%--------- Inference from y(t) -------------
    [phi,y,u]=z2phi(z,t,nnn);
 
%--------- Update model M1 with y(t) --------------
    den1=inv(R+phi'*P1*phi);
    epsi1=y-phi'*th1;
    K=P1*phi*den1;
    th1=th1+K*epsi1;
    P1=P1-K*phi'*P1+Q;
    P1=.5*(P1+P1');
    f1=f1+phi*y;
    R1=R1+phi*phi';
    y2sum1=y2sum1+y*y;
    n1=n1+1;


%----------- Continue only if M0 and M1 are accurate ------------
    if n0>=L & n1 == L;    %t-tstart > 2*L+d;    


%----------- Compute the distance measure ---------
      V0=y2sum0-2*f0'*th0+th0'*R0*th0;
      V1=y2sum1-2*f1'*th1+th1'*R1*th1;
      V1th0=y2sum1-2*f1'*th0+th0'*R1*th0;
      Vtot=y2sum0+y2sum1-(f0+f1)'*inv(R0+R1)*(f0+f1);
      % H1: no jump
      l1=(n0+n1-2)*log(Vtot/(n0+n1-4))+log(det(R0+R1))+prior;
      % H2: possible jump in theta
      l2=(n0+n1-2)*log((V0+V1)/(n0+n1-4))-log(det(P0)) -log(det(P1))+2*prior+ 2*qprior; 
      % H3: possible jump in lambda
      l3=(n0-2)*log(V0/(n0-4)) + (n1-2)*log(V1th0/(n1-4)) - 2*log(det(P0))+2*prior + 2*qprior;
      % H4: possible jump in theta and lambda
      l4=(n0-2)*log(V0/(n0-4)) + (n1-2)*log(V1/(n1-4)) - log(det(P0)) - log(det(P1))+2*prior + 2*qprior; 
      % Mask hypothesis
      l2=hypmask(1)*l2+(l1+1)*(1-hypmask(1));
      l3=hypmask(2)*l3+(l1+1)*(1-hypmask(2));
      l4=hypmask(3)*l4+(l1+1)*(1-hypmask(3));

%------------ Check alarm --------------------
      Hhatp=Hhat;
      [lmin,Hhat]=min([l1 l2 l3 l4]);
      ldiffp=ldiff;
      ldiff=l1-lmin;      % positive after a jump
      if ldiff>0; 
        ncount=ncount+1; 
      else
        ncount=0; 
      end;
      if (ldiffp>ldiff) & (ncount>0.75*L) & (Hhat==Hhatp);
	% Accept the alarm only if
	% 1) the difference is decreasing, i.e. the peak is passed
	% 2) the difference has been positive for 75 % of a window length
	% 3) the estimated hypothesis has remained the same for two samples
        jumptime=[jumptime t-L];
        jumptype=[jumptype Hhat-1];

        Rp=Rp+R0;
        fp=fp+f0;
        if R0>0; Vp=Vp+y2sum0-f0'*inv(R0)*f0; else; Vp=Vp; end;
        nlam=nlam+n0;
        nth=nth+n0;
        if nlam>0; lamp=Vp/nlam; else; lamp=0; end;
        thp=inv(Rp)*fp;
%[thp lamp nth nlam th0 th1]
        thseg(:,1:t-L)=[thseg(:,1:tthp) thp*ones(1,t-L-tthp)];
        lambdaseg(:,1:t-L)=[lambdaseg(:,1:tlamp) lamp*ones(1,t-L-tlamp)];
        if Hhat==2;     % jump in theta
          Rp=zeros(d,d);
          fp=zeros(d,1);
          nth=0;
          tthpp=tthp;    tthp=t-L;
          tlampp=tlampp; tlamp=tlamp;
        elseif Hhat==3; % jump in lambda
          Vp=0;
          nlam=0;
          tthpp=tthpp;   tthp=tthp;
          tlampp=tlamp;  tlamp=t-L;
        elseif Hhat==4; % jump in theta and lambda
          Rp=zeros(d,d);
          fp=zeros(d,1);
          nth=0;
          Vp=0;
          nlam=0;
          tthpp=tthp;    tthp=t-L;
          tlampp=tlamp;  tlamp=t-L;
        end

        tstart=t+1;
        gt=0;
        gtm=0;
	th0=th1;	th1=zeros(d,1);       
	P0=P1;		P1=p0*eye(d);        
	n0=n1;		n1=0;
        f0=f1;          f1=0;
        R0=R1;		R1=1/p0*eye(d);
	y2sum0=y2sum1;	y2sum1=0;             
    end;
end;

%---------- Display some info --------------
    if dsp>0;
      if t/dsp==round(t/dsp);
        clc,home
        disp(['t = ',num2str(t),'      '])
        disp(['th0 = ',num2str(th0),'      '])
        disp(['th1 = ',num2str(th1),'      '])
        disp(['P0 = ',num2str(P0),'      '])
        disp(['P1 = ',num2str(P1),'      '])
        disp(['n0 = ',num2str(n0),'      '])
        disp(['n1 = ',num2str(n1),'      '])
        disp(['l1 = ',num2str(l1),'      '])
        disp(['l2 = ',num2str(l2),'      '])
        disp(['l3 = ',num2str(l3),'      '])
        disp(['l4 = ',num2str(l4),'      '])
        dum=min([length(jumptime) 6]);
        disp(['Six first jumptimes = ']),disp(jumptime(1:dum))
        pause
      end;
    end;
end;

Rp=Rp+R0;
fp=fp+f0;
if n0>0;
  Vp=Vp+y2sum0-f0'*inv(R0)*f0;
end;
nlam=nlam+n0;
nth=nth+n0;
lamp=Vp/nlam;
thp=inv(Rp)*fp;
%[thp lamp nth nlam th0 th1]
tthpp=tthp;   tthp=t;
tlampp=tlamp; tlamp=t;
thseg(:,1:tthp)=[thseg(:,1:tthpp) thp*ones(1,tthp-tthpp)];
lambdaseg(:,1:tlamp)=[lambdaseg(:,1:tlampp) lamp*ones(1,tlamp-tlampp)];

jumptime=jumptime(2:length(jumptime));

 

