function nnn

% nnn denotes the model structure for data z.
% 6 different model types, mtype=0--6
% See also modeltype
%
% 0. nnn = 0 and z = y;
%    Assumes noise with piecewise constant variance
%    y(t) = e(t)
%
% 1. nnn = -1 and z = y;
%    Assumes a piecewise constant mean model 
%    y(t) = th(t) + e(t)
%
% 2. nnn = -2 and z = [y Phi]
%    Assumes a piecewise constant linear regression model 
%    y(t)=Phi(t,:)*th(t) + e(t)
% 
% 3. nnn = na and z = y
%    Assumes a piecewise constant AR(na) model:
%    y(t) = - th(1)*y(t-1) - .. - th(na)*y(t-na) + e(t)
%
% 4. nnn = [na nb nk] and z = [y u]
%    Assumes a piecewise constant ARX(na,nb,nk) model:
%    y(t) = - th(1)*y(t-1) - .. - th(na)*y(t-na) + 
%           + th(na+1)u(t-nk) + .. + th(na+nb)*u(t-nk-nb) + e(t)
%
% 5. Assumes a linear state space model with additive changes th, input u and state noise v.
%    and z = [y u]
%
%     x(t+1) = A*x(t) + Bu*u(t) + Bv*v(t) + Bd*d(t) + Bth*th(t) 
%      y(t)  = C*x(t) + Du*u(t) +    e(t)           + Dth*th(t)
%      x(0)  = x0
%     E(vv') = Q
%     E(ee') = lambda*R
%     E(x0x0') = P0
%     f=[th;lambda]
%     n = [ nx nu nv nd nf ny];
% 
%    A more compact description is
%     x(t+1) = A*x(t) + B*ubar(t)
%      y(t)  = C*x(t) + D*ubar(t) + e(t)
%    where ubar=[u; th; v], B=[Bu Bth Bv], D=[Du Dth 0].
%
%    Model definition
%    nnn='filename' is a general user specified state space models using
%    simulinks S-function syntax:
%
%      [n,x0,P0]=nnnplane(0,0,0,0);
%      xp=nnnplane(t,x,f,2);
%      y =nnnplane(t,x,f,3);
%      [A,Bu,Bv,Bd,Bth,Q]=nnnplane(t,x,f,5);
%      [C,Du,Dth,R]=nnnplane(t,x,f,6);
%
%    Note that the last two optional calls are not standard simulink,
%    but used to increase speed in filtering when supplied.
%
%    For state space model (5), the matrix th and vector jumptimes define the 
%    amplitude and time for the additive changes.
%    For the linear regression models (1-4), jumptimes define the segments where the
%    parameter vector is piecewise constant and th the corresponding parameters.
%
%    This structure is used for 
%    -- simulation: y=simchange(z,nnn), where y in z is replaced by e.
%    -- filtering:  [thhat,epsi]=filt(z,nnn,ff), with forgetting ff.
%    -- detection using a sliding window: [jumphat,thhat]=detect1(z,nnn,dm,sr)
%    -- detection using a sliding window: [jumphat,thhat]=detect2(z,nnn,L,dm,sr)
%    -- detection using the GLR test:     [jumphat,thhat]=glr(z,nnn,L)
%    -- detection using the MLR test:     [jumphat,thhat]=mlr(z,nnn)
%    -- MAP segmentation:                 [jumphat,thhat]=detectM(z,nnn)

help nnn
