function [sys,x0,str,ts] = nnnssex(t,x,u,flag)

% nnnssex Example state-space M-file S-function with internal A,B,C,D matrices
%   This S-function implements a system described by state-space equations:
%   
%           x(t+1) = A*x + B*u
%                y = C*x + D*u
%   
%   where x is the state vector, u is vector of inputs, and y is the vector
%   of outputs. The matrices, A,B,C,D are embedded into the M-file.
%   
% This is a replica of a Simulink template, modified for the
% Adaptive filtering and change detection toolbox.
% Extra flags are added for the purpose of filtering
%
% Simulation works as usually with input [u;d;f] and B=[Bu Bd Bf]
%
%           x(t+1) = A*x + [Bu Bd Bf]*[u;f]
%                y = C*x + [Du Dd Df]*[u;f]
% That is, the implicit function calls used in Simulink and e.g. simchange are
%     x(t+1)=nnnssex(t,x,[u;f],2);
%       y(t)=nnnssex(t,x,[u;f],3);
% Note that process noise v, disturbance d, and measurement noise y 
% have to be generated within the function, but that simchange and
% the GUI can affect the fault.
%
% Filtering uses flags 5, 6, 7
%     [A,Bu,Bv,Bd,Bf,Q]=nnnssex(t,x,u,5);
%     [C,Du,Df,R]=nnnssex(t,x,u,6);
%     [n,x0,P0]=nnnssex(0,0,0,7);
%     %n=[nx nu nv nd nf ny];


% State-space matrices. Edit here! 
A = [-0.3000         0         0
      2.9000   -0.6200   -2.3000
           0    2.3000         0];

Bu = [1; 0; 0];  
Bd = [0; 0; 0];  % Zero vector for non-existing signals!
Bv = [0 0; 0 1;0 1];  % Process noise 2D
Bf = [1; 0; 0];  % Here input fault

C = [1 1 0; 1 -3 1];

Du = [0; 1];
Dd = [0; 0];
Df = [0; 0];

Q=[0.001 0; 0 0.0001];
R=eye(2);

nx=size(A,1);
nu=size(Bu,2);
nv=size(Bv,2);
nd=0; %size(Bd,2); % Note, zero size for non-existing signals!
nf=size(Bf,2);
nv=size(Bv,2);

n=[nx nu nv nd nf ny];

switch flag,

  %%%%%%%%%%%%%%%%%%
  % Initialization %
  %%%%%%%%%%%%%%%%%%
  case 0     
    [sys,x0,str,ts] = mdlInitializeSizes(A,B,C,D);                                

  %%%%%%%%%%%%%%%
  % Derivatives %
  %%%%%%%%%%%%%%%
  case 1,
    sys = mdlDerivatives(t,x,u,A,B,C,D);

  %%%%%%%%%%%
  % Outputs %
  %%%%%%%%%%%
  case 3,
    sys = mdlOutputs(t,x,u,A,B,C,D);
 
  %%%%%%%%%%%%%
  % Terminate %
  %%%%%%%%%%%%%
  case 9                                                
    sys = []; % do nothing

end

%end nnnssex

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts] = mdlInitializeSizes(A,B,C,D)

sizes = simsizes;

sizes.NumContStates  = 0;
sizes.NumDiscStates  = size(A,1);
sizes.NumOutputs     = size(C,1);
sizes.NumInputs      = size([Bu Bf],2);
sizes.DirFeedthrough = any(any(Du~=0));
sizes.NumSampleTimes = 1;

sys = simsizes(sizes);
x0  = ones(size(A,1),1); 
str = [];
ts  = [0 0];

% end mdlInitializeSizes
%
%=============================================================================
% mdlDerivatives
% Return the derivatives for the continuous states.
%=============================================================================
%
function sys = mdlDerivatives(t,x,u,A,B,C,D)

sys = A*x + B*u;

% end mdlDerivatives
%
%=============================================================================
% mdlOutputs
% Return the output vector for the S-function
%=============================================================================
%
function sys = mdlOutputs(t,x,u,A,B,C,D)

sys = C*x + D*u;

% end mdlOutputs


