function [thhat,lamhat,epsi,S,X,K]=radfilter(z,nnn,adg,adm,X)

% RADFILTER  recursive implementation of ADFILTER
%
% function [thhat,lamhat,epsi,S,X,K]=radfilter(z,nnn,adg,adm,X)
%
% z=[y u]   output-input data
% nnn       model structure
% adg       Adaption gain, forgetting factor, step size or R1.
% adm       Adaptation method: RLS (default), WLS, LMS, NLMS or KF
% ff        forgetting factor used in RLS and/or to estimate lambda
% thhat=xhat estimated parameter/state vector for each t
% lamhat    estimated noise variance for each t (requires ff<1)
% Epsi      the prediction errors for each t
% X         state vector that must be provided. 
%           If z equals the empty vector, then X is initialized, e.g.
%           X=radfilter([],nnn);
%           X"="[d,lamhat,mtype,n,th,P,Z], Z is the d latest z (stacked)
%
% This function is useful in combination with recursive change detectors
% to find out if there are any abrupt changes.

  
if nargin<3; adg=1; adm='RLS'; end;

if ~isstr(adm)
  if adm==1;
    adm='RLS';
  elseif adm==2;
    adm='WLS';
  elseif adm==3;
    adm='LMS';
  elseif adm==4;
    adm='NLMS';
  elseif adm==5;
    adm='KF';
  end
end

mtype=modeltype(nnn);
[N,nz]=size(z);
if isempty(z) | isnan(z(1));         % Initialize state
  mtype=modeltype(nnn);
  if mtype>4;
    if isstr(nnn)==0;
      [A,B,C,D,R1,R2,P0,x0]=nnn2ss(nnn);
    else
      eval(['[A,B,C,D,R1,R2,P0,x0]=',nnn,'(0)']);
    end;
    d=size(A,1);
    L=0;
    P=P0;
    xhat=x0; % zeros(d,1);
    Z=zeros(L*d+d,length(z));   % Memory of old data for creating phi
    lamhat=0;
    n=0;
    X=[[d lamhat mtype n; zeros(L*d+d-1,4)] [xhat P;zeros(L*d,d+1)] Z ] ;
  else    % initialise regression filter
    if strcmp(adm,'WLS'); L=adg; else L=0; end
    [tstart,d]=z2phi(z,0,nnn);
    P=100*eye(d);
    thhat=zeros(d,1);
    phi=zeros(d,1);
    Z=zeros(L*d+d,length(z));   % Memory of old data for creating phi
    lamhat=0;
    n=0;
    X=[[d lamhat mtype n; zeros(L*d+d-1,4)] [thhat P;zeros(L*d,d+1)] Z] ;
  end;
  thhat=X;
else               % Unpack state

  if mtype>4;   % State space model
    L=0;
    d=X(1,1); lamhat=X(1,2); mtype=X(1,3); n=X(1,4);
    [mx,nx]=size(X); 
    xhat=X(1:d,5);
    P=X(1:d,6:5+d);
    Z=X(:,6+d:nx);
    Z=[Z(2:L*d+d,:);z];  % Shift in new data
    ff=adg;
    n=n+1;
    if isstr(nnn)==0;
      [A,B,C,D,R1,R2,P0]=nnn2ss(nnn);
    else
      eval(['[A,B,C,D,R1,R2]=',nnn,'(n,xhat)']);
    end;
    ny=size(C,1);
    y=z(1:ny);
    y=y(:);
    if length(z)>ny;
      u=z(ny+1:length(z)); u=u(:);
    else
      u=0;
    end
    M=P*C';
    S=R2+C*M;
    den=inv(S);
    epsi=y-C*xhat;
    K=M*den;
    xhat=A*(xhat+K*epsi)+B*u;
    P=A*(P-K*M')*A'+R1;
    P=.5*(P+P');
    if ff<1;
      lamhat=ff*lamhat+(1-ff)*epsi'*den*epsi;
    else
      lamhat=(n-1)/n*lamhat+1/n*epsi'*den*epsi;
    end
    X=[[d lamhat mtype n; zeros(L*d+d-1,4)] [xhat P;zeros(L*d,d+1)] Z] ;
    thhat=xhat;
  
  else        % Regression model
    
    if nargin<4; adm='RLS'; end;
    ff=1; R1=0; L=0; 
    d=X(1,1); lamhat=X(1,2); mtype=X(1,3); n=X(1,4);
    if strcmp(adm,'RLS')
      ff=adg; 
      if ff>1|ff<0; ff=0.98; disp('ff changed to 0.98'); end
    elseif strcmp(adm,'LMS') | strcmp(adm,'NLMS')
      mu=adg;
    elseif strcmp(adm,'WLS')
      L=adg;
      if L<1; L=10; disp('L changed to 10'); end
    elseif strcmp(adm,'KF')
      R1=adg*eye(d);
    else
      disp('Not a valid method adm')
      disp('adm set to RLS with adg=0.98!')
      ff=0.98;
      adm='RLS';
    end;
    [mx,nx]=size(X); 
    thhat=X(1:d,5);
    P=X(1:d,6:5+d);
    Z=X(:,6+d:nx);
    Z=[Z(2:L*d+d,:);z];  % Shift in new data
    n=n+1;
    [phi,y,u]=z2phi(Z,d+L*d,nnn);
    epsi=y-phi'*thhat;
    if strcmp(adm,'RLS') |strcmp(adm,'KF') |strcmp(adm,'WLS')
      M=P*phi;
      S=ff+phi'*M;
      den=inv(S);
      K=M*den;
      P=P-K*M'+R1;
      P=.5/ff*(P+P');
    elseif strcmp(adm,'LMS')
      K=mu*phi;
    elseif strcmp(adm,'NLMS')
      K=mu/(mu/1000+phi'*phi)*phi;
    end;
      
    thhat=thhat+K*epsi;

    if ff<1; 
      lamhat=ff*lamhat+(1-ff)*epsi'*epsi;
    else
      n=n+1;
      lamhat=(n-1)/n*lamhat+1/n*epsi'*epsi;      
    end
    if strcmp(adm,'WLS');         % Sliding window
      if n>L;             % Remove influence of y(t-L)
        [phi,y,u]=z2phi(Z,d,nnn);
	M=P*phi;
        S=-ff+phi'*M;            % Note the minus sign
        den=inv(S);
        epsi=y-phi'*thhat;
        K=M*den;
        thhat=thhat+K*epsi;
        P=P-K*M'+R1;
        P=.5/ff*(P+P');
        lamhat=lamhat-1/n*epsi'*epsi;
	n=n-1;
      end
    end
    X=[[d lamhat mtype n; zeros(L*d+d-1,4)] [thhat P;zeros(L*d,d+1)] Z] ;
  end;	
end;


