function [alarm,X]=rdetect1(epsi,X,dm,sr,lambda)

% RONEMODEL recursive version of ONEMODEL
%
% function [alarm,X]=rdetect1(epsi,X)
%
% alarm     equals 0 if no alarm and +/-1 otherwise 
%           (+1 for the test m>0 and -1 for the test m<0)
% epsi      normalized innovation, Var(epsi)=1 under H0
% X         state vector initialized when epsi=[]
%
%           X=rdetect1([],[],dm,sr,lambda);
%
%           X=[gt,gtm,n,lambda,dm,nu,gamma,q,h,hm,noiseest]
% dm        distance measure: 1 = normalized sample mean
%                             2 = normalized squared sample mean
% sr        stopping rule
%           sr=[1 h nu] CUSM detector with threshold h and drift parameter nu
%           sr=[2 h nu] two-sided CUSM detector with threshold h and 
%           drift parameter nu
%           sr=[3 h gamma] GMA detector with threshold h and forgetting gamma 
%           sr=[4 h gamma] two-sided GMA detector with threshold h and 
%           forgetting gamma 
% lambda    noise variance/scaling. Default: estimated


%           Initialize state vector
if isempty(epsi);  
  if nargin<5 lambda=-1; end; 
  if nargin<4 sr=-1; end;   
  if nargin<3 dm=-1; end;   
  if lambda<0; lambda=1; noiseest=1; else noiseest=0; end;
  if sr<0,  sr=[1 10 0]; end
  if dm<0,  dm=1 ;end
  gt=0; gtm=0;
  h=sr(2);
  hm=1; nu=1; gamma=1; q=1; %dummy values
  if sr(1)==1;
    stoprule=1; hm=Inf; nu=sr(3);   % One-sided CUSUM
  elseif sr(1)==2;
    stoprule=1; hm=h;  nu=sr(3);   % Two-sided CUSUM
  elseif sr(1)==3;
    stoprule=2; hm=Inf; gamma=sr(3);  % One-sided GMA
  elseif sr(1)==4;
    stoprule=2; gamma=sr(3);    % Two-sided GMA
  elseif sr(1)==5;
    stoprule=3; hm=Inf; q=sr(3);    % Bayes
  end;
  n=0;
  X=[gt,gtm,n,lambda,dm,nu,gamma,q,h,hm,noiseest,stoprule];
  alarm=X;
else
  gt=X(1); gtm=X(2); n=X(3); lambda=X(4); dm=X(5); nu=X(6);
    gamma=X(7); q=X(8); h=X(9); hm=X(10); noiseest=X(11); stoprule=X(12);
    n=n+1;
    if noiseest==1; 
        lambda=(n-1)/n*lambda+1/n*epsi^2;
    end;
    if dm==1;
        st=epsi;
    elseif dm==2;
        st=epsi^2;
    end;
    if stoprule==1;                 %CUSUM
        gt=max([0; gt+st-nu]);
        gtm=max([0; gt-st-nu]);
    elseif stoprule==2;             %GMA
        gt=gamma*gt+(1-gamma)*st;
        gtm=gamma*gt-(1-gamma)*st;
    elseif stoprule==3;             %Bayes
        gt=log(q+exp(gt))-log(1-q)+st;
    end;
    if (gt>h | gtm> hm);
        if gt>h; alarm=1; else; alarm=-1; end;
        gt=0;
        gtm=0;
        lambda=1;
	n=0; 
    else
        alarm=0;
    end;
    X=[gt,gtm,n,lambda,dm,nu,gamma,q,h,hm,noiseest,stoprule];
end;

