function [alarm,X]=rdetect2(epsi0,epsi1,lam0,lam1,X,dm,sr)

% RTWOMODEL recursive version of TWOMODEL
%
%function [alarm,X]=rdetect2(epsi0,epsi1,lam0,lam1,X,dm,sr)
%
% alarm     equals 0 if no alarm and +/-1 otherwise 
%           (+1 for the test m>0 and -1 for the test m<0)
% epsi      normalized innovation, Var(epsi)=1 under H0
% lam       estimated noise variance (or fixed value)
% 0/1       0 model with long time horizon and 1 with a short one,
%           typically a sliding window
% X         state vector initialized when epsi0=[]
%
%           X=rdetect2([],[],[],[],[],dm,sr);
%
%           X=[gt,gtm,dm,nu,gamma,q,h,hm]

% dm        distance measure: 1 = Brandt's GLR algorithm 
%                             2 = divergence test 
%                             3 = mean
% sr        stopping rule
%           sr=[1 h nu] CUSM detector with threshold h and drift parameter nu
%           sr=[2 h nu] two-sided CUSUM detector with threshold h and drift parameter nu
%           sr=[3 h gamma] GMA detector with threshold h and forgetting gamma 
%           sr=[4 h gamma] two-sided GMA detector with threshold h and forgetting gamma 
%           sr=[5 h q] Bayes detector with threshold h and jump probability q.

if isempty(epsi0);
  % Initialize state vector
  if nargin<9 sr=-1; end;   
  if nargin<8 dm=-1; end;   

  if sr<0,  sr=[1 10 0]; end
  if dm<0,  dm=1 ;end
  
  gt=0; gtm=0;
  h=sr(2);
  hm=1; nu=1; gamma=1; q=1; %dummy values
  if sr(1)==1;
    stoprule=1; hm=Inf; nu=sr(3);   % One-sided CUSUM
  elseif sr(1)==2;
    stoprule=1; hm=h;  nu=sr(3);   % Two-sided CUSUM
  elseif sr(1)==3;
    stoprule=2; hm=Inf; gamma=sr(3);  % One-sided GMA
  elseif sr(1)==4;
    stoprule=2; g0=-Inf; gamma=sr(3);    % Two-sided GMA
  elseif sr(1)==5;
    stoprule=3; g0=-Inf; q=sr(3);    % Bayes
  end;
  X=[gt,gtm,dm,nu,gamma,q,h,hm,stoprule];
  alarm=X;
else
  % Recursion
  %------ Unpack X -----------------------
  gt=X(1); gtm=X(2); 
  dm=X(3); nu=X(4); gamma=X(5); q=X(6); h=X(7); hm=X(8); stoprule=X(9);

  %----------- Compute the distance measure ---------
  if lam0>0 & lam1>0;    % Continue only if enough data
    if dm==1;
      st=0.5*(log(lam0/lam1)+epsi0^2/lam0-epsi1^2/lam1);
    elseif dm==2;
      st=0.5*(lam0/lam1-1 + (1+lam0/lam1)*epsi0^2/lam0- 2*epsi0*epsi1/lam1);
    elseif dm==3;
      st=epsi1-epsi0;
    end;
  else
    st=0;
  end;
  %----------- Apply the stopping rule --------------
  if stoprule==1;                    %CUSUM
    gt=max([0; gt+st-nu]);
    gtm=max([0; gt-st-nu]);
  elseif stoprule==2;                %GMA
    gt=gamma*gt+(1-gamma)*st;
    gtm=gamma*gt-(1-gamma)*st;
  elseif stoprule==3;                %Bayes
    gt=log(q+exp(gt))-log(1-q)+st;
  end;

  %------------ Check alarm --------------------
  if (gt>h | gtm> hm) 
    if gt>h; alarm=1; else; alarm=-1; end;
    gt=0;
    gtm=0;
  else
    alarm=0;
  end;
  X=[gt,gtm,dm,nu,gamma,q,h,hm,stoprule];
end;

