function  [alarm,X,nuhat,jumphat,lrmax,Pnu]=rglr(epsi,S,K,nnn,X,M,h);

%  GLR recursive implementation of GLR using a sliding window M.
%
%  [alarm,X,nuhat,jumphat,lrmax,Pnu]=rglr(epsi,S,K,nnn,X);
%
%  Computes the log likelihood ratios of a jump at time k versus no jump 
%  according to the formulas in Willsky and Jones (1976).
%
% alarm    equals 0 if no alarm and +/-1 otherwise 
% jumphat  estimated jump time is t-jumphat (OBS), no jump if jumphat=0
% nuhat    estimated jump
% X        state vector initialized when epsi=[]
%          X=rglr(z,nnn,[],M,h);
%          where X="[M,h,xhat,P,phi,mu]"
%
% epsi     Kalman filter innovation (under th no jump hypothesis)
% S        covariance matrix for epsi
% K        Kalman gain
% nnn      structure parameters (A,B,C,D etc)
%          these may be time-varying, but some changes needed first
% M        size of sliding window of considered change times. 
% h        threshold in the test (default 10)
%          Default is unknown jump (pnu=100).
%
% Fredrik Gustafsson, 960325, fredrik@isy.liu.se

if isempty(epsi);  
  % Initialize state vector
  if nargin<6; M=10; end
  if nargin<7; h=10; end;
  [A,B,C,D,Q,R,P0]=nnn2ss(nnn);
  n=length(C);
  RN=kron(0.001*ones(1,M),eye(n));
  fN=zeros(n,M);
  phi=zeros(n,M);
  mu=zeros(n,M*n);
  X=[[M h;zeros(n-1,2)] phi,fN,mu,RN];
  alarm=X;
else
  % Time recursion
  [A,B,C,D,Q,R,P0]=nnn2ss(nnn);
  M=X(1,1); h=X(1,2); 
  [n,dum]=size(X);
  phi=X(:,3:2+M);
  fN=X(:,3+M:2+2*M);
  mu=X(:,3+2*M:2+M*(n+2));
  RN=X(:,3+M*(n+2):2+M*(2*n+2));
  Sinv=inv(S);

  % Matched filters
  % Shift
  for k=M-1:-1:1;
    phi(:,k+1)=phi(:,k);
    mu(:,(k+1)*n-n+1:(k+1)*n)=mu(:,k*n-n+1:k*n);
    RN(:,(k+1)*n-n+1:(k+1)*n)=RN(:,k*n-n+1:k*n);
    fN(:,k+1)=fN(:,k);
  end;
  % Update
  for k=2:M;
    phi(:,k)=(C*(A^(k)-A*mu(:,k*n-n+1:k*n)))';
    mu(:,k*n-n+1:k*n)=K*phi(:,k)'+A*mu(:,k*n-n+1:k*n);
    RN(:,k*n-n+1:k*n)=phi(:,k)*Sinv*phi(:,k)'+RN(:,k*n-n+1:k*n);
    fN(:,k)=phi(:,k)*Sinv*epsi+fN(:,k);
  end;
  % Initialize
  RN(:,1*n-n+1:1*n)=C'*Sinv*C+inv(eye(n));
  fN(:,1)=C'*Sinv*epsi;
  mu(:,1*n-n+1:1*n)=K*C;

  % GLR estimates
  for k=1:M;
    lr(k)=(fN(:,k)'*inv(RN(:,k*n-n+1:k*n))*fN(:,k))/2;
  end;
  [lrmax,jumphat]=max(lr);
  if lrmax>h;
    alarm=1;
    Pnu=inv(RN(:,jumphat*n-n+1:jumphat*n));
    nuhat=Pnu*fN(:,jumphat);
    jumphat=jumphat-1;
    RN=kron(0.001*ones(1,M),eye(n));
    fN=zeros(n,M);
    phi=zeros(n,M);
    mu=zeros(n,M*n);
  else;
    alarm=0;
    jumphat=0;
    nuhat=zeros(n,1);
    Pnu=[];
  end;
  X=[[M h;zeros(n-1,2)] phi fN mu RN];
end;
