function [y,th0]=simchange(z,nnn,jumptimes,TH,Lambda,ipm,ippar)

% SIMCHANGE simulates a linear system with abrupt changes
%
% function [y,th0]=simchange(z,nnn,jumptimes,TH,Lambda,ipm,ippar,option)
%
% y         Simulated output
% th0       Time-varying parameter or state vector
% z         data matrix equal to [e], [e u], [e Phi] or [u]
%           For state space models, z=u or z=[u d] when the disturbance
%           is deterministic.
% nnn       model structure
% jumptimes times for abrupt changes
% TH/nu     parametric changes
%           For regression models, TH contains the parameter vectors
%           in each segment, so the number of columns in TH equals
%           the number of elements in jumptimes plus one. For state
%           space models nu=TH has as many columns as there are jumps
% Lambda    scaling of R
% ipm       Interpolation method for the impulsive changes in TH
%           imp='no'  No interpolation (default for parametric models)
%           imp='lc'  Interpolated change from jumptime to jumptime+ipw
%           imp='ic'  Integrated change (default for state space models)
% ippar=[L n] Interpolation window size L and degree n used for interpolated
%           changes (see above). n=1 gives linear interpolation.
%
% For state space models, Gaussian noise with covariances Q and R
% are created.

% Copyright Fredrik Gustafsson, Sigmoid AB
% $  Revision: 1.0, 07-Aug-2003  $

if nargin<4; TH=[]; end;
if nargin<3; jumptimes=[]; end;
if nargin<5; Lambda=[]; end;
if isempty(Lambda); Lambda=ones(length(jumptimes)+1,1); end;

mtype=modeltype(nnn);

[N,nz]=size(z);
if nz==1 & N==1;  % No inputs
  N=z; nz=0;
end
[nth,mth]=size(TH);

if mtype<5;  % regression model
  if nargin<6
    ipm='no';
  end
  if strcmp(ipm,'lc')  
    if nargin<7 
      ippar=[5 1];
    end
    ipL=ippar(1);
    ipn=ippar(2);
  end

  [tstart,d]=z2phi(z,0,nnn);
  if nth~=d;
    disp(['TH has ',num2str(nth),' rows while parameter vector has size ',num2str(d)])
    error('Specified model order does not match TH'),return
  end;
  if mth~=length(jumptimes)+1;
    disp(['TH has ',num2str(mth),' columns while number of jumps are ',num2str(length(jumptimes))])
    error('Specified number of jump times does not match TH'),return
  end;
  if mtype==1 & nz~=1; error('Incorrect number of inputs'), return, end
  if mtype==2 & nz~=d+1; error('Incorrect number of inputs'), return, end
  if mtype==3 & nz~=1; error('Incorrect number of inputs'), return, end
  if mtype==4 & nz~=2; error('Incorrect number of inputs'), return, end

  y=zeros(N,1);
  thjump=zeros(nth,N);
  if mth>1;  % jump exists
    thjump(:,[1;jumptimes(:)]')=[TH(:,1) diff(TH')'];
    if strcmp(ipm,'lc');
      for i=1:ipn;
        thjump=filter(ones(1,ipL)/ipL,1,thjump')';
      end
    end
  end
  th=TH(:,1);
  if length(Lambda)==1;
    sqrtlambda=sqrt(Lambda)*ones(N,1);
  else
    Lambda=[1;Lambda(:)]';
    sqrtlambda=segm2par(sqrt(Lambda),[1;jumptimes(:)]',N);
  end
%  sqrtlambda=sqrt(Lambda(1));
  zyu=z;   % Different z in simchange and z2phi

  for t=tstart:N;
     if mtype==3 | mtype==4
      zyu(1:t-1,1)=y(1:t-1); % to be able to compute the AR part of phi
     end
    
    [phi,e,u,w]=z2phi(zyu,t,nnn); 
    th=th+sqrtlambda(t)*w;
    y(t)=phi'*th+sqrtlambda(t)*e;
    th0(:,t)=th;
    th=th+thjump(:,t);

%    if ~isempty(jumptimes); 
%      if ~isempty(jumptimes); ind=find(t==jumptimes); else; ind=[]; end;
%      if ~isempty(ind);
%        th=TH(:,ind+1);    % Linear regressions
%        sqrtlambda=sqrt(Lambda(ind+1)); % Noise variance change
%      end
%    end;
  end
else
  %----State space model----
  if nargin<6
    ipm='ic';  % integrated change
  end
  if strcmp(ipm,'li')  % not used currently
    if nargin<7 
      ippar=[5 1];
    end
    ipL=ippar(1);
    ipn=ippar(2);
  end
  % Initialize
  [n,x0,P0]=feval(nnn,0,0,0,0);
  nx=n(1);
  nu=n(2);
  nv=n(3);
  nd=n(4);
  nth2=n(5);
  ny=n(6);
  if length(Lambda)==1;
     lam0=Lambda*ones(N,1);
  elseif length(Lambda)==length(jumptimes);  % add a 1 before first jump 
    Lambda=[1 Lambda];
    lam0=segm2par(Lambda,jumptimes,N);
  else
    lam0=segm2par(Lambda,jumptimes,N);
  end
  th0=zeros(nth,N);
  th0(:,jumptimes)=TH;
  if strcmp(ipm,'ic');
    th0=cumsum(th0')';
  end
  if nth~=nth2 & mth>0;
    error(['Specified state order ',num2str(nth),...
	   ' does not match the number of rows ',num2str(nth2),' in TH'])
  end;
  if mth~=length(jumptimes);
    error('Specified number of jump times does not match the columns in TH')
  end;

  y=zeros(ny,N);
  xhat0=zeros(nx,N);
  xhat=x0;
  sqrtlambda=sqrt(Lambda(1));
%  Recursion
  for t=1:N;
    if nz==0;
      utmp=[th0(:,t); lam0(t)];
    else
      utmp=[z(t,:)'; th0(:,t); lam0(t)];
    end
% if any(t==jumptimes-1),t,utmp,end
    yp=feval(nnn,t,xhat,utmp,3);    % y(t)
    xhat=feval(nnn,t,xhat,utmp,2);  % x(t+1)
    y(:,t)=yp;
    xhat0(:,t)=xhat;
  end
  y=y';
  th0=xhat0;
end;
