function [z,w,T]=th2tfd(thm,lamhat,y,wbin,tbin,plottype);

% TH2TFD plots the time-frequency description of an adaptively estimated  model
%
% [z,w,T]=th2tfd(threc,lamhat,y,wbin,tbin,plottype);
%
% Computes the transfer function of the recursively estimated 
% AR model in threc, plots the result.
% lamhat, if provided, is the local energy in the AR model. 
% y, if provided, is the original signal used only for plotting.
% wbin is the number of frequency bins and tbin the number of time bins.
% Defaults are 32 for both.
% Both wbin and tbin could be vectors of actual bin values.
% w and T are the frequency and time bins.
%
% With no output arguments, one (or more if plottype is a vector) 
% of the following is obtained:
% plottype(1)=1  Imagesc  plot       (default)
% plottype(1)=2  Waterfall plot
% plottype(1)=3  Contour plot
% plottype(2)=1  Projections plotted (default)
% 
% Note: sampling interval 1 assumed.
%
% Fredrik Gustafsson

thm=thm';
[N,na]=size(thm);

if nargin<6; plottype=[1 1]; end;
if length(plottype)==1; plottype(2)=1; end
if nargin<5; tbin=[]; end; if isempty(tbin); tbin=32; end
if nargin<4; wbin=[]; end; if isempty(wbin); wbin=32; end
if nargin<3; y=[]; end; if isempty(y); y=zeros(N,1); end
if nargin<2; lamhat=[]; end; if isempty(lamhat); lamhat=ones(N,1); end

if length(wbin)==1;
  w=logspace(-1,pi,wbin); %pi*(0:wbin-1)/wbin;
else
  w=wbin;
  wbin=length(w);
end
if length(tbin)==1;
  tstep=N/tbin;
  T=tstep:tstep:N;
else
  T=tbin;
  tbin=length(T);
end

z=zeros(tbin,wbin);
for t=1:tbin;
  z(t,:)=freqz(1,[1 thm(round(T(t)),:)],w)*lamhat(round(T(t)));
end;

if nargout==0;
 if plottype(2)==1;
  clf
  ha=axes('position',[0.3 0.3 0.65 0.65]);
  if find(plottype(1)==1)
    imagesc(T,w(wbin:-1:1),log10(abs(z(:,wbin:-1:1)')));
%    imagesc(T,w,log10(abs(z')));
%    gca
    set(ha,'YDir','normal')
%    hset(ha)
    xlabel('Time'), ylabel('Frequency [rad/s]'), 
  elseif find(plottype(1)==2)
    waterfall(w,T,log10(abs(z)));
    view([20,45]);
    ylabel('Time'), xlabel('Frequency [rad/s]'), 
  elseif find(plottype(1)==3)
    contour(T,w,log10(abs(z'))), 
    set(gca,'YDir','normal')
    xlabel('Time'), ylabel('Frequency [rad/s]'),
  end
  axes('position',[0.3 0.05 0.65 0.15]);
  plot(y)   
  title('Signal plot')
  ha=axes('position',[0.05 0.3 0.15 0.65]);
  wfft=2*pi*(1:N/2)/N;
  Phiy=abs(fft(y)).^2;
  semilogx(Phiy(1:N/2),wfft);
  title('Periodogram')
  a=axis;
  axis([a(1) a(2) 0 wfft(length(wfft))])
  set(ha,'xdir','reverse')
%  colormap('gray')
 else
  if find(plottype(1)==1)
    %gca
    imagesc(T,w(wbin:-1:1)/pi,log10(abs(z(:,wbin:-1:1)')));
    set(gca,'YDir','normal')
%   set(ha,'xdir','reverse')
%    imagesc(T,w,log10(abs(z')));
%    hset(ha)
    xlabel('Time'), ylabel('Frequency [rad/s]'), 
  elseif find(plottype(1)==2)
    waterfall(w,T,log10(abs(z)));
    view([20,45]);
    ylabel('Time'), xlabel('Frequency [rad/s]'), 
  elseif find(plottype(1)==3)
    contour(T,w,log10(abs(z'))), 
    xlabel('Time'), ylabel('Frequency [rad/s]'), 
  end
 end
end;



