saveplots=1;

%    Demostration of design of CUSUM detectors
%   ===========================================
%
% Assume we observe a signal y(t) which is N(0,sigma^2) before the change and
% N(theta,sigma) after the change. We apply the CUSUM algorithm with threshold h
% and drift nu. A succesful design requires of course that theta > nu/2.
% Assume in this example that theta = nu.
%
% The average run length (ARL) function for a stopping rule for 
% detecting a change theta in the mean of a signal is defined as
% 
%        E(ta|theta)
% 
% where ta is the stopping time.
% There are two design parameter in the CUSUM test, the threshold h
% and the drift parameter nu. If sigma is the standard deviation of the noise, then
%
%             /   h      theta-nu/2 \
%     ta = f |  ----- , ------------ |
%             \ sigma     sigma     /
%
% That is, it is a function of two arguments. We can assume that sigma=1
% and, for simplicity, nu = theta (a standard choice of drift if theta is known).
% The mean time between false alarms is f(h,-theta/2) and 
% the mean time for detection is        f(h,+theta/2).
% To check these for a given h, use
%

NMC=500;
h=3;
th=1;
nu=th/2;
mu=th-nu;
[ta0,N0] = cusumarl(h,-mu);
[ta1,N0] = cusumarl(h,+mu);
disp([ta0, ta1])
%
% That is, the mean time between false alarms is 4 times larger.
% N0 denotes the mean time beween resets in the CUSUM algorithm.
% A reset is caused by either an alarm or a negative test statistic.
%
% The approximation suggested by Siegmund is obtained by

[ta0siegmund] = cusumarl(h,-mu,0.5,-1)
[ta1siegmund] = cusumarl(h,+mu,0.5,-1)

% and the approximation suggested by Wald is obtained by

[ta0wald] = cusumarl(h,-mu,0.5,-2)
[ta1wald] = cusumarl(h,+mu,0.5,-2)

% The mean times do not say anything about the distribution of the run length,
% which can be quite unsymmetric. Monte Carlo simulations can be used for further
% analysis of the run length function:
%
L0=cusumMC(h,-mu,NMC);
h1=cdfigure(1);
plotfix
hist(L0,30);
title('Distribution of false alarm times')
L1=cusumMC(h,mu,NMC);
h2=cdfigure(2,1);
plotfix
hist(L1,30);
title('Distribution of delay for detection')
% Compare theoretical with achieved mean times;

disp('Numerical    Wald      Siegmund       MC')
disp([ta0 ta0wald ta0siegmund mean(L0)])
disp([ta1 ta1wald ta1siegmund mean(L1)])

%close([h1 h2])


% The distribution for false alarms is basically binominal where 
% p=P(reset caused by alarm | reset)  since N0 is small.
% A rough estimate of the delay for detection is L0=h/(theta-nu);
% 
% What is really wanted is to determine h from a specified 
% mean time for detection L0. This is computed by a numerical search in
%
hmax=10;
L0=10;
h=cusumdesign(L0,mu,hmax)

% Evaluate the design by Monte Carlo simulations
L1=cusumMC(h,mu,NMC);
mean(L1)


% Finally, we can generate a table:

MU=[-0.5 0 0.5 1 1.5 2.0];
h=3;
for i=1:length(MU);
  [ta0,N0] = cusumarl(h,MU(i),0.1);
  [tas,N0] = cusumarl(h,MU(i),0.1,-1);
  [taw,N0] = cusumarl(h,MU(i),0.1,-2);
  L1=cusumMC(h,MU(i),NMC);
  tab(i,:)=[MU(i),ta0,taw,tas,mean(L1)];
end

format short e
disp(' ')
disp('       mu       Exact   Wald     Siegmund      MC')
%disp(tab(1:3,:));
%format short
disp(tab(:,:));
disp(' ')
disp('Table 5.1 in Basseville and Nikiforov.')

h3=cdfigure(3,2);
plotfix
handle=semilogy(MU,tab(:,2),'-',MU,tab(:,3),'--',MU,tab(:,4),':',MU,tab(:,5),'-.');
legend(handle,'Theoretical','Siegmund''s approx.','Wald''s approx.','Monte Carlo aprox.')
title('Mean delay for alarm: from simulations and theory')
xlabel('\theta - \nu/2')



figure(h1)
if saveplots==1, print -depsc /home/rt/fredrik/books/manual/figures/tutcusum1.eps, end
figure(h2)
if saveplots==1, print -depsc /home/rt/fredrik/books/manual/figures/tutcusum2.eps, end
figure(h3)
if saveplots==1, print -depsc /home/rt/fredrik/books/manual/figures/tutcusum3.eps, end

MU=[-1:0.2:2];
H=[1:0.5:5];
for i=1:length(MU);
  for j=1:length(H);
    tab2(j,i) = cusumarl(H(j),MU(i),0.1,-2);
  end
end
cdfigure(4,3);
plotfix
%mesh(MU(length(MU):-1:1),H(length(H):-1:1),log10(tab2))
mesh(MU(length(MU):-1:1),H,log10(tab2))
zlabel('log(ARL)')
xlabel('\theta-\nu/2')
ylabel('h')
if saveplots==1, print -depsc /home/rt/fredrik/books/manual/figures/tutcusum4.eps, end
