saveplots=0;
pause off
%         RECURSIVE ONE-MODEL DETECTION
%        ===============================
%
% There are a number of recursive implementations of the functions
% in DetectTB. These have the prefix 'r', e.g. radfilter.
% The syntax of these functions are almost the same as in the original,
% except that a state has to be included in the input and output arguments.
% The state is initialized by calling the function itself with no data.
% In this way, the user can use his own filters, e.g. EKF, and make
% appropriate actions after a detected change.
%
% In this demo, a first order AR model with one jump is simulated.

randn('seed',1)
N=100;
u=randn(N,1);
e=randn(N,1);
th0=[0.5 1.5;0.8 1.8]';
d=2;
lam=0.1;
nnn=[1 1 1];
y=simchange([lam*e u],nnn,N/2,th0);
pause

% Then, we initialize the recursive versions of filt and detect1

[Xfilt]=radfilter([NaN NaN],nnn);
dm=2;
h=5;
nu=lam;
sr=[1 h nu];
[Xone]=rdetect1([],[],dm,sr,-1);
pause

% Let's also initialize some parameters we want to save for plots

TH=[];
LAM=[];
GT1=[];
alarmtimes=[];
pause

% The following loop does what 
%
%     detect1([y u],nnn,dm,sr,-1)
%
% would do.
%
% Note that the task of residual generation and the detector is split into 
% separate functions in contrast to detect1.

pause

for t=1:N;
  [thhat,lamhat,epsi,S,Xfilt]=radfilter([y(t) u(t)],nnn,0.97,'RLS',Xfilt);
  [alarm,Xone]=rdetect1(epsi/sqrt(S),Xone);
  GT1=[GT1;Xone(1)]; 
  TH=[TH thhat];
  LAM=[LAM;lamhat];
  if alarm~=0; 
    alarmtimes=[alarmtimes t]; 
  end;
end;

% for comparison, we generate the smoothed parameter estimates from DETECTM,
% the test statistic off-line from DETECT1 and the estimates from ADFILTER.

[jhat,thseg]=detectM([y u],nnn);
[jdum,thdum,GT2]=detect1([y u],nnn,dm,sr,-1);
[TH2,LAM2]=adfilter([y u],nnn,0.97);
h1=cdfigure(1);
plot([thseg' TH' TH2'])
axis([0 N -1 3])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec1.eps, end
h2=cdfigure(2,1);
segplot([GT1],alarmtimes)
hold on, plot(GT2), hold off
title(['Test statistic and alarm times, Threshold = ',num2str(h)])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec2.eps, end

% From figure 1 we conclude that the same estimates are generated
% and from figure 2 that we get several alarms after the change.
pause

% The most appropriate thing to do after the alarm is to increase 
% the tracking ability of the filter. This is done below by increasing
% the covariance matrix of the parameters a factor 10.

[Xfilt]=radfilter([NaN NaN],nnn);
[Xone]=rdetect1([],[],dm,sr,-1);
TH=[];
LAM=[];
GT1=[];
alarmtimes=[];
pause

for t=1:N;
  [thhat,lamhat,epsi,S,Xfilt]=radfilter([y(t) u(t)],nnn,0.97,'RLS',Xfilt);
  [alarm,Xone]=rdetect1(epsi/sqrt(S),Xone);
  GT1=[GT1;Xone(1)]; 
  TH=[TH thhat];
  LAM=[LAM;lamhat];
  if alarm~=0; 
    alarmtimes=[alarmtimes t]; 
    Xfilt(:,5:4+d)=10*Xfilt(:,5:4+d);
  end;
end;
h3=cdfigure(3,2);
segplot([GT1],alarmtimes)
title(['Test statistic and alarm times, Threshold = ',num2str(h)])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec3.eps, end

% This time there is only one alarm.
pause
close(h1)
close(h2)
close(h3)
echo off


%         RECURSIVE TWO-MODEL DETECTION
%        ===============================
%
% There are a number of recursive implementations of the functions
% in DetectTB. These have the prefix 'r', e.g. radfilter.
% The syntax of these functions are almost the same as in the original,
% except that a state has to be included in the input and output arguments.
% The state is initialized by calling the function itself with no data.
% In this way, the user can use his own filters, e.g. EKF, and make
% appropriate actions after a detected change.
%
% In this demo, a first order AR model with one jump is simulated.

randn('seed',1)
N=100;
u=randn(N,1);
e=randn(N,1);
th0=[0.5 1.5;0.8 1.8]';
d=2;
lam=0.1;
nnn=[1 1 1];
y=simchange([lam*e u],nnn,N/2,th0);
pause

% Then, we initialize the recursive versions of filt, one using all data
% and one using a sliding window and detect2

L=10;
[X0]=radfilter([NaN NaN],nnn,1,'RLS');
[X1]=radfilter([NaN NaN],nnn,L,'WLS');
dm=2;
h=10;
nu=lam;
sr=[1 h nu];
[Xdet]=rdetect2([],[],[],[],[],dm,sr);
pause

% Let's also initialize some parameters we want to save for plots

TH0=[];
TH1=[];
LAM=[];
GT1=[];
alarmtimes=[];
pause

% The following loop does what 
%
%     detect2([y u],nnn,dm,sr,-1)
%
% would do.
%
% Note that the task of residual generation and the detector is split into 
% separate functions in contrast to detect2.

% Start recursion
pause
 
for t=1:N;
  [thhat0,lam0,epsi0,S0,X0]=radfilter([y(t) u(t)],nnn,1,'RLS',X0);
  [thhat1,lam1,epsi1,S1,X1]=radfilter([y(t) u(t)],nnn,L,'WLS',X1);
  [alarm,Xdet]=rdetect2(epsi0/sqrt(S0),epsi1/sqrt(S1),lam0,lam1,Xdet);
  GT1=[GT1;Xdet(1)]; 
  TH0=[TH0 thhat0];
  TH1=[TH1 thhat1];
  LAM=[LAM;lam0];
  if alarm~=0; 
    alarmtimes=[alarmtimes t]; 
  end;
end;

% Illustrate the result

h1=cdfigure(1);
plot([TH0' TH1'])
axis([0 N -1 3])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec4.eps, end
h2=cdfigure(2,1);
segplot([real(GT1)],alarmtimes)
title(['Test statistic and alarm times, Threshold = ',num2str(h)])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec5.eps, end
pause

% For comparison, we generate the smoothed parameter estimates from DETECTM,
% the test statistic off-line from DETECT2 and the estimates from FILT.

[jhat,thseg]=detectM([y u],nnn);
[jdum,thdum,GT2]=detect2([y u],nnn,dm,sr,-1);
[TH2,LAM2]=adfilter([y u],nnn,0.97);
figure(h1);
plot([thseg' TH0' TH1' TH2'])
axis([0 N -1 3])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec6.eps, end
figure(h2);
segplot([real(GT1)],alarmtimes)
hold on, plot(real(GT2)), hold off
title(['Test statistic and alarm times, Threshold = ',num2str(h)])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec7.eps, end

% From figure 1 we conclude that the same estimates are generated
% and from figure 2 that we get several alarms after the change.
pause

% The most appropriate thing to do after the alarm is to increase 
% the tracking ability of the filter. This is done below by increasing
% the covariance matrix of the parameters a factor 10.

[X0]=radfilter([NaN NaN],nnn,1,'RLS');
[X1]=radfilter([NaN NaN],nnn,L,'WLS');
[Xdet]=rdetect2([],[],[],[],[],dm,sr);
TH=[];
LAM=[];
GT1=[];
alarmtimes=[];

% Start recursion
pause

for t=1:N;
  [thhat0,lam0,epsi0,S0,X0]=radfilter([y(t) u(t)],nnn,1,'RLS',X0);
  [thhat1,lam1,epsi1,S1,X1]=radfilter([y(t) u(t)],nnn,L,'WLS',X1);
  [alarm,Xdet]=rdetect2(epsi0/sqrt(S0),epsi1/sqrt(S1),lam0,lam1,Xdet);
  GT1=[GT1;Xdet(1)]; 
  TH=[TH thhat0];
  LAM=[LAM;lam0];
  if alarm~=0; 
    alarmtimes=[alarmtimes t]; 
    X0(:,6:5+d)=10*X0(:,6:5+d);
  end;
end;
h3=cdfigure(3,2);
segplot([real(GT1)],alarmtimes)
title(['Test statistic and alarm times, Threshold = ',num2str(h)])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec8.eps, end

% This time there is only one alarm.
pause
close(h1)
close(h2)
close(h3)
echo off


%         RECURSIVE GLR DETECTION
%        =========================
%
% First, a second order state space model with one jump is simulated.

randn('seed',0)
N=100;
d=2;
%w=randn(N,2);
u=randn(N,1);
%e=randn(N,1);
A=[1 1;0 1]; B=[0.5;1]; C=[1 0]; D=0; R1=.01*eye(2); R2=0.1; P0=1*eye(2);
nnn=ss2nnn(A,B,C,D,R1,R2,P0);

th0=[1;...
     2];

%y=simchange([sqrt(R2)*e u w*sqrt(R1)],nnn,N/2,th0);
y=simchange([u],nnn,N/2,th0);

h1=cdfigure(1);
plot(y)
title('Signal')
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec9.eps, end
pause

% First, let's examine the off-line versions of GLR and also MLR

M=-10;  % size of sliding window, batch test here
flops(0)
[jhatglr,taglr,lrglr,Xhat,nuhatglr,mu]=glr([y u],nnn,M);
flopsglr=flops;
[jhatmlr,lrmlr,flopsmlr]=mlr([y u],nnn,1);
jhatglr,jhatmlr
nuhatglr

figure(h1)
plot([y, (C*Xhat)'])
hold on
plot(N,C*(Xhat(:,N)+mu*nuhatglr(:,1)),'o')
hold off
title('Signal and Kalman filtered estimate')
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec10.eps, end

h2=cdfigure(2,1);
plot([lrglr lrmlr])
title('Decision function for GLR and MLR')
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec11.eps, end
  
% That is, a threshold of 70 seems appropriate
% The number of flops compares as follows
disp('GLR   MLR')
disp([flopsglr flopsmlr])
pause


% Then, we initialize the recursive versions of filt and glr

[Xfilt]=radfilter([NaN NaN],nnn);
h=72;
M=10;
[Xglr]=rglr([],[],[],nnn,[],M,h);
pause

% Let's also initialize some parameters we want to save for plots

TH=[];
EPSI=[];
LAM=[];
GT1=[];
alarmtimes=[];
pause

% The following loop uses the recursive implementation of GLR
%
% Note that the task of residual generation and the detector is split into 
% separate functions.
%
% Start recursion
pause

for t=1:N;
  [thhat,lamhat,epsi,S,Xfilt,K]=radfilter([y(t) u(t)],nnn,1,'RLS',Xfilt);
  f0=flops;
  [alarm,Xglr,nuhat,jhat,lrmax,P]=rglr(epsi,S,K,nnn,Xglr);
  f1=flops;
  GT1=[GT1;lrmax]; 
  EPSI=[EPSI;epsi];
  TH=[TH nuhat];
  if alarm~=0; 
    nuhat
    P
    alarmtimes=[alarmtimes t]; 
    Xfilt(:,6:5+d)=100*Xfilt(:,6:5+d);   % Increase P in the Kalman filter
  end;
end;
% The number of flops per iteration is
disp([f1-f0])

h3=cdfigure(3,2);
segplot([GT1],alarmtimes)
title(['Test statistic and alarm times, Threshold = ',num2str(h)])
if saveplots==1, print -depsc /home/rt/fredrik/detect/figures/tutrec12.eps, end
pause

close(h1)
close(h2)
close(h3)
echo off
