function [uhat,thhat]=viterbi(y,th,alphabet,r2,utrain)
%
% VITERBI implements the Viterbi algorithm for equalization
%
% [uhat]=viterbi(y,th,alphabet,r2)
% [uhat,thhat]=viterbi(y,th,alphabet,r2,utrain)
%   
% Assumes that an estimate of the FIR channel is available, or that
% there is a training sequence available.
% Works for arbitrary input alphabets.
%
% y        Output from channel
% utrain   If provided, utrain is the training sequence
%          and th the FIR order, and thhat is the FIR estimate
% uhat     Estimated input
% th       Estimated FIR parameters in a column vector, or,
%          if utrain is provided, the FIR order.
% alphabet Finite input alphabet
% r2       Noise variance
%
% Limitations: 
%  1. Only FIR equalizers
%  2. Only scalar channels
%
% References: 
%  G. D. Forney, The Viterbi Algorithm, Proceedings of the IEEE,
%  1973, 268-278
% 
% See also lineq implementing a linear equalizer with a similar syntax
% Fredrik Gustafsson, 980206



[N,m]=size(y);
[n]=size(th,2);
alphabet=alphabet(:)';
th=th(:);
if nargin<4;
  r2=0.0001;
end

if nargin>4;  % training sequence
  n=th;
  Ntrain=length(utrain);
  th=adfilter([y(1:Ntrain) utrain],[0 n 0]);   % requires adfilter
%th=rarx([[0;y(1:Ntrain)] [utrain;0]],[0 n 1],'ff',1); % requires SITB
%th=th';th=th(:,2:Ntrain+1);
  th=th(:,Ntrain);
  y(1:Ntrain)=[];
  N=N-Ntrain;
end
M=1;
utest=[];
nsymb=length(alphabet);

for j=1:n;
  utest=kron(ones(1,nsymb),utest);
  utest(j,:)=kron(alphabet,ones(1,M));
  M=nsymb*M;
end
uref=utest;
alfa=[zeros(1,M)];


for t=n:N;
%-------Update the filters-----------------------------
    for j=1:M,
        phi=utest(t:-1:t-n+1,j);
        epsi(j)=y(t)-phi.'*th;
        alfa(j)=alfa(j) - epsi(j)'*epsi(j)/(2*r2);
    end;
epsi;
%-------Split the input sequence-----------------------
for j=1:M/nsymb;
[dum maxind]=max(alfa(nsymb*j-nsymb+1:nsymb*j));
saveind(j)=nsymb*j-nsymb+maxind;
end;
alfa=alfa(saveind);
alfa=kron(ones(1,nsymb),alfa);
utest=utest(:,saveind);
utest=kron(ones(1,nsymb),utest);
utest(t+1,:)=kron(alphabet,ones(1,M/nsymb));

alfa=alfa-max(alfa);

end


%---------Compute signal estimate------------------------------

[dum,maxind]=max(alfa);
uhat=utest(1:N,maxind);
if nargin>4   % uhat for training sequence is set to "0"
  uhat=[zeros(Ntrain,1);uhat];
end
thhat=th;